# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''Classes to handle Unix style, MMDF style, and MH style mailboxes.'''
import rfc822
import os
__all__ = [
    'UnixMailbox',
    'MmdfMailbox',
    'MHMailbox',
    'Maildir',
    'BabylMailbox',
    'PortableUnixMailbox']

class _Mailbox:
    
    def __init__(self, fp, factory = rfc822.Message):
        self.fp = fp
        self.seekp = 0
        self.factory = factory

    
    def __iter__(self):
        return iter(self.next, None)

    
    def next(self):
        while None:
            
            try:
                self._search_start()
            except EOFError:
                self.seekp = self.fp.tell()
                return None

            start = self.fp.tell()
            self._search_end()
            self.seekp = stop = self.fp.tell()
            if start != stop:
                break
                continue
        return self.factory(_Subfile(self.fp, start, stop))



class _Subfile:
    
    def __init__(self, fp, start, stop):
        self.fp = fp
        self.start = start
        self.stop = stop
        self.pos = self.start

    
    def _read(self, length, read_function):
        if self.pos >= self.stop:
            return ''
        
        remaining = self.stop - self.pos
        if length is None and length < 0 or length > remaining:
            length = remaining
        
        self.fp.seek(self.pos)
        data = read_function(length)
        self.pos = self.fp.tell()
        return data

    
    def read(self, length = None):
        return self._read(length, self.fp.read)

    
    def readline(self, length = None):
        return self._read(length, self.fp.readline)

    
    def readlines(self, sizehint = -1):
        lines = []
        while None:
            line = self.readline()
            if not line:
                break
            
            if sizehint >= 0:
                sizehint = sizehint - len(line)
                if sizehint <= 0:
                    break
                
            sizehint <= 0
        return lines

    
    def tell(self):
        return self.pos - self.start

    
    def seek(self, pos, whence = 0):
        if whence == 0:
            self.pos = self.start + pos
        elif whence == 1:
            self.pos = self.pos + pos
        elif whence == 2:
            self.pos = self.stop + pos
        

    
    def close(self):
        del self.fp



class UnixMailbox(_Mailbox):
    
    def _search_start(self):
        while None:
            pos = self.fp.tell()
            line = self.fp.readline()
            if not line:
                raise EOFError
            
            if line[:5] == 'From ' and self._isrealfromline(line):
                self.fp.seek(pos)
                return None
                continue

    
    def _search_end(self):
        self.fp.readline()
        while None:
            pos = self.fp.tell()
            line = self.fp.readline()
            if not line:
                return None
            
            if line[:5] == 'From ' and self._isrealfromline(line):
                self.fp.seek(pos)
                return None
                continue

    _fromlinepattern = 'From \\s*[^\\s]+\\s+\\w\\w\\w\\s+\\w\\w\\w\\s+\\d?\\d\\s+\\d?\\d:\\d\\d(:\\d\\d)?(\\s+[^\\s]+)?\\s+\\d\\d\\d\\d\\s*$'
    _regexp = None
    
    def _strict_isrealfromline(self, line):
        if not self._regexp:
            import re
            self._regexp = re.compile(self._fromlinepattern)
        
        return self._regexp.match(line)

    
    def _portable_isrealfromline(self, line):
        return True

    _isrealfromline = _strict_isrealfromline


class PortableUnixMailbox(UnixMailbox):
    _isrealfromline = UnixMailbox._portable_isrealfromline


class MmdfMailbox(_Mailbox):
    
    def _search_start(self):
        while None:
            line = self.fp.readline()
            if not line:
                raise EOFError
            
            if line[:5] == '\x01\x01\x01\x01\n':
                return None
                continue

    
    def _search_end(self):
        while None:
            pos = self.fp.tell()
            line = self.fp.readline()
            if not line:
                return None
            
            if line == '\x01\x01\x01\x01\n':
                self.fp.seek(pos)
                return None
                continue



class MHMailbox:
    
    def __init__(self, dirname, factory = rfc822.Message):
        import re
        pat = re.compile('^[1-9][0-9]*$')
        self.dirname = dirname
        list = os.listdir(self.dirname)
        list = filter(pat.match, list)
        list = map(long, list)
        list.sort()
        self.boxes = map(str, list)
        self.boxes.reverse()
        self.factory = factory

    
    def __iter__(self):
        return iter(self.next, None)

    
    def next(self):
        if not self.boxes:
            return None
        
        fn = self.boxes.pop()
        fp = open(os.path.join(self.dirname, fn))
        msg = self.factory(fp)
        
        try:
            msg._mh_msgno = fn
        except (AttributeError, TypeError):
            pass

        return msg



class Maildir:
    
    def __init__(self, dirname, factory = rfc822.Message):
        self.dirname = dirname
        self.factory = factory
        newdir = os.path.join(self.dirname, 'new')
        boxes = _[1]
        curdir = os.path.join(self.dirname, 'cur')
        [] += _[1]
        boxes.reverse()
        self.boxes = boxes

    
    def __iter__(self):
        return iter(self.next, None)

    
    def next(self):
        if not self.boxes:
            return None
        
        fn = self.boxes.pop()
        fp = open(fn)
        return self.factory(fp)



class BabylMailbox(_Mailbox):
    
    def _search_start(self):
        while None:
            line = self.fp.readline()
            if not line:
                raise EOFError
            
            if line == '*** EOOH ***\n':
                return None
                continue

    
    def _search_end(self):
        while None:
            pos = self.fp.tell()
            line = self.fp.readline()
            if not line:
                return None
            
            if line == '\x1f\x0c\n' or line == '\x1f':
                self.fp.seek(pos)
                return None
                continue



def _test():
    import sys
    args = sys.argv[1:]
    if not args:
        for key in ('MAILDIR', 'MAIL', 'LOGNAME', 'USER'):
            if key in os.environ:
                mbox = os.environ[key]
                break
                continue
        else:
            print '$MAIL, $LOGNAME nor $USER set -- who are you?'
            return None
    else:
        mbox = args[0]
    if mbox[:1] == '+':
        mbox = os.environ['HOME'] + '/Mail/' + mbox[1:]
    elif '/' not in mbox:
        if os.path.isfile('/var/mail/' + mbox):
            mbox = '/var/mail/' + mbox
        else:
            mbox = '/usr/mail/' + mbox
    
    if os.path.isdir(mbox):
        if os.path.isdir(os.path.join(mbox, 'cur')):
            mb = Maildir(mbox)
        else:
            mb = MHMailbox(mbox)
    else:
        fp = open(mbox, 'r')
        mb = PortableUnixMailbox(fp)
    msgs = []
    while None:
        msg = mb.next()
        if msg is None:
            break
        
        if len(args) <= 1:
            msg.fp = None
            continue
    if len(args) > 1:
        num = int(args[1])
        print 'Message %d body:' % num
        msg = msgs[num - 1]
        msg.rewindbody()
        sys.stdout.write(msg.fp.read())
    else:
        print 'Mailbox', mbox, 'has', len(msgs), 'messages:'
        for msg in msgs:
            if not msg.getheader('from'):
                pass
            f = ''
            if not msg.getheader('subject'):
                pass
            s = ''
            if not msg.getheader('date'):
                pass
            d = ''
            print '-%20.20s   %20.20s   %-30.30s' % (f, d[5:], s)
        

if __name__ == '__main__':
    _test()

