# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

import os
import apt_pkg
import apt
import tarfile
import urllib2
import tempfile
import shutil
import GnuPGInterface
from gettext import gettext as _

class DistUpgradeFetcherCore(object):
    ''' base class (without GUI) for the upgrade fetcher '''
    
    def __init__(self, new_dist, progress):
        self.new_dist = new_dist
        self._progress = progress
        self.run_options = []

    
    def showReleaseNotes(self):
        return True

    
    def error(self, summary, message):
        ''' dummy implementation for error display, should be overwriten
            by subclasses that want to more fancy method
        '''
        print summary
        print message
        return False

    
    def authenticate(self):
        if self.new_dist.upgradeToolSig:
            f = self.tmpdir + '/' + os.path.basename(self.new_dist.upgradeTool)
            sig = self.tmpdir + '/' + os.path.basename(self.new_dist.upgradeToolSig)
            print "authenticate '%s' against '%s' " % (f, sig)
            if not self.gpgauthenticate(f, sig):
                return False
            
        
        return True

    
    def gpgauthenticate(self, file, signature, keyring = '/etc/apt/trusted.gpg'):
        ''' authenticated a file against a given signature, if no keyring
            is given use the apt default keyring
        '''
        gpg = GnuPGInterface.GnuPG()
        gpg.options.extra_args = [
            '--no-options',
            '--homedir',
            self.tmpdir,
            '--no-default-keyring',
            '--keyring',
            keyring]
        proc = gpg.run([
            '--verify',
            signature,
            file], create_fhs = [
            'status',
            'logger',
            'stderr'])
        gpgres = proc.handles['status'].read()
        
        try:
            proc.wait()
        except IOError:
            e = None
            print 'exception from gpg: %s' % e
            print 'Debug information: '
            print proc.handles['status'].read()
            print proc.handles['stderr'].read()
            print proc.handles['logger'].read()
            return False

        if 'VALIDSIG' in gpgres:
            return True
        
        print 'invalid result from gpg:'
        print gpgres
        return False

    
    def extractDistUpgrader(self):
        print "extracting '%s'" % (self.tmpdir + '/' + os.path.basename(self.uri))
        tar = tarfile.open(self.tmpdir + '/' + os.path.basename(self.uri), 'r')
        for tarinfo in tar:
            tar.extract(tarinfo)
        
        tar.close()
        return True

    
    def verifyDistUprader(self):
        self.script = script = '%s/%s' % (self.tmpdir, self.new_dist.name)
        if not os.path.exists(script):
            return error(_('Could not run the upgrade tool'), _('This is most likely a bug in the upgrade tool. Please report it as a bug'))
        
        return True

    
    def fetchDistUpgrader(self):
        self.tmpdir = tmpdir = tempfile.mkdtemp()
        os.chdir(tmpdir)
        fetcher = apt_pkg.GetAcquire(self._progress)
        if self.new_dist.upgradeToolSig != None:
            uri = self.new_dist.upgradeToolSig
            af = apt_pkg.GetPkgAcqFile(fetcher, uri, descr = _('Upgrade tool signature'))
        
        if self.new_dist.upgradeTool != None:
            self.uri = self.new_dist.upgradeTool
            af = apt_pkg.GetPkgAcqFile(fetcher, self.uri, descr = _('Upgrade tool'))
            if fetcher.Run() != fetcher.ResultContinue:
                return False
            
            return True
        
        return False

    
    def runDistUpgrader(self):
        if os.getuid() != 0:
            os.execv('/usr/bin/sudo', [
                'sudo',
                self.script])
        else:
            os.execv(self.script, [
                self.script] + self.run_options)

    
    def cleanup(self):
        os.chdir('..')
        shutil.rmtree(self.tmpdir)

    
    def run(self):
        if not self.showReleaseNotes():
            return None
        
        if not self.fetchDistUpgrader():
            self.error(_('Failed to fetch'), _('Fetching the upgrade failed. There may be a network problem. '))
            return None
        
        if not self.extractDistUpgrader():
            self.error(_('Failed to extract'), _('Extracting the upgrade failed. There may be a problem with the network or with the server. '))
            return None
        
        if not self.verifyDistUprader():
            self.error(_('Verfication failed'), _('Verifying the upgrade failed.  There may be a problem with the network or with the server. '))
            self.cleanup()
            return None
        
        if not self.authenticate():
            self.error(_('Authentication failed'), _('Authenticating the upgrade failed. There may be a problem with the network or with the server. '))
            self.cleanup()
            return None
        
        self.runDistUpgrader()


if __name__ == '__main__':
    self.error('summary', 'message')
    d = DistUpgradeFetcher(None, None)
    print d.authenticate('/tmp/Release', '/tmp/Release.gpg')

