# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''
The printing sub-system.
WWW: http://4suite.com/4DOM         e-mail: support@4suite.com

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
'''
import string
import re
from xml.dom import Node
from xml.dom.ext.Visitor import Visitor, WalkerInterface
from xml.dom import ext, XMLNS_NAMESPACE, XML_NAMESPACE, XHTML_NAMESPACE
from xml.dom.html import TranslateHtmlCdata
from xml.dom.html import HTML_4_TRANSITIONAL_INLINE
from xml.dom.html import HTML_FORBIDDEN_END
from xml.dom.html import HTML_BOOLEAN_ATTRS
ILLEGAL_LOW_CHARS = '[\x01-\x08\x0b-\x0c\x0e-\x1f]'
SURROGATE_BLOCK = '[\xf0-\xf7][\x80-\xbf][\x80-\xbf][\x80-\xbf]'
ILLEGAL_HIGH_CHARS = '\xef\xbf[\xbe\xbf]'
XML_ILLEGAL_CHAR_PATTERN = re.compile('%s|%s' % (ILLEGAL_LOW_CHARS, ILLEGAL_HIGH_CHARS))
g_utf8TwoBytePattern = re.compile('([\xc0-\xc3])([\x80-\xbf])')
g_cdataCharPattern = re.compile('[&<]|]]>')
g_charToEntity = {
    '&': '&amp;',
    '<': '&lt;',
    ']]>': ']]&gt;' }

try:
    import codecs
    from types import UnicodeType
    
    def utf8_to_code(text, encoding):
        encoder = codecs.lookup(encoding)[0]
        if type(text) is not UnicodeType:
            text = unicode(text, 'utf-8')
        
        return encoder(text)[0]

    
    def strobj_to_utf8str(text, encoding):
        if string.upper(encoding) not in [
            'UTF-8',
            'ISO-8859-1',
            'LATIN-1']:
            raise ValueError('Invalid encoding: %s' % encoding)
        
        encoder = codecs.lookup(encoding)[0]
        if type(text) is not UnicodeType:
            text = unicode(text, 'utf-8')
        
        return str(encoder(text)[0])

except ImportError:
    
    def utf8_to_code(text, encoding):
        encoding = string.upper(encoding)
        if encoding == 'UTF-8':
            return text
        
        wstring = wstring
        import xml.unicode.iso8859
        wstring.install_alias('ISO-8859-1', 'ISO_8859-1:1987')
        ws = wstring.from_utf8(text)
        text = ws.encode(encoding)
        return text

    strobj_to_utf8str = utf8_to_code


def TranslateCdataAttr(characters):
    '''Handles normalization and some intelligence about quoting'''
    if not characters:
        return ('', "'")
    
    if "'" in characters:
        delimiter = '"'
        new_chars = re.sub('"', '&quot;', characters)
    else:
        delimiter = "'"
        new_chars = re.sub("'", '&apos;', characters)
    if '\n' in characters:
        new_chars = re.sub('\n', '&#10;', new_chars)
    
    return (new_chars, delimiter)


def TranslateCdata(characters, encoding = 'UTF-8', prev_chars = '', markupSafe = 0, charsetHandler = utf8_to_code):
    '''
    charsetHandler is a function that takes a string or unicode object as the
    first argument, representing the string to be procesed, and an encoding
    specifier as the second argument.  It must return a string or unicode
    object
    '''
    if not characters:
        return ''
    
    if not markupSafe:
        if g_cdataCharPattern.search(characters):
            new_string = g_cdataCharPattern.subn((lambda m, d = g_charToEntity: d[m.group()]), characters)[0]
        else:
            new_string = characters
        if prev_chars[-2:] == ']]' and characters[0] == '>':
            new_string = '&gt;' + new_string[1:]
        
    else:
        new_string = characters
    if XML_ILLEGAL_CHAR_PATTERN.search(new_string):
        new_string = XML_ILLEGAL_CHAR_PATTERN.subn((lambda m: '&#%i;' % ord(m.group())), new_string)[0]
    
    new_string = charsetHandler(new_string, encoding)
    return new_string


class PrintVisitor(Visitor):
    
    def __init__(self, stream, encoding, indent = '', plainElements = None, nsHints = None, isXhtml = 0, force8bit = 0):
        self.stream = stream
        self.encoding = encoding
        self._namespaces = [
            { }]
        if not nsHints:
            pass
        self._nsHints = { }
        self._indent = indent
        self._depth = 0
        self._inText = 0
        if not plainElements:
            pass
        self._plainElements = []
        self._html = None
        self._isXhtml = isXhtml
        self.force8bit = force8bit

    
    def _write(self, text):
        if self.force8bit:
            obj = strobj_to_utf8str(text, self.encoding)
        else:
            obj = utf8_to_code(text, self.encoding)
        self.stream.write(obj)

    
    def _tryIndent(self):
        if not (self._inText) and self._indent:
            self._write('\n' + self._indent * self._depth)
        

    
    def visit(self, node):
        if self._html is None:
            self._html = hasattr(node.ownerDocument, 'getElementsByName')
        
        nodeType = node.nodeType
        if node.nodeType == Node.ELEMENT_NODE:
            return self.visitElement(node)
        elif node.nodeType == Node.ATTRIBUTE_NODE:
            return self.visitAttr(node)
        elif node.nodeType == Node.TEXT_NODE:
            return self.visitText(node)
        elif node.nodeType == Node.CDATA_SECTION_NODE:
            return self.visitCDATASection(node)
        elif node.nodeType == Node.ENTITY_REFERENCE_NODE:
            return self.visitEntityReference(node)
        elif node.nodeType == Node.ENTITY_NODE:
            return self.visitEntity(node)
        elif node.nodeType == Node.PROCESSING_INSTRUCTION_NODE:
            return self.visitProcessingInstruction(node)
        elif node.nodeType == Node.COMMENT_NODE:
            return self.visitComment(node)
        elif node.nodeType == Node.DOCUMENT_NODE:
            return self.visitDocument(node)
        elif node.nodeType == Node.DOCUMENT_TYPE_NODE:
            return self.visitDocumentType(node)
        elif node.nodeType == Node.DOCUMENT_FRAGMENT_NODE:
            return self.visitDocumentFragment(node)
        elif node.nodeType == Node.NOTATION_NODE:
            return self.visitNotation(node)
        
        raise Exception('Unknown node type: %s' % repr(node))

    
    def visitNodeList(self, node, exclude = None):
        for curr in node:
            if curr is not exclude:
                pass
            self.visit(curr)
        

    
    def visitNamedNodeMap(self, node):
        for item in node.values():
            self.visit(item)
        

    
    def visitAttr(self, node):
        if node.namespaceURI == XMLNS_NAMESPACE:
            return None
        
        self._write(' ' + node.name)
        value = node.value
        if value or not (self._html):
            text = TranslateCdata(value, self.encoding)
            (text, delimiter) = TranslateCdataAttr(text)
            self.stream.write('=%s%s%s' % (delimiter, text, delimiter))
        

    
    def visitProlog(self):
        if not self.encoding:
            pass
        self._write("<?xml version='1.0' encoding='%s'?>" % 'utf-8')
        self._inText = 0

    
    def visitDocument(self, node):
        if not self._html:
            pass
        self.visitProlog()
        if node.doctype:
            pass
        self.visitDocumentType(node.doctype)
        self.visitNodeList(node.childNodes, exclude = node.doctype)

    
    def visitDocumentFragment(self, node):
        self.visitNodeList(node.childNodes)

    
    def visitElement(self, node):
        self._namespaces.append(self._namespaces[-1].copy())
        inline = node.tagName in self._plainElements
        if not inline:
            pass
        self._tryIndent()
        self._write('<%s' % node.tagName)
        if self._isXhtml or not (self._html):
            namespaces = ''
            if self._isXhtml:
                nss = {
                    'xml': XML_NAMESPACE,
                    None: XHTML_NAMESPACE }
            else:
                nss = ext.GetAllNs(node)
            if self._nsHints:
                self._nsHints.update(nss)
                nss = self._nsHints
                self._nsHints = { }
            
            del nss['xml']
            for prefix in nss.keys():
                if not self._namespaces[-1].has_key(prefix) or self._namespaces[-1][prefix] != nss[prefix]:
                    (nsuri, delimiter) = TranslateCdataAttr(nss[prefix])
                    if prefix:
                        xmlns = ' xmlns:%s=%s%s%s' % (prefix, delimiter, nsuri, delimiter)
                    else:
                        xmlns = ' xmlns=%s%s%s' % (delimiter, nsuri, delimiter)
                    namespaces = namespaces + xmlns
                
                self._namespaces[-1][prefix] = nss[prefix]
            
            self._write(namespaces)
        
        for attr in node.attributes.values():
            self.visitAttr(attr)
        
        if len(node.childNodes):
            self._write('>')
            self._depth = self._depth + 1
            self.visitNodeList(node.childNodes)
            self._depth = self._depth - 1
            if not (self._html) or node.tagName not in HTML_FORBIDDEN_END:
                if not self._inText and inline:
                    pass
                self._tryIndent()
                self._write('</%s>' % node.tagName)
            
        elif not self._html:
            self._write('/>')
        elif node.tagName not in HTML_FORBIDDEN_END:
            self._write('></%s>' % node.tagName)
        else:
            self._write('>')
        del self._namespaces[-1]
        self._inText = 0

    
    def visitText(self, node):
        text = node.data
        if self._indent:
            if string.strip(text):
                pass
            text = text
        
        if text:
            if self._html:
                text = TranslateHtmlCdata(text, self.encoding)
            else:
                text = TranslateCdata(text, self.encoding)
            self.stream.write(text)
            self._inText = 1
        

    
    def visitDocumentType(self, doctype):
        if not (doctype.systemId) and not (doctype.publicId):
            return None
        
        self._tryIndent()
        self._write('<!DOCTYPE %s' % doctype.name)
        if doctype.systemId and '"' in doctype.systemId:
            system = "'%s'" % doctype.systemId
        else:
            system = '"%s"' % doctype.systemId
        if doctype.publicId and '"' in doctype.publicId:
            public = "'%s'" % doctype.publicId
        else:
            public = '"%s"' % doctype.publicId
        if doctype.publicId and doctype.systemId:
            self._write(' PUBLIC %s %s' % (public, system))
        elif doctype.systemId:
            self._write(' SYSTEM %s' % system)
        
        if doctype.entities or doctype.notations:
            self._write(' [')
            self._depth = self._depth + 1
            self.visitNamedNodeMap(doctype.entities)
            self.visitNamedNodeMap(doctype.notations)
            self._depth = self._depth - 1
            self._tryIndent()
            self._write(']>')
        else:
            self._write('>')
        self._inText = 0

    
    def visitEntity(self, node):
        '''Visited from a NamedNodeMap in DocumentType'''
        self._tryIndent()
        self._write('<!ENTITY %s' % node.nodeName)
        if node.publicId:
            pass
        self._write(' PUBLIC %s' % node.publicId)
        if node.systemId:
            pass
        self._write(' SYSTEM %s' % node.systemId)
        if node.notationName:
            pass
        self._write(' NDATA %s' % node.notationName)
        self._write('>')

    
    def visitNotation(self, node):
        '''Visited from a NamedNodeMap in DocumentType'''
        self._tryIndent()
        self._write('<!NOTATION %s' % node.nodeName)
        if node.publicId:
            pass
        self._write(' PUBLIC %s' % node.publicId)
        if node.systemId:
            pass
        self._write(' SYSTEM %s' % node.systemId)
        self._write('>')

    
    def visitCDATASection(self, node):
        self._tryIndent()
        self._write('<![CDATA[%s]]>' % node.data)
        self._inText = 0

    
    def visitComment(self, node):
        self._tryIndent()
        self._write('<!--%s-->' % node.data)
        self._inText = 0

    
    def visitEntityReference(self, node):
        self._write('&%s;' % node.nodeName)
        self._inText = 1

    
    def visitProcessingInstruction(self, node):
        self._tryIndent()
        self._write('<?%s %s?>' % (node.target, node.data))
        self._inText = 0



class PrintWalker(WalkerInterface):
    
    def __init__(self, visitor, startNode):
        WalkerInterface.__init__(self, visitor)
        self.start_node = startNode

    
    def step(self):
        '''There is really no step to printing.  It prints the whole thing'''
        self.visitor.visit(self.start_node)

    
    def run(self):
        return self.step()


