# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

import string
import gettext
import re
import apt_pkg
import glob
import shutil
import time
import os.path as os
import sys
from distinfo import DistInfo

def is_mirror(master_uri, compare_uri):
    '''check if the given add_url is idential or a mirror of orig_uri
    e.g. master_uri = archive.ubuntu.com
      compare_uri = de.archive.ubuntu.com
      -> True
  '''
    compare_uri = compare_uri.rstrip('/ ')
    master_uri = master_uri.rstrip('/ ')
    if compare_uri == master_uri:
        return True
    
    
    try:
        compare_srv = compare_uri.split('//')[1]
        master_srv = master_uri.split('//')[1]
    except IndexError:
        return False

    if '.' in compare_srv and compare_srv[compare_srv.index('.') + 1:] == master_srv:
        return True
    
    return False


def uniq(s):
    ''' simple and efficient way to return uniq list '''
    return list(set(s))


class SourceEntry:
    ''' single sources.list entry '''
    
    def __init__(self, line, file = None):
        self.invalid = False
        self.disabled = False
        self.type = ''
        self.uri = ''
        self.dist = ''
        self.comps = []
        self.comment = ''
        self.line = line
        if file == None:
            file = apt_pkg.Config.FindDir('Dir::Etc') + apt_pkg.Config.Find('Dir::Etc::sourcelist')
        
        self.file = file
        self.parse(line)
        self.template = None
        self.children = []

    
    def __eq__(self, other):
        ''' equal operator for two sources.list entries '''
        if self.disabled == other.disabled and self.type == other.type and self.uri == other.uri and self.dist == other.dist:
            pass
        return self.comps == other.comps

    
    def mysplit(self, line):
        ''' a split() implementation that understands the sources.list
        format better and takes [] into account (for e.g. cdroms) '''
        line = string.strip(line)
        pieces = []
        tmp = ''
        p_found = False
        space_found = False
        for i in range(len(line)):
            if line[i] == '[':
                p_found = True
                tmp += line[i]
                continue
            if line[i] == ']':
                p_found = False
                tmp += line[i]
                continue
            if space_found and not line[i].isspace():
                space_found = False
                pieces.append(tmp)
                tmp = line[i]
                continue
            if line[i].isspace() and not p_found:
                space_found = True
                continue
            tmp += line[i]
        
        if len(tmp) > 0:
            pieces.append(tmp)
        
        return pieces

    
    def parse(self, line):
        ''' parse a given sources.list (textual) line and break it up
        into the field we have '''
        line = string.strip(self.line)
        if line == '' or line == '#':
            self.invalid = True
            return None
        
        if line[0] == '#':
            self.disabled = True
            pieces = string.split(line[1:])
            if pieces[0] not in ('rpm', 'rpm-src', 'deb', 'deb-src'):
                self.invalid = True
                return None
            else:
                line = line[1:]
        
        i = line.find('#')
        if i > 0:
            self.comment = line[i + 1:]
            line = line[:i]
        
        pieces = self.mysplit(line)
        if len(pieces) < 3:
            self.invalid = True
            return None
        
        self.type = string.strip(pieces[0])
        if self.type not in ('deb', 'deb-src', 'rpm', 'rpm-src'):
            self.invalid = True
            return None
        
        self.uri = string.strip(pieces[1])
        if len(self.uri) < 1:
            self.invalid = True
        
        self.dist = string.strip(pieces[2])
        if len(pieces) > 3:
            self.comps = pieces[3:]
        else:
            self.comps = []

    
    def set_enabled(self, new_value):
        ''' set a line to enabled or disabled '''
        self.disabled = not new_value
        if new_value == True:
            i = 0
            self.line = string.lstrip(self.line)
            while self.line[i] == '#':
                i += 1
            self.line = self.line[i:]
        elif string.strip(self.line)[0] != '#':
            self.line = '#' + self.line
        

    
    def __str__(self):
        ''' debug helper '''
        return self.str().strip()

    
    def str(self):
        ''' return the current line as string '''
        if self.invalid:
            return self.line
        
        line = ''
        if self.disabled:
            line = '# '
        
        line += '%s %s %s' % (self.type, self.uri, self.dist)
        if len(self.comps) > 0:
            line += ' ' + ' '.join(self.comps)
        
        if self.comment != '':
            line += ' #' + self.comment
        
        line += '\n'
        return line



class NullMatcher(object):
    ''' a Matcher that does nothing '''
    
    def match(self, s):
        return True



class SourcesList:
    ''' represents the full sources.list + sources.list.d file '''
    
    def __init__(self, withMatcher = True, matcherPath = '/usr/share/python-apt/templates/'):
        self.list = []
        if withMatcher:
            self.matcher = SourceEntryMatcher(matcherPath)
        else:
            self.matcher = NullMatcher()
        self.refresh()

    
    def refresh(self):
        ''' update the list of known entries '''
        self.list = []
        dir = apt_pkg.Config.FindDir('Dir::Etc')
        file = apt_pkg.Config.Find('Dir::Etc::sourcelist')
        self.load(dir + file)
        partsdir = apt_pkg.Config.FindDir('Dir::Etc::sourceparts')
        for file in glob.glob('%s/*.list' % partsdir):
            self.load(file)
        
        for source in self.list:
            if source.invalid == False:
                self.matcher.match(source)
                continue
        

    
    def __iter__(self):
        ''' simple iterator to go over self.list, returns SourceEntry
        types '''
        for entry in self.list:
            yield entry
        
        raise StopIteration

    
    def add(self, type, uri, dist, orig_comps, comment = '', pos = -1, file = None):
        '''
    Add a new source to the sources.list.
    The method will search for existing matching repos and will try to 
    reuse them as far as possible
    '''
        comps = orig_comps[:]
        for source in self.list:
            if source.disabled == False and source.invalid == False and source.type == type and uri == source.uri and source.dist == dist:
                for new_comp in comps:
                    if new_comp in source.comps:
                        del comps[comps.index(new_comp)]
                        if len(comps) == 0:
                            return source
                        
                    len(comps) == 0
                
        
        for source in self.list:
            if source.disabled == False and source.invalid == False and source.type == type and uri == source.uri and source.dist == dist:
                comps = uniq(source.comps + comps)
                source.comps = comps
                return source
                continue
            if source.disabled == True and source.invalid == False and source.type == type and uri == source.uri and source.dist == dist and len(set(source.comps) & set(comps)) == len(comps):
                source.disabled = False
                return source
                continue
        
        line = '%s %s %s' % (type, uri, dist)
        for c in comps:
            line = line + ' ' + c
        
        if comment != '':
            line = '%s #%s\n' % (line, comment)
        
        line = line + '\n'
        new_entry = SourceEntry(line)
        if file != None:
            new_entry.file = file
        
        self.matcher.match(new_entry)
        self.list.insert(pos, new_entry)
        return new_entry

    
    def remove(self, source_entry):
        ''' remove the specified entry from the sources.list '''
        self.list.remove(source_entry)

    
    def restoreBackup(self, backup_ext):
        ''' restore sources.list files based on the backup extension '''
        dir = apt_pkg.Config.FindDir('Dir::Etc')
        file = apt_pkg.Config.Find('Dir::Etc::sourcelist')
        if os.path.exists(dir + file + backup_ext) and os.path.exists(dir + file):
            shutil.copy(dir + file + backup_ext, dir + file)
        
        partsdir = apt_pkg.Config.FindDir('Dir::Etc::sourceparts')
        for file in glob.glob('%s/*.list' % partsdir):
            if os.path.exists(file + backup_ext):
                shutil.copy(file + backup_ext, file)
                continue
        

    
    def backup(self, backup_ext = None):
        ''' make a backup of the current source files, if no backup extension
        is given, the current date/time is used (and returned) '''
        already_backuped = set()
        if backup_ext == None:
            backup_ext = time.strftime('%y%m%d.%H%M')
        
        for source in self.list:
            if source.file not in already_backuped and os.path.exists(source.file):
                shutil.copy(source.file, '%s%s' % (source.file, backup_ext))
                continue
        
        return backup_ext

    
    def load(self, file):
        ''' (re)load the current sources '''
        
        try:
            f = open(file, 'r')
            lines = f.readlines()
            for line in lines:
                source = SourceEntry(line, file)
                self.list.append(source)
        except:
            print "could not open file '%s'" % file

        f.close()

    
    def save(self):
        ''' save the current sources '''
        files = { }
        if len(self.list) == 0:
            path = '%s%s' % (apt_pkg.Config.FindDir('Dir::Etc'), apt_pkg.Config.Find('Dir::Etc::sourcelist'))
            header = '## See sources.list(5) for more information, especialy\n# Remember that you can only use http, ftp or file URIs\n# CDROMs are managed through the apt-cdrom tool.\n'
            open(path, 'w').write(header)
            return None
        
        for source in self.list:
            if not files.has_key(source.file):
                files[source.file] = open(source.file, 'w')
            
            files[source.file].write(source.str())
        
        for f in files:
            files[f].close()
        

    
    def check_for_relations(self, sources_list):
        '''get all parent and child channels in the sources list'''
        parents = []
        used_child_templates = { }
        for source in sources_list:
            if source.template == None:
                continue
            
            if source.template.child == True:
                key = source.template
                if not used_child_templates.has_key(key):
                    used_child_templates[key] = []
                
                temp = used_child_templates[key]
                temp.append(source)
                continue
            if len(source.template.children) > 0:
                parents.append(source)
                continue
        
        return (parents, used_child_templates)



class SourceEntryMatcher:
    
    def __init__(self, matcherPath):
        self.templates = []
        spec_files = glob.glob('%s/*.info' % matcherPath)
        for f in spec_files:
            f = os.path.basename(f)
            i = f.find('.info')
            f = f[0:i]
            dist = DistInfo(f, base_dir = matcherPath)
            for template in dist.templates:
                if template.match_uri != None:
                    self.templates.append(template)
                    continue
            
        

    
    def match(self, source):
        '''Add a matching template to the source'''
        _ = gettext.gettext
        found = False
        for template in self.templates:
            if re.search(template.match_uri, source.uri) and re.match(template.match_name, source.dist):
                found = True
                source.template = template
                break
                continue
            if template.is_mirror(source.uri) and re.match(template.match_name, source.dist):
                found = True
                source.template = template
                break
                continue
        
        return found


if __name__ == '__main__':
    apt_pkg.InitConfig()
    sources = SourcesList()
    for entry in sources:
        print entry.str()
    
    mirror = is_mirror('http://archive.ubuntu.com/ubuntu/', 'http://de.archive.ubuntu.com/ubuntu/')
    print 'is_mirror(): %s' % mirror
    print is_mirror('http://archive.ubuntu.com/ubuntu', 'http://de.archive.ubuntu.com/ubuntu/')
    print is_mirror('http://archive.ubuntu.com/ubuntu/', 'http://de.archive.ubuntu.com/ubuntu')

