# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

import os
import sys
import pydoc
from os.path import abspath, expanduser, join, basename
import traceback
import gtk
import gobject
import deskbar
import deskbar.Handler as deskbar
import deskbar.Categories as deskbar
from deskbar.Watcher import DirWatcher
from deskbar.ModuleContext import ModuleContext

class ModuleLoader(gobject.GObject):
    '''An auxilary class to ModuleList. Create an instance of ModuleLoader by
\tspecifying the which directories to search and what extension to accept.
\tThe load_all() method will load all qualified modules into the ModuleList
\tspecified in the constructor.
\t
\tMost methods have a _async variant. These methods emits signals that is handled
\tby the mainloop.
\t\t\t
\tHint: If you pass None as the dirs argument the ModuleLoader will not search
\tfor modules at all. This is useful if you want to reload a single module for
\twhich you know the path.
\t'''
    __gsignals__ = {
        'module-loaded': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [
            gobject.TYPE_PYOBJECT]),
        'modules-loaded': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
        'module-initialized': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [
            gobject.TYPE_PYOBJECT]),
        'module-not-initialized': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [
            gobject.TYPE_PYOBJECT]),
        'module-stopped': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [
            gobject.TYPE_PYOBJECT]) }
    
    def __init__(self, dirs, extension = '.py'):
        '''
\t\tdirs: A list of directories to search. Relative pathnames and paths
\t\t\t  containing ~ will be expanded. If dirs is None the 
\t\t\t  ModuleLoader will not search for modules.
\t\textension: What extension should this ModuleLoader accept (string).
\t\t'''
        gobject.GObject.__init__(self)
        self.ext = extension
        self.watcher = DirWatcher()
        self.watch_id = self.watcher.connect('changed', self._on_handler_file_changed)

    
    def _on_handler_file_changed(self, watcher, f):
        if f in self.filelist or not self.is_module(f):
            return None
        
        self.load(f)
        self.filelist.append(f)

    
    def build_filelist(self):
        '''Returns a list containing the filenames of all qualified modules.
\t\tThis method is automatically invoked by the constructor.
\t\t'''
        res = []
        for d in self.dirs:
            
            try:
                if not os.path.exists(d):
                    continue
                
                for m in os.listdir(d):
                    if self.is_module(m):
                        continue
                    _[1][join(d, m)]
            continue
            except OSError:
                err = None
                print >>sys.stderr, 'Error reading directory %s, skipping.' % d
                traceback.print_exc()
                continue
            

        
        self.filelist = res

    
    def is_module(self, filename):
        '''Tests whether the filename has the appropriate extension.'''
        return filename[-len(self.ext):] == self.ext

    
    def import_module(self, filename):
        '''Tries to import the specified file. Returns the python module on succes.
\t\tPrimarily for internal use.'''
        
        try:
            mod = pydoc.importfile(filename)
        except Exception:
            print >>sys.stderr, 'Error loading the file: %s.' % filename
            traceback.print_exc()
            return None

        
        try:
            if mod.HANDLERS:
                pass
        except AttributeError:
            print >>sys.stderr, 'The file %s is not a valid module. Skipping.' % filename
            print >>sys.stderr, 'A module must have the variable HANDLERS defined as a dictionary.'
            traceback.print_exc()
            return None

        if mod.HANDLERS == None:
            if not hasattr(mod, 'ERROR'):
                mod.ERROR = 'Unspecified Reason'
            
            print >>sys.stderr, '*** The file %s decided to not load itself: %s' % (filename, mod.ERROR)
            return None
        
        for handler, infos in mod.HANDLERS.items():
            if hasattr(getattr(mod, handler), 'initialize') and 'name' in infos:
                pass
            else:
                print >>sys.stderr, "Class %s in file %s does not have an initialize(self) method or does not define a 'name' attribute. Skipping." % (handler, filename)
                return None
            if 'requirements' in infos:
                (status, msg, callback) = infos['requirements']()
                if status == deskbar.Handler.HANDLER_IS_NOT_APPLICABLE:
                    print >>sys.stderr, '***'
                    print >>sys.stderr, '*** The file %s (%s) decided to not load itself: %s' % (filename, handler, msg)
                    print >>sys.stderr, '***'
                    return None
                
            status == deskbar.Handler.HANDLER_IS_NOT_APPLICABLE
        
        return mod

    
    def load(self, filename):
        """Loads the given file as a module and emits a 'module-loaded' signal
\t\tpassing a corresponding ModuleContext as argument.
\t\t"""
        mod = self.import_module(filename)
        if mod is None:
            return None
        
        for handler, infos in mod.HANDLERS.items():
            print "Loading module '%s' from file %s." % (infos['name'], filename)
            mod_instance = getattr(mod, handler)()
            context = ModuleContext(mod_instance.get_icon(), False, mod_instance, filename, handler, infos)
            self.emit('module-loaded', context)
        
        return context

    
    def load_all(self):
        """Tries to load all qualified modules detected by the ModuleLoader.
\t\tEach time a module is loaded it will emit a 'module-loaded' signal
\t\tpassing a corresponding module context.
\t\t"""
        if self.dirs is None:
            print >>sys.stderr, 'The ModuleLoader at %s has no filelist!' % str(id(self))
            print >>sys.stderr, 'It was probably initialized with dirs=None.'
            return None
        
        for f in self.filelist:
            self.load(f)
        
        self.emit('modules-loaded')

    
    def initialize_module(self, context):
        """
\t\tInitializes the module in the given context. Emits a 'module-initialized' signal
\t\twhen done, passing the (now enabled) contextas argument.
\t\tIf module is already initialized, do nothing.
\t\t"""
        if context.enabled:
            return None
        
        print 'Initializing %s' % context.infos['name']
        if context.infos.has_key('requirements'):
            (status, message, callback) = context.infos['requirements']()
            if status == deskbar.Handler.HANDLER_HAS_REQUIREMENTS or status == deskbar.Handler.HANDLER_IS_NOT_APPLICABLE:
                print 'Error while initializing %s. Requirements not met.' % context.infos['name']
                print message
                context.enabled = False
                self.emit('module-not-initialized', context)
                return None
            
        
        
        try:
            context.module.initialize()
            if 'categories' in context.infos:
                for catname, catinfo in context.infos['categories'].items():
                    deskbar.Categories.CATEGORIES[catname] = catinfo
                
        except Exception:
            msg = None
            print 'Error while initializing %s: %s' % (context.infos['name'], msg)
            traceback.print_exc()
            context.enabled = False
            self.emit('module-not-initialized', context)
            return None

        context.enabled = True
        self.emit('module-initialized', context)

    
    def stop_module(self, context):
        """
\t\tStops the module an sets context.enabled = False. Furthermore the context.module
\t\tinstance is also set to None. Emits a 'context-stopped' signal when done passing
\t\tthe stopped context as argument.
\t\t"""
        print 'Stopping %s' % context.infos['name']
        context.module.stop()
        context.enabled = False
        self.emit('module-stopped', context)

    
    def load_all_async(self):
        '''
\t\tSame as load_all() except the loading is done in an idle mainloop call.
\t\t'''
        gobject.idle_add(self.load_all)

    
    def initialize_module_async(self, context):
        '''
\t\tInvokes initialize_module in an idle mainloop call.
\t\t'''
        gobject.idle_add(self.initialize_module, context)

    
    def stop_module_async(self, context):
        '''
\t\tInvokes stop_module in an idle mainloop call.
\t\t'''
        gobject.idle_add(self.stop_module, context)


if gtk.pygtk_version < (2, 8, 0):
    gobject.type_register(ModuleLoader)

