# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

import gobject

class CuemiacLayoutProvider:
    """
\tThis abstract class is a part of the 'Cuemiac Duo' together
\twith C{CuemiacUIManager}. These two classes is all you need
\tto do a layout with the cuemiac.
\t
\tC{CuemiacLayoutProvider} is responsible for the layout and focus
\tlogic of a CuemiacUIManager.
\t
\tTo avoid signal-setup-nightmare C{CuemiacUIManager} uses direct
\tcallbacks on the layout provider. You should think of this class
\tas a template pattern in the ui manager, if you are familiar with
\tdesign patterns.
\t
\tB{Important}
\tTo keep the classes coherent and ease debugging, please take care
\tto only do layout anf focus stuff in the provided callbacks.
\t
\tB{Basic Usage}
\tSubclass and overwrite the callbacks as you see fit. Some of the
\tcallbacks has a default behavior - if this is true, it will be
\tdescribed in the documentation for that method.
\t
\tThe defaults will normally only work properly in a single window
\tlayout. If your layout includes several windows you might have luck
\twith first calling C{gtk.Window.present_with_time(event.time)} and 
\tthen calling the default callback with 
\tC{CuemiacLayoutProvider.on_foo(self, cuim)}.
\t"""
    
    def __init__(self):
        pass

    
    def on_match_selected(self, cuim, match):
        '''
\t\tCalled when a match is selected by the user.
\t\t@param cuim: C{CuemiacUIManager}
\t\t@param match: C{deskbar.Match}
\t\t'''
        pass

    
    def on_history_match_selected(self, cuim, match):
        '''
\t\tCalled when a match is selected from the history view
\t\tby the user.
\t\t@param cuim: C{CuemiacUIManager}
\t\t@param match: C{deskbar.Match}
\t\t'''
        pass

    
    def on_stop(self, cuim):
        '''
\t\tCalled when the ui manager emits a "stop-query" signal.
\t\tThis will typically hide the view, but there is no default behavior.
\t\t@param cuim: C{CuemiacUIManager}
\t\t'''
        pass

    
    def on_start_query(self, cuim, qstring):
        '''
\t\tCalled when the ui manager emits a "start-query" signal.
\t\t@param cuim: C{CuemiacUIManager}
\t\t@param qstring: The query string
\t\t'''
        pass

    
    def on_matches_added(self, cuim):
        '''
\t\tCalled when matches are added to the view. The layout will
\t\ttypically call show() on the view or its parent window, but
\t\tthere is no default behavior.
\t\t@param cuim: C{CuemiacUIManager}
\t\t'''
        pass

    
    def on_up_from_history_top(self, cuim):
        '''
\t\tThe user hits the Up key, when the top match is selected in the
\t\thistory view.
\t\t@param cuim: C{CuemiacUIManager}
\t\t'''
        pass

    
    def on_down_from_history_bottom(self, cuim):
        '''
\t\tThe user hits the Down key, when the bottom match is selected in the
\t\thistory view.
\t\t@param cuim: C{CuemiacUIManager}
\t\t'''
        pass

    
    def on_up_from_view_top(self, cuim, event):
        '''
\t\tThe user hits the Up key, when the top match is selected in the
\t\tnormal match view.
\t\tThis defaults to focussing the entry.
\t\t@param cuim: C{CuemiacUIManager}
\t\t'''
        gobject.timeout_add(10, (lambda : self._focus_entry(cuim, event)))

    
    def on_down_from_view_bottom(self, cuim, event):
        '''
\t\tThe user hits the Up key, when the bottom match is selected in the
\t\tnormal match view.
\t\tThis defaults to focussing the entry.
\t\t@param cuim: C{CuemiacUIManager}
\t\t'''
        gobject.timeout_add(10, (lambda : self._focus_entry(cuim, event)))

    
    def on_up_from_entry(self, cuim, event):
        '''
\t\tCalled when the user hits the Up key when the entry is focussed.
\t\tOnly called when there are visible matches, else the ui manager
\t\twill cycle the history.
\t\tThis defaults focussing the bottom match in the view.
\t\t'''
        cuim.get_view().grab_focus()
        cuim.get_view().focus_bottom_match()

    
    def on_down_from_entry(self, cuim, event):
        '''
\t\tCalled when the user hits the Down key when the entry is focussed.
\t\tOnly called when there are visible matches, else the ui manager
\t\twill cycle the history.
\t\tThis defaults focussing the top match in the view.
\t\t'''
        cuim.get_view().grab_focus()
        cuim.get_view().focus_top_match()

    
    def set_layout_by_orientation(self, cuim, orient):
        '''
\t\tThis method is automatically called by the cuemiac ui manager.
\t\tNote that gnomeapplet.ORIENT_UP typically means that the ui
\t\tis located at the I{bottom} of the screen (and vice versa for ORIENT_DOWN).
\t\t@param cuim: The C{CuemiacUIManager} responsible for the callback.
\t\t@param orient: One of gnomeapplet.ORIENT_{LEFT,RIGHT,UP,DOWN}.
\t\t'''
        pass

    
    def _focus_entry(self, cuim, event):
        cuim.get_entry().grab_focus()
        cuim.unselect_all()


