# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''A very experimental approach to the refreshable Braille display.  This
module treats each line of the display as a sequential set of regions, where
each region can potentially backed by an Accessible object.  Depending upon
the Accessible object, the cursor routing keys can be used to perform
operations on the Accessible object, such as invoking default actions or
moving the text caret.
'''
__id__ = '$Id: braille.py 1995 2007-02-11 16:40:57Z wwalker $'
__version__ = '$Revision: 1995 $'
__date__ = '$Date: 2007-02-11 11:40:57 -0500 (Sun, 11 Feb 2007) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'
import logging
log = logging.getLogger('braille')
import signal
import threading
import atspi

try:
    import brlapi
    brlAPI = None
    useBrlAPIBindings = True
    brlAPIRunning = False
except:
    import brl
    useBrlAPIBindings = False
    brlAPIRunning = False


try:
    import brlmon
except:
    pass

import debug
import eventsynthesizer
import settings
import util
from orca_i18n import _
from rolenames import getShortBrailleForRoleName
from rolenames import getLongBrailleForRoleName
_initialized = False
monitor = None
CMD_NOOP = 0
CMD_LNUP = 1
CMD_LNDN = 2
CMD_WINUP = 3
CMD_WINDN = 4
CMD_PRDIFLN = 5
CMD_NXDIFLN = 6
CMD_ATTRUP = 7
CMD_ATTRDN = 8
CMD_TOP = 9
CMD_BOT = 10
CMD_TOP_LEFT = 11
CMD_BOT_LEFT = 12
CMD_PRPGRPH = 13
CMD_NXPGRPH = 14
CMD_PRPROMPT = 15
CMD_NXPROMPT = 16
CMD_PRSEARCH = 17
CMD_NXSEARCH = 18
CMD_CHRLT = 19
CMD_CHRRT = 20
CMD_HWINLT = 21
CMD_HWINRT = 22
CMD_FWINLT = 23
CMD_FWINRT = 24
CMD_FWINLTSKIP = 25
CMD_FWINRTSKIP = 26
CMD_LNBEG = 27
CMD_LNEND = 28
CMD_HOME = 29
CMD_BACK = 30
CMD_FREEZE = 31
CMD_DISPMD = 32
CMD_SIXDOTS = 33
CMD_SLIDEWIN = 34
CMD_SKPIDLNS = 35
CMD_SKPBLNKWINS = 36
CMD_CSRVIS = 37
CMD_CSRHIDE = 38
CMD_CSRTRK = 39
CMD_CSRSIZE = 40
CMD_CSRBLINK = 41
CMD_ATTRVIS = 42
CMD_ATTRBLINK = 43
CMD_CAPBLINK = 44
CMD_TUNES = 45
CMD_HELP = 46
CMD_INFO = 47
CMD_LEARN = 48
CMD_PREFMENU = 49
CMD_PREFSAVE = 50
CMD_PREFLOAD = 51
CMD_MENU_FIRST_ITEM = 52
CMD_MENU_LAST_ITEM = 53
CMD_MENU_PREV_ITEM = 54
CMD_MENU_NEXT_ITEM = 55
CMD_MENU_PREV_SETTING = 56
CMD_MENU_NEXT_SETTING = 57
CMD_SAY_LINE = 58
CMD_SAY_ABOVE = 59
CMD_SAY_BELOW = 60
CMD_MUTE = 61
CMD_SPKHOME = 62
CMD_SWITCHVT_PREV = 63
CMD_SWITCHVT_NEXT = 64
CMD_CSRJMP_VERT = 65
CMD_PASTE = 66
CMD_RESTARTBRL = 67
CMD_RESTARTSPEECH = 68
CMD_MAX = 68
BRL_FLG_REPEAT_INITIAL = 8388608
BRL_FLG_REPEAT_DELAY = 4194304
command_name = { }
command_name[CMD_FWINLT] = _('Line Left')
command_name[CMD_FWINRT] = _('Line Right')
command_name[CMD_LNUP] = _('Line Up')
command_name[CMD_LNDN] = _('Line Down')
command_name[CMD_TOP_LEFT] = _('Top Left')
command_name[CMD_BOT_LEFT] = _('Bottom Right')
command_name[CMD_HOME] = _('Cursor Position')
_displaySize = [
    32,
    1]
_lines = []
_regionWithFocus = None
_viewport = [
    0,
    0]
_callback = None
endIsShowing = False
beginningIsShowing = False
cursorCell = 0

def _printBrailleEvent(level, command):
    '''Prints out a Braille event.  The given level may be overridden
    if the eventDebugLevel (see debug.setEventDebugLevel) is greater in
    debug.py.

    Arguments:
    - command: the BrlAPI command for the key that was pressed.
    '''
    debug.printInputEvent(level, 'BRAILLE EVENT: %x' % command)


class Region:
    '''A Braille region to be displayed on the display.  The width of
    each region is determined by its string.
    '''
    
    def __init__(self, string, cursorOffset = 0):
        '''Creates a new Region containing the given string.

        Arguments:
        - string: the string to be displayed
        - cursorOffset: a 0-based index saying where to draw the cursor
                        for this Region if it gets focus.
        '''
        if not string:
            string = ''
        
        string = string.decode('UTF-8')
        if string[-1:] == '\n':
            string = string[:-1]
        
        self.string = string.encode('UTF-8')
        self.cursorOffset = cursorOffset

    
    def processCursorKey(self, offset):
        '''Processes a cursor key press on this Component.  The offset is
        0-based, where 0 represents the leftmost character of string
        associated with this region.  Note that the zeroeth character may have
        been scrolled off the display.'''
        pass



class Component(Region):
    '''A subclass of Region backed by an accessible.  This Region will react
    to any cursor routing key events and perform the default action on the
    accessible, if a default action exists.
    '''
    
    def __init__(self, accessible, string, cursorOffset = 0):
        '''Creates a new Component.

        Arguments:
        - accessible: the accessible
        - string: the string to use to represent the component
        - cursorOffset: a 0-based index saying where to draw the cursor
                        for this Region if it gets focus.
        '''
        Region.__init__(self, string, cursorOffset)
        self.accessible = accessible

    
    def processCursorKey(self, offset):
        '''Processes a cursor key press on this Component.  The offset is
        0-based, where 0 represents the leftmost character of string
        associated with this region.  Note that the zeroeth character may have
        been scrolled off the display.'''
        actions = self.accessible.action
        if actions:
            actions.doAction(0)
        else:
            debug.println(debug.LEVEL_FINEST, 'braille.Component.processCursorKey: no action')
            
            try:
                eventsynthesizer.clickObject(self.accessible, 1)
            except:
                debug.printException(debug.LEVEL_SEVERE)




class Text(Region):
    '''A subclass of Region backed by a Text object.  This Region will
    react to any cursor routing key events by positioning the caret in
    the associated text object. The line displayed will be the
    contents of the text object preceded by an optional label.
    [[[TODO: WDW - need to add in text selection capabilities.  Logged
    as bugzilla bug 319754.]]]'''
    
    def __init__(self, accessible, label = None):
        '''Creates a new Text region.

        Arguments:
        - accessible: the accessible that implements AccessibleText
        - label: an optional label to display
        '''
        self.accessible = accessible
        (string, self.caretOffset, self.lineOffset) = util.getTextLineAtCaret(self.accessible)
        if self.lineOffset < 0:
            self.caretOffset = 0
            self.lineOffset = 0
            (string, startOffset, endOffset) = self.accessible.text.getTextAtOffset(0, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
        
        cursorOffset = self.caretOffset - self.lineOffset
        self.label = label
        if self.label:
            string = self.label + ' ' + string
            cursorOffset += len(self.label.decode('UTF-8')) + 1
        
        Region.__init__(self, string, cursorOffset)

    
    def repositionCursor(self):
        '''Attempts to reposition the cursor in response to a new
        caret position.  If it is possible (i.e., the caret is on
        the same line as it was), reposition the cursor and return
        True.  Otherwise, return False.
        '''
        (string, caretOffset, lineOffset) = util.getTextLineAtCaret(self.accessible)
        cursorOffset = caretOffset - lineOffset
        if self.label:
            cursorOffset += len(self.label.decode('UTF-8')) + 1
        
        if lineOffset != self.lineOffset:
            return False
        else:
            self.caretOffset = caretOffset
            self.lineOffset = lineOffset
            self.cursorOffset = cursorOffset
        return True

    
    def processCursorKey(self, offset):
        '''Processes a cursor key press on this Component.  The offset is
        0-based, where 0 represents the leftmost character of text associated
        with this region.  Note that the zeroeth character may have been
        scrolled off the display.'''
        if self.label:
            offset = offset - len(self.label.decode('UTF-8')) - 1
            if offset < 0:
                return None
            
        
        newCaretOffset = self.lineOffset + offset
        self.accessible.text.setCaretOffset(newCaretOffset)



class ReviewComponent(Component):
    '''A subclass of Component that is to be used for flat review mode.'''
    
    def __init__(self, accessible, string, cursorOffset, zone):
        '''Creates a new Component.

        Arguments:
        - accessible: the accessible
        - string: the string to use to represent the component
        - cursorOffset: a 0-based index saying where to draw the cursor
                        for this Region if it gets focus.
        - zone: the flat review Zone associated with this component
        '''
        Component.__init__(self, accessible, string, cursorOffset)
        self.zone = zone



class ReviewText(Region):
    '''A subclass of Region backed by a Text object.  This Region will
    does not react to the caret changes, but will react if one updates
    the cursorPosition.  This class is meant to be used by flat review
    mode to show the current character position.
    '''
    
    def __init__(self, accessible, string, lineOffset, zone):
        '''Creates a new Text region.

        Arguments:
        - accessible: the accessible that implements AccessibleText
        - string: the string to use to represent the component
        - lineOffset: the character offset into where the text line starts
        - zone: the flat review Zone associated with this component
        '''
        Region.__init__(self, string)
        self.accessible = accessible
        self.lineOffset = lineOffset
        self.zone = zone

    
    def processCursorKey(self, offset):
        '''Processes a cursor key press on this Component.  The offset is
        0-based, where 0 represents the leftmost character of text associated
        with this region.  Note that the zeroeth character may have been
        scrolled off the display.'''
        newCaretOffset = self.lineOffset + offset
        self.accessible.text.setCaretOffset(newCaretOffset)



class Line:
    '''A horizontal line on the display.  Each Line is composed of a sequential
    set of Regions.
    '''
    
    def __init__(self, region = None):
        self.regions = []
        self.string = ''
        if region:
            self.addRegion(region)
        

    
    def addRegion(self, region):
        self.regions.append(region)

    
    def addRegions(self, regions):
        self.regions.extend(regions)

    
    def getLineInfo(self):
        '''Computes the complete string for this line as well as a
        0-based index where the focused region starts on this line.
        If the region with focus is not on this line, then the index
        will be -1.

        Returns [string, offsetIndex]
        '''
        string = ''
        focusOffset = -1
        for region in self.regions:
            if region == _regionWithFocus:
                focusOffset = len(string.decode('UTF-8'))
            
            if region.string:
                string += region.string.replace('\xe2\x80\xa6', '...')
                continue
        
        return [
            string,
            focusOffset]

    
    def getRegionAtOffset(self, offset):
        '''Finds the Region at the given 0-based offset in this line.

        Returns the [region, offsetinregion] where the region is
        the region at the given offset, and offsetinregion is the
        0-based offset from the beginning of the region, representing
        where in the region the given offset is.'''
        region = None
        string = ''
        pos = 0
        for region in self.regions:
            string = string + region.string
            if len(string.decode('UTF-8')) > offset:
                break
                continue
            pos = len(string.decode('UTF-8'))
        
        return [
            region,
            offset - pos]

    
    def processCursorKey(self, offset):
        '''Processes a cursor key press on this Component.  The offset is
        0-based, where 0 represents the leftmost character of string
        associated with this line.  Note that the zeroeth character may have
        been scrolled off the display.'''
        (region, regionOffset) = self.getRegionAtOffset(offset)
        region.processCursorKey(regionOffset)



def getRegionAtCell(cell):
    """Given a 1-based cell offset, return the braille region
    associated with that cell in the form of [region, offsetinregion]
    where 'region' is the region associated with the cell and
    'offsetinregion' is the 0-based offset of where the cell is
    in the region, where 0 represents the beginning of the region, """
    if len(_lines) > 0:
        offset = (cell - 1) + _viewport[0]
        lineNum = _viewport[1]
        return _lines[lineNum].getRegionAtOffset(offset)
    else:
        return [
            None,
            -1]


def clear():
    '''Clears the logical structure, but keeps the Braille display as is
    (until a refresh operation).
    '''
    global _lines, _regionWithFocus, _viewport
    _lines = []
    _regionWithFocus = None
    _viewport = [
        0,
        0]


def setLines(lines):
    global _lines
    _lines = lines


def addLine(line):
    '''Adds a line to the logical display for painting.  The line is added to
    the end of the current list of known lines.  It is necessary for the
    viewport to be over the lines and for refresh to be called for the new
    line to be painted.

    Arguments:
    - line: an instance of Line to add.
    '''
    _lines.append(line)
    line._index = len(_lines)


def getShowingLine():
    '''Returns the Line that is currently being painted on the display.
    '''
    return _lines[_viewport[1]]


def setFocus(region, panToFocus = True):
    '''Specififes the region with focus.  This region will be positioned
    at the home position if panToFocus is True.

    Arguments:
    - region: the given region, which much be in a line that has been
              added to the logical display
    '''
    global _regionWithFocus
    _regionWithFocus = region
    if not panToFocus or not _regionWithFocus:
        return None
    
    lineNum = 0
    done = False
    for line in _lines:
        for reg in line.regions:
            if reg == _regionWithFocus:
                _viewport[1] = lineNum
                done = True
                break
                continue
        
        if done:
            break
            continue
        lineNum += 1
    
    line = _lines[_viewport[1]]
    (string, offset) = line.getLineInfo()
    if _regionWithFocus.cursorOffset >= _displaySize[0]:
        offset += (_regionWithFocus.cursorOffset - _displaySize[0]) + 1
    
    _viewport[0] = max(0, offset)


def refresh(panToCursor = True, targetCursorCell = 0):
    """Repaints the Braille on the physical display.  This clips the entire
    logical structure by the viewport and also sets the cursor to the
    appropriate location.  [[[TODO: WDW - I'm not sure how BrlTTY handles
    drawing to displays with more than one line, so I'm only going to handle
    drawing one line right now.]]]

    Arguments:

    - panToCursor: if True, will adjust the viewport so the cursor is
                   showing.
    - targetCursorCell: Only effective if panToCursor is True.
                        0 means automatically place the cursor somewhere
                        on the display so as to minimize movement but
                        show as much of the line as possible.  A positive
                        value is a 1-based target cell from the left side
                        of the display and a negative value is a 1-based
                        target cell from the right side of the display.
    """
    global cursorCell, cursorCell, cursorCell, monitor, beginningIsShowing, endIsShowing
    if len(_lines) == 0:
        if useBrlAPIBindings:
            if brlAPIRunning:
                brlAPI.writeText('', 0)
            
        else:
            brl.writeText(0, '')
        return None
    
    if targetCursorCell < 0:
        targetCursorCell = _displaySize[0] + targetCursorCell + 1
    
    line = _lines[_viewport[1]]
    (string, focusOffset) = line.getLineInfo()
    cursorOffset = -1
    if focusOffset >= 0:
        cursorOffset = focusOffset + _regionWithFocus.cursorOffset
    
    if panToCursor and cursorOffset >= 0:
        if len(string.decode('UTF-8')) <= _displaySize[0]:
            _viewport[0] = 0
        elif targetCursorCell:
            _viewport[0] = max(0, (cursorOffset - targetCursorCell) + 1)
        elif cursorOffset < _viewport[0]:
            _viewport[0] = max(0, cursorOffset)
        elif cursorOffset >= _viewport[0] + _displaySize[0]:
            _viewport[0] = max(0, (cursorOffset - _displaySize[0]) + 1)
        
    
    startPos = _viewport[0]
    endPos = startPos + _displaySize[0]
    cursorCell = cursorOffset - startPos
    if cursorCell < 0 or cursorCell >= _displaySize[0]:
        cursorCell = 0
    else:
        cursorCell += 1
    debug.println(debug.LEVEL_INFO, "BRAILLE LINE:  '%s'" % string)
    log.info("line:'%s'" % string)
    debug.println(debug.LEVEL_INFO, "     VISIBLE:  '%s', cursor=%d" % (string[startPos:endPos], cursorCell))
    log.info("visible:'%s'" % string[startPos:endPos])
    log.info('cursor:%d' % cursorCell)
    string = string.decode('UTF-8')
    substring = string[startPos:endPos].encode('UTF-8')
    if useBrlAPIBindings:
        if brlAPIRunning:
            
            try:
                writeStruct = brlapi.WriteStruct()
            except:
                writeStruct = brlapi.Write()

            writeStruct.text = substring
            writeStruct.regionBegin = 1
            writeStruct.regionSize = len(substring.decode('UTF-8'))
            while writeStruct.regionSize < _displaySize[0]:
                writeStruct.text += ' '
                writeStruct.regionSize += 1
                continue
                writeStruct
            writeStruct.cursor = cursorCell
            writeStruct.charset = 'UTF-8'
            brlAPI.write(writeStruct)
        
    else:
        brl.writeText(cursorCell, substring)
    if settings.enableBrailleMonitor:
        if not monitor:
            monitor = brlmon.BrlMon(_displaySize[0])
            monitor.show_all()
        
        monitor.writeText(cursorCell, substring)
    elif monitor:
        monitor.destroy()
    
    beginningIsShowing = startPos == 0
    endIsShowing = endPos >= len(string)


def displayRegions(regionInfo):
    '''Displays a list of regions on a single line, setting focus to the
       specified region.  The regionInfo parameter is something that is
       typically returned by a call to braillegenerator.getBrailleRegions.

    Arguments:
    - regionInfo: a list where the first element is a list of regions
                  to display and the second element is the region
                  with focus (must be in the list from element 0)
    '''
    regions = regionInfo[0]
    focusedRegion = regionInfo[1]
    clear()
    line = Line()
    for item in regions:
        line.addRegion(item)
    
    addLine(line)
    setFocus(focusedRegion)
    refresh()


def displayMessage(message, cursor = -1):
    '''Displays a single line, setting the cursor to the given position,
    ensuring that the cursor is in view.

    Arguments:
    - message: the string to display
    - cursor: the 0-based cursor position, where -1 (default) means no cursor
    '''
    clear()
    region = Region(message, cursor)
    addLine(Line(region))
    setFocus(region)
    refresh(True)


def panLeft(panAmount = 0):
    '''Pans the display to the left, limiting the pan to the beginning
    of the line being displayed.

    Arguments:
    - panAmount: the amount to pan.  A value of 0 means the entire
                 width of the physical display.

    Returns True if a pan actually happened.
    '''
    oldX = _viewport[0]
    if panAmount == 0:
        panAmount = _displaySize[0]
    
    if _viewport[0] > 0:
        _viewport[0] = max(0, _viewport[0] - panAmount)
    
    return oldX != _viewport[0]


def panRight(panAmount = 0):
    '''Pans the display to the right, limiting the pan to the length
    of the line being displayed.

    Arguments:
    - panAmount: the amount to pan.  A value of 0 means the entire
                 width of the physical display.

    Returns True if a pan actually happened.
    '''
    oldX = _viewport[0]
    if panAmount == 0:
        panAmount = _displaySize[0]
    
    if len(_lines) > 0:
        lineNum = _viewport[1]
        newX = _viewport[0] + panAmount
        (string, focusOffset) = _lines[lineNum].getLineInfo()
        if newX < len(string.decode('UTF-8')):
            _viewport[0] = newX
        
    
    return oldX != _viewport[0]


def panToOffset(offset):
    '''Automatically pan left or right to make sure the current offset is
    showing.'''
    while offset < _viewport[0]:
        debug.println(debug.LEVEL_FINEST, 'braille.panToOffset (left) %d' % offset)
        if not panLeft():
            break
            continue
    while offset >= _viewport[0] + _displaySize[0]:
        debug.println(debug.LEVEL_FINEST, 'braille.panToOffset (right) %d' % offset)
        if not panRight():
            break
            continue


def returnToRegionWithFocus(inputEvent = None):
    '''Pans the display so the region with focus is displayed.

    Arguments:
    - inputEvent: the InputEvent instance that caused this to be called.

    Returns True to mean the command should be consumed.
    '''
    setFocus(_regionWithFocus)
    refresh(True)
    return True


def _processBrailleEvent(command):
    '''Handles BrlTTY command events.  This passes commands on to Orca for
    processing.  If Orca does not handle them (as indicated by a return value
    of false from the callback passed to init, it will attempt to handle the
    command itself - either by panning the viewport or passing cursor routing
    keys to the Regions for handling.

    Arguments:
    - command: the BrlAPI command for the key that was pressed.
    '''
    _printBrailleEvent(debug.LEVEL_FINE, command)
    if command & BRL_FLG_REPEAT_INITIAL:
        command &= ~(BRL_FLG_REPEAT_INITIAL | BRL_FLG_REPEAT_DELAY)
    elif command & BRL_FLG_REPEAT_DELAY:
        return True
    
    consumed = False
    if settings.timeoutCallback and settings.timeoutTime > 0:
        signal.signal(signal.SIGALRM, settings.timeoutCallback)
        signal.alarm(settings.timeoutTime)
    
    if _callback:
        
        try:
            consumed = _callback(command)
        debug.printException(debug.LEVEL_WARNING)
        consumed = False

    
    if command >= 256 and command < 256 + _displaySize[0]:
        if len(_lines) > 0:
            cursor = (command - 256) + _viewport[0]
            lineNum = _viewport[1]
            _lines[lineNum].processCursorKey(cursor)
            consumed = True
        
    
    if settings.timeoutCallback and settings.timeoutTime > 0:
        signal.alarm(0)
    
    return consumed


def _brlAPIKeyReader():
    '''Method to read a key from the BrlAPI bindings.  This is a
        gidle handler.
        '''
    key = brlAPI.readKey(False)
    if key:
        flags = key >> 32
        lower = key & 0xFFFFFFFFL
        keyType = lower >> 29
        keyCode = lower & 536870911
        if keyCode & 65536:
            keyCode = 256 | keyCode & 255
        
        if keyCode:
            _processBrailleEvent(keyCode)
        
    
    return brlAPIRunning


def setupKeyRanges(keys):
    '''Hacky method to tell BrlTTY what to send and not send us via
    the readKey method.  This only works with BrlTTY v3.8 and better.

    Arguments:
    -keys: a list of BrlAPI commands.
    '''
    if not brlAPIRunning:
        return None
    
    
    try:
        brlAPI.ignoreKeys(brlapi.rangeType_all, [
            0])
        keySet = [
            brlapi.KEY_TYPE_CMD | brlapi.KEY_CMD_ROUTE]
        for key in keys:
            keySet.append(brlapi.KEY_TYPE_CMD | key)
        
        brlAPI.acceptKeys(brlapi.rangeType_command, keySet)
        debug.println(debug.LEVEL_FINEST, 'Using BrlAPI v0.5.0+')
    except:
        debug.printException(debug.LEVEL_FINEST)
        
        try:
            brlAPI.ignoreKeyRange(0, brlapi.KEY_FLAGS_MASK | brlapi.KEY_TYPE_MASK | brlapi.KEY_CODE_MASK)
            brlAPI.acceptKeyRange(brlapi.KEY_TYPE_CMD | brlapi.KEY_CMD_ROUTE, brlapi.KEY_TYPE_CMD | brlapi.KEY_CMD_ROUTE | brlapi.KEY_CMD_ARG_MASK)
            keySet = []
            for key in keys:
                keySet.append(brlapi.KEY_TYPE_CMD | key)
            
            if len(keySet):
                brlAPI.acceptKeySet(keySet)
            
            debug.println(debug.LEVEL_FINEST, 'Using BrlAPI pre-release v0.5.0')
        debug.printException(debug.LEVEL_FINEST)
        debug.println(debug.LEVEL_WARNING, 'Braille module cannot listen for braille input events')




def init(callback = None, tty = 7):
    '''Initializes the braille module, connecting to the BrlTTY driver.

    Arguments:
    - callback: the method to call with a BrlTTY input event.
    - tty: the tty port to take ownership of (default = 7)
    Returns True if the initialization procedure was run or False if this
    module has already been initialized.
    '''
    global _callback, brlAPI, brlAPIRunning, brlAPIRunning, _displaySize, _displaySize, _initialized
    if _initialized:
        return False
    
    _callback = callback
    if useBrlAPIBindings:
        
        try:
            import gobject
            gobject.threads_init()
            brlAPI = brlapi.Connection()
            
            try:
                import os
                windowPath = os.environ['WINDOWPATH']
                brlAPI.enterTtyModeWithPath()
                brlAPIRunning = True
                gobject.idle_add(_brlAPIKeyReader)
                debug.println(debug.LEVEL_CONFIGURATION, 'Braille module has been initialized using WINDOWPATH=' + '%s' % windowPath)
            except:
                brlAPI.enterTtyMode(tty)
                brlAPIRunning = True
                gobject.idle_add(_brlAPIKeyReader)
                debug.println(debug.LEVEL_CONFIGURATION, 'Braille module has been initialized using tty=%d' % tty)

        debug.printException(debug.LEVEL_FINEST)
        return False

    elif brl.init(tty):
        debug.println(debug.LEVEL_CONFIGURATION, 'Braille module has been initialized.')
        brl.registerCallback(_processBrailleEvent)
    else:
        debug.println(debug.LEVEL_CONFIGURATION, 'Braille module has NOT been initialized.')
        return False
    if useBrlAPIBindings:
        (x, y) = brlAPI.displaySize
        _displaySize = [
            x,
            1]
    else:
        _displaySize = [
            brl.getDisplayWidth(),
            1]
    debug.println(debug.LEVEL_CONFIGURATION, 'braille display size = (%d, %d)' % (_displaySize[0], _displaySize[1]))
    clear()
    refresh(True)
    _initialized = True
    return True


def shutdown():
    '''Shuts down the braille module.   Returns True if the shutdown procedure
    was run or False if this module has not been initialized.
    '''
    global brlAPIRunning, _initialized
    if not _initialized:
        return False
    
    if useBrlAPIBindings:
        if brlAPIRunning:
            brlAPIRunning = False
            brlAPI.leaveTtyMode()
        
    else:
        brl.shutdown()
    _initialized = False
    return True

