# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''Provides debug utilities for Orca.  Debugging is managed by a debug
level, which is held in the debugLevel field.  All other methods take
a debug level, which is compared to the current debug level to
determine if the content should be output.'''
__id__ = '$Id: debug.py 1808 2006-12-12 03:40:50Z lmonsanto $'
__version__ = '$Revision: 1808 $'
__date__ = '$Date: 2006-12-11 22:40:50 -0500 (Mon, 11 Dec 2006) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'
import logging
logging.basicConfig()
log = logging.getLogger()
import traceback
LEVEL_OFF = 10000
LEVEL_SEVERE = 1000
LEVEL_WARNING = 900
LEVEL_INFO = 800
LEVEL_CONFIGURATION = 700
LEVEL_FINE = 600
LEVEL_FINER = 500
LEVEL_FINEST = 400
LEVEL_ALL = 0
debugLevel = LEVEL_SEVERE
debugFile = None
eventDebugLevel = LEVEL_FINEST
eventDebugFilter = None

def printException(level):
    '''Prints out information regarding the current exception.

    Arguments:
    - level: the accepted debug level
    '''
    if level >= debugLevel:
        println(level)
        traceback.print_exc(100, debugFile)
        println(level)
    


def printStack(level):
    '''Prints out the current stack.

    Arguments:
    - level: the accepted debug level
    '''
    if level >= debugLevel:
        println(level)
        traceback.print_stack(None, 100, debugFile)
        println(level)
    


def println(level, text = ''):
    '''Prints the text to stdout if debug is enabled.

    Arguments:
    - level: the accepted debug level
    - text: the text to print (default is a blank line)
    '''
    if level >= debugLevel:
        if debugFile:
            debugFile.writelines([
                text,
                '\n'])
        else:
            print text
    


def printObjectEvent(level, event, sourceInfo = None):
    '''Prints out an Python Event object.  The given level may be
    overridden if the eventDebugLevel is greater.  Furthermore, only
    events with event types matching the eventDebugFilter regular
    expression will be printed.

    Arguments:
    - level: the accepted debug level
    - event: the Python Event to print
    - sourceInfo: additional string to print out
    '''
    if eventDebugFilter and not eventDebugFilter.match(event.type):
        return None
    
    level = max(level, eventDebugLevel)
    text = 'OBJECT EVENT: %-40s detail=(%d,%d)' % (event.type, event.detail1, event.detail2)
    println(level, text)
    if sourceInfo:
        println(level, '             ' + sourceInfo)
    


def printInputEvent(level, string):
    '''Prints out an input event.  The given level may be overridden
    if the eventDebugLevel (see setEventDebugLevel) is greater.

    Arguments:
    - level: the accepted debug level
    - string: the string representing the input event
    '''
    println(max(level, eventDebugLevel), string)


def printDetails(level, indent, accessible, includeApp = True):
    '''Lists the details of the given accessible with the given
    indentation.

    Arguments:
    - level: the accepted debug level
    - indent: a string containing spaces for indentation
    - accessible: the accessible whose details are to be listed
    - includeApp: if True, include information about the app
    '''
    if level >= debugLevel and accessible:
        println(level, accessible.toString(indent, includeApp))
    

