# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''The default Script for presenting information to the user using
both speech and Braille.  This is based primarily on the de-facto
standard implementation of the AT-SPI, which is the GAIL support
for GTK.'''
__id__ = '$Id: default.py 2090 2007-03-04 23:08:20Z wwalker $'
__version__ = '$Revision: 2090 $'
__date__ = '$Date: 2007-03-04 18:08:20 -0500 (Sun, 04 Mar 2007) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'
import time
import atspi
import braille
import chnames
import debug
import find
import flat_review
import input_event
import keybindings
import mag
import orca
import orca_state
import rolenames
import script
import settings
import speech
import speechserver
import string
import util
import where_am_I
from orca_i18n import _

class Script(script.Script):
    
    def __init__(self, app):
        '''Creates a new script for the given application.

        Arguments:
        - app: the application to create a script for.
        '''
        script.Script.__init__(self, app)
        self.flatReviewContext = None
        self.windowActivateTime = None
        self.lastReviewCurrentEvent = None
        self.lastWhereAmIEvent = None

    
    def setupInputEventHandlers(self):
        '''Defines InputEventHandler fields for this script that can be
        called by the key and braille bindings.'''
        self.inputEventHandlers['leftClickReviewItemHandler'] = input_event.InputEventHandler(Script.leftClickReviewItem, _('Performs left click on current flat review item.'))
        self.inputEventHandlers['rightClickReviewItemHandler'] = input_event.InputEventHandler(Script.rightClickReviewItem, _('Performs right click on current flat review item.'))
        self.inputEventHandlers['sayAllHandler'] = input_event.InputEventHandler(Script.sayAll, _('Speaks entire document.'))
        self.inputEventHandlers['whereAmIHandler'] = input_event.InputEventHandler(Script.whereAmI, _('Performs the where am I operation.'))
        self.inputEventHandlers['findHandler'] = input_event.InputEventHandler(orca._showFindGUI, _('Opens the Orca Find dialog.'))
        self.inputEventHandlers['findNextHandler'] = input_event.InputEventHandler(Script.findNext, _('Searches for the next instance of a string.'))
        self.inputEventHandlers['findPreviousHandler'] = input_event.InputEventHandler(Script.findPrevious, _('Searches for the previous instance of a string.'))
        self.inputEventHandlers['showZonesHandler'] = input_event.InputEventHandler(Script.showZones, _('Paints and prints the visible zones in the active window.'))
        self.inputEventHandlers['toggleFlatReviewModeHandler'] = input_event.InputEventHandler(Script.toggleFlatReviewMode, _('Enters and exits flat review mode.'))
        self.inputEventHandlers['reviewPreviousLineHandler'] = input_event.InputEventHandler(Script.reviewPreviousLine, _('Moves flat review to the beginning of the previous line.'))
        self.inputEventHandlers['reviewHomeHandler'] = input_event.InputEventHandler(Script.reviewHome, _('Moves flat review to the home position.'))
        self.inputEventHandlers['reviewCurrentLineHandler'] = input_event.InputEventHandler(Script.reviewCurrentLine, _('Speaks the current flat review line.'))
        self.inputEventHandlers['reviewNextLineHandler'] = input_event.InputEventHandler(Script.reviewNextLine, _('Moves flat review to the beginning of the next line.'))
        self.inputEventHandlers['reviewEndHandler'] = input_event.InputEventHandler(Script.reviewEnd, _('Moves flat review to the end position.'))
        self.inputEventHandlers['reviewPreviousItemHandler'] = input_event.InputEventHandler(Script.reviewPreviousItem, _('Moves flat review to the previous item or word.'))
        self.inputEventHandlers['reviewAboveHandler'] = input_event.InputEventHandler(Script.reviewAbove, _('Moves flat review to the word above the current word.'))
        self.inputEventHandlers['reviewCurrentItemHandler'] = input_event.InputEventHandler(Script.reviewCurrentItem, _('Speaks or spells the current flat review item or word.'))
        self.inputEventHandlers['reviewCurrentAccessibleHandler'] = input_event.InputEventHandler(Script.reviewCurrentAccessible, _('Speaks the current flat review object.'))
        self.inputEventHandlers['reviewNextItemHandler'] = input_event.InputEventHandler(Script.reviewNextItem, _('Moves flat review to the next item or word.'))
        self.inputEventHandlers['reviewBelowHandler'] = input_event.InputEventHandler(Script.reviewBelow, _('Moves flat review to the word below the current word.'))
        self.inputEventHandlers['reviewPreviousCharacterHandler'] = input_event.InputEventHandler(Script.reviewPreviousCharacter, _('Moves flat review to the previous character.'))
        self.inputEventHandlers['reviewEndOfLineHandler'] = input_event.InputEventHandler(Script.reviewEndOfLine, _('Moves flat review to the end of the line.'))
        self.inputEventHandlers['reviewCurrentCharacterHandler'] = input_event.InputEventHandler(Script.reviewCurrentCharacter, _('Speaks the current flat review character.'))
        self.inputEventHandlers['reviewNextCharacterHandler'] = input_event.InputEventHandler(Script.reviewNextCharacter, _('Moves flat review to the next character.'))
        self.inputEventHandlers['toggleTableCellReadModeHandler'] = input_event.InputEventHandler(Script.toggleTableCellReadMode, _('Toggles whether to read just the current table cell or the whole row.'))
        self.inputEventHandlers['readCharAttributesHandler'] = input_event.InputEventHandler(Script.readCharAttributes, _('Reads the attributes associated with the current text character.'))
        self.inputEventHandlers['reportScriptInfoHandler'] = input_event.InputEventHandler(Script.reportScriptInfo, _('Reports information on current script.'))
        self.inputEventHandlers['panBrailleLeftHandler'] = input_event.InputEventHandler(Script.panBrailleLeft, _('Pans the braille display to the left.'), False)
        self.inputEventHandlers['panBrailleRightHandler'] = input_event.InputEventHandler(Script.panBrailleRight, _('Pans the braille display to the right.'), False)
        self.inputEventHandlers['reviewBottomLeftHandler'] = input_event.InputEventHandler(Script.reviewBottomLeft, _('Moves flat review to the bottom left.'))
        self.inputEventHandlers['goBrailleHomeHandler'] = input_event.InputEventHandler(Script.goBrailleHome, _('Returns to object with keyboard focus.'))
        self.inputEventHandlers['enterLearnModeHandler'] = input_event.InputEventHandler(Script.enterLearnMode, _('Enters learn mode.  Press escape to exit learn mode.'))
        self.inputEventHandlers['exitLearnModeHandler'] = input_event.InputEventHandler(Script.exitLearnMode, _('Exits learn mode.'))
        self.inputEventHandlers['decreaseSpeechRateHandler'] = input_event.InputEventHandler(speech.decreaseSpeechRate, _('Decreases the speech rate.'))
        self.inputEventHandlers['increaseSpeechRateHandler'] = input_event.InputEventHandler(speech.increaseSpeechRate, _('Increases the speech rate.'))
        self.inputEventHandlers['decreaseSpeechPitchHandler'] = input_event.InputEventHandler(speech.decreaseSpeechPitch, _('Decreases the speech pitch.'))
        self.inputEventHandlers['increaseSpeechPitchHandler'] = input_event.InputEventHandler(speech.increaseSpeechPitch, _('Increases the speech pitch.'))
        self.inputEventHandlers['shutdownHandler'] = input_event.InputEventHandler(orca._showQuitGUI, _('Quits Orca'))
        self.inputEventHandlers['keystrokeRecordingHandler'] = input_event.InputEventHandler(orca.toggleKeystrokeRecording, _('Toggles keystroke recording on and off.'))
        self.inputEventHandlers['preferencesSettingsHandler'] = input_event.InputEventHandler(orca._showPreferencesGUI, _('Displays the preferences configuration dialog.'))
        self.inputEventHandlers['loadUserSettingsHandler'] = input_event.InputEventHandler(orca.loadUserSettings, _('Reloads user settings and reinitializes services as necessary.'))
        self.inputEventHandlers['toggleSilenceSpeechHandler'] = input_event.InputEventHandler(orca._toggleSilenceSpeech, _('Toggles the silencing of speech.'))
        self.inputEventHandlers['listAppsHandler'] = input_event.InputEventHandler(orca.printApps, _('Prints a debug listing of all known applications to the console where Orca is running.'))
        self.inputEventHandlers['cycleDebugLevelHandler'] = input_event.InputEventHandler(orca.cycleDebugLevel, _('Cycles the debug level at run time.'))
        self.inputEventHandlers['printActiveAppHandler'] = input_event.InputEventHandler(orca.printActiveApp, _('Prints debug information about the currently active application to the console where Orca is running.'))
        self.inputEventHandlers['printAncestryHandler'] = input_event.InputEventHandler(orca.printAncestry, _('Prints debug information about the ancestry of the object with focus'))
        self.inputEventHandlers['printHierarchyHandler'] = input_event.InputEventHandler(orca.printHierarchy, _('Prints debug information about the application with focus'))
        self.inputEventHandlers['nextPresentationManagerHandler'] = input_event.InputEventHandler(orca._switchToNextPresentationManager, _('Switches to the next presentation manager.'))

    
    def getInputEventHandlerKey(self, inputEventHandler):
        '''Returns the name of the key that contains an inputEventHadler
        passed as argument
        '''
        key = None
        for keyName, handler in self.inputEventHandlers.iteritems():
            if handler._function == inputEventHandler._function:
                key = keyName
                continue
        
        return key

    
    def getListeners(self):
        '''Sets up the AT-SPI event listeners for this script.
        '''
        listeners = script.Script.getListeners(self)
        listeners['focus:'] = self.onFocus
        listeners['object:property-change:accessible-name'] = self.onNameChanged
        listeners['object:text-caret-moved'] = self.onCaretMoved
        listeners['object:text-changed:delete'] = self.onTextDeleted
        listeners['object:text-changed:insert'] = self.onTextInserted
        listeners['object:text-selection-changed'] = self.noOp
        listeners['object:active-descendant-changed'] = self.onActiveDescendantChanged
        listeners['object:children-changed:'] = self.noOp
        listeners['object:link-selected'] = self.onLinkSelected
        listeners['object:bounds-changed'] = self.noOp
        listeners['object:state-changed:'] = self.onStateChanged
        listeners['object:selection-changed'] = self.onSelectionChanged
        listeners['object:property-change:accessible-value'] = self.onValueChanged
        listeners['object:property-change'] = self.noOp
        listeners['object:value-changed:'] = self.onValueChanged
        listeners['object:visible-changed'] = self.noOp
        listeners['window:activate'] = self.onWindowActivated
        listeners['window:create'] = self.noOp
        listeners['window:deactivate'] = self.onWindowDeactivated
        listeners['window:destroy'] = self.noOp
        listeners['window:maximize'] = self.noOp
        listeners['window:minimize'] = self.noOp
        listeners['window:rename'] = self.noOp
        listeners['window:restore'] = self.noOp
        listeners['window:switch'] = self.noOp
        listeners['window:titlelize'] = self.noOp
        return listeners

    
    def __getDesktopBindings(self):
        '''Returns an instance of keybindings.KeyBindings that use the
        numeric keypad for focus tracking and flat review.
        '''
        keyBindings = keybindings.KeyBindings()
        keyBindings.add(keybindings.KeyBinding('KP_Divide', 0, 0, self.inputEventHandlers['leftClickReviewItemHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Multiply', 0, 0, self.inputEventHandlers['rightClickReviewItemHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Subtract', 0, 0, self.inputEventHandlers['toggleFlatReviewModeHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Add', 0, 0, self.inputEventHandlers['sayAllHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Enter', 0, 0, self.inputEventHandlers['whereAmIHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Delete', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['findHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Delete', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_SHIFT, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['findNextHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Delete', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_SHIFT, 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_SHIFT, self.inputEventHandlers['findPreviousHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_7', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewPreviousLineHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Home', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewPreviousLineHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_7', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewHomeHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Home', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewHomeHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_8', 0, 0, self.inputEventHandlers['reviewCurrentLineHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Up', 0, 0, self.inputEventHandlers['reviewCurrentLineHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_9', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewNextLineHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Page_Up', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewNextLineHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_9', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewEndHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Page_Up', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewEndHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_4', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewPreviousItemHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Left', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewPreviousItemHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_4', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewAboveHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Left', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewAboveHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_5', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewCurrentItemHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Begin', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewCurrentItemHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_5', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewCurrentAccessibleHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Begin', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewCurrentAccessibleHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_6', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewNextItemHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Right', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewNextItemHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_6', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewBelowHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Right', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewBelowHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_1', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewPreviousCharacterHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_End', 1 << settings.MODIFIER_ORCA, 0, self.inputEventHandlers['reviewPreviousCharacterHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_1', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewEndOfLineHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_End', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewEndOfLineHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_2', 0, 0, self.inputEventHandlers['reviewCurrentCharacterHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Down', 0, 0, self.inputEventHandlers['reviewCurrentCharacterHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_3', 0, 0, self.inputEventHandlers['reviewNextCharacterHandler']))
        keyBindings.add(keybindings.KeyBinding('KP_Page_Down', 0, 0, self.inputEventHandlers['reviewNextCharacterHandler']))
        return keyBindings

    
    def __getLaptopBindings(self):
        '''Returns an instance of keybindings.KeyBindings that use the
        the main keyboard keys for focus tracking and flat review.
        '''
        keyBindings = keybindings.KeyBindings()
        keyBindings.add(keybindings.KeyBinding('7', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['leftClickReviewItemHandler']))
        keyBindings.add(keybindings.KeyBinding('8', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['rightClickReviewItemHandler']))
        keyBindings.add(keybindings.KeyBinding('p', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['toggleFlatReviewModeHandler']))
        keyBindings.add(keybindings.KeyBinding('semicolon', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['sayAllHandler']))
        keyBindings.add(keybindings.KeyBinding('Return', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['whereAmIHandler']))
        keyBindings.add(keybindings.KeyBinding('slash', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['whereAmIHandler']))
        keyBindings.add(keybindings.KeyBinding('bracketleft', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['findHandler']))
        keyBindings.add(keybindings.KeyBinding('bracketright', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['findNextHandler']))
        keyBindings.add(keybindings.KeyBinding('bracketright', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, self.inputEventHandlers['findPreviousHandler']))
        keyBindings.add(keybindings.KeyBinding('u', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewPreviousLineHandler']))
        keyBindings.add(keybindings.KeyBinding('u', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, self.inputEventHandlers['reviewHomeHandler']))
        keyBindings.add(keybindings.KeyBinding('i', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewCurrentLineHandler']))
        keyBindings.add(keybindings.KeyBinding('o', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewNextLineHandler']))
        keyBindings.add(keybindings.KeyBinding('o', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, self.inputEventHandlers['reviewEndHandler']))
        keyBindings.add(keybindings.KeyBinding('j', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewPreviousItemHandler']))
        keyBindings.add(keybindings.KeyBinding('j', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, self.inputEventHandlers['reviewAboveHandler']))
        keyBindings.add(keybindings.KeyBinding('k', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewCurrentItemHandler']))
        keyBindings.add(keybindings.KeyBinding('k', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, self.inputEventHandlers['reviewCurrentAccessibleHandler']))
        keyBindings.add(keybindings.KeyBinding('l', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewNextItemHandler']))
        keyBindings.add(keybindings.KeyBinding('l', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, self.inputEventHandlers['reviewBelowHandler']))
        keyBindings.add(keybindings.KeyBinding('m', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewPreviousCharacterHandler']))
        keyBindings.add(keybindings.KeyBinding('m', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, self.inputEventHandlers['reviewEndOfLineHandler']))
        keyBindings.add(keybindings.KeyBinding('comma', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewCurrentCharacterHandler']))
        keyBindings.add(keybindings.KeyBinding('period', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reviewNextCharacterHandler']))
        return keyBindings

    
    def getKeyBindings(self):
        '''Defines the key bindings for this script.

        Returns an instance of keybindings.KeyBindings.
        '''
        keyBindings = script.Script.getKeyBindings(self)
        if settings.keyboardLayout == settings.GENERAL_KEYBOARD_LAYOUT_DESKTOP:
            for keyBinding in self._Script__getDesktopBindings().keyBindings:
                keyBindings.add(keyBinding)
            
        else:
            for keyBinding in self._Script__getLaptopBindings().keyBindings:
                keyBindings.add(keyBinding)
            
        keyBindings.add(keybindings.KeyBinding('Num_Lock', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['showZonesHandler']))
        keyBindings.add(keybindings.KeyBinding('F11', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['toggleTableCellReadModeHandler']))
        keyBindings.add(keybindings.KeyBinding('SunF36', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['toggleTableCellReadModeHandler']))
        keyBindings.add(keybindings.KeyBinding('f', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['readCharAttributesHandler']))
        keyBindings.add(keybindings.KeyBinding('F3', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['reportScriptInfoHandler']))
        keyBindings.add(keybindings.KeyBinding('F1', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['enterLearnModeHandler']))
        keyBindings.add(keybindings.KeyBinding('Left', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['decreaseSpeechRateHandler']))
        keyBindings.add(keybindings.KeyBinding('Right', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['increaseSpeechRateHandler']))
        keyBindings.add(keybindings.KeyBinding('Down', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['decreaseSpeechPitchHandler']))
        keyBindings.add(keybindings.KeyBinding('Up', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['increaseSpeechPitchHandler']))
        keyBindings.add(keybindings.KeyBinding('q', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['shutdownHandler']))
        keyBindings.add(keybindings.KeyBinding('Pause', 0, 0, self.inputEventHandlers['keystrokeRecordingHandler']))
        keyBindings.add(keybindings.KeyBinding('space', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['preferencesSettingsHandler']))
        keyBindings.add(keybindings.KeyBinding('space', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, self.inputEventHandlers['loadUserSettingsHandler']))
        keyBindings.add(keybindings.KeyBinding('s', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['toggleSilenceSpeechHandler']))
        keyBindings.add(keybindings.KeyBinding('F5', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['listAppsHandler']))
        keyBindings.add(keybindings.KeyBinding('F4', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['cycleDebugLevelHandler']))
        keyBindings.add(keybindings.KeyBinding('F6', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['printActiveAppHandler']))
        keyBindings.add(keybindings.KeyBinding('F7', 1 << settings.MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['printAncestryHandler']))
        keyBindings.add(keybindings.KeyBinding('F8', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['printHierarchyHandler']))
        keyBindings.add(keybindings.KeyBinding('F10', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['nextPresentationManagerHandler']))
        keyBindings = settings.overrideKeyBindings(self, keyBindings)
        return keyBindings

    
    def getBrailleBindings(self):
        '''Defines the braille bindings for this script.

        Returns a dictionary where the keys are BrlTTY commands and the
        values are InputEventHandler instances.
        '''
        brailleBindings = script.Script.getBrailleBindings(self)
        brailleBindings[braille.CMD_FWINLT] = self.inputEventHandlers['panBrailleLeftHandler']
        brailleBindings[braille.CMD_FWINRT] = self.inputEventHandlers['panBrailleRightHandler']
        brailleBindings[braille.CMD_LNUP] = self.inputEventHandlers['reviewAboveHandler']
        brailleBindings[braille.CMD_LNDN] = self.inputEventHandlers['reviewBelowHandler']
        brailleBindings[braille.CMD_TOP_LEFT] = self.inputEventHandlers['reviewHomeHandler']
        brailleBindings[braille.CMD_BOT_LEFT] = self.inputEventHandlers['reviewBottomLeftHandler']
        brailleBindings[braille.CMD_HOME] = self.inputEventHandlers['goBrailleHomeHandler']
        return brailleBindings

    
    def processKeyboardEvent(self, keyboardEvent):
        '''Processes the given keyboard event. It uses the super
        class equivalent to do most of the work. The only thing done here
        is to detect when the user is trying to get out of learn mode.

        Arguments:
        - keyboardEvent: an instance of input_event.KeyboardEvent
        '''
        if keyboardEvent.type == atspi.Accessibility.KEY_PRESSED_EVENT and keyboardEvent.event_string == 'Escape':
            settings.learnModeEnabled = False
        
        return script.Script.processKeyboardEvent(self, keyboardEvent)

    
    def __sayAllProgressCallback(self, context, type):
        if type == speechserver.SayAllContext.PROGRESS:
            pass
        elif type == speechserver.SayAllContext.INTERRUPTED:
            context.obj.text.setCaretOffset(context.currentOffset)
        elif type == speechserver.SayAllContext.COMPLETED:
            orca.setLocusOfFocus(None, context.obj, False)
            context.obj.text.setCaretOffset(context.currentOffset)
        

    
    def sayAll(self, inputEvent):
        if not orca_state.locusOfFocus:
            pass
        elif orca_state.locusOfFocus.text:
            speech.sayAll(util.textLines(orca_state.locusOfFocus), self._Script__sayAllProgressCallback)
        else:
            speech.speakUtterances(self.speechGenerator.getSpeech(orca_state.locusOfFocus, False))
        return True

    
    def isTextArea(self, obj):
        '''Returns True if obj is a GUI component that is for entering text.

        Arguments:
        - obj: an accessible
        '''
        if not obj and obj.role and obj.role == rolenames.ROLE_TEXT:
            pass
        return obj.role == rolenames.ROLE_PARAGRAPH

    
    def getText(self, obj, startOffset, endOffset):
        """Returns the substring of the given object's text specialization.

        Arguments:
        - obj: an accessible supporting the accessible text specialization
        - startOffset: the starting character position
        - endOffset: the ending character position
        """
        return obj.text.getText(startOffset, endOffset)

    
    def sayPhrase(self, obj, startOffset, endOffset):
        """Speaks the text of an Accessible object between the start and
        end offsets, unless the phrase is empty in which case it's ignored.

        Arguments:
        - obj: an Accessible object that implements the AccessibleText
               interface
        - startOffset: the start text offset.
        - endOffset: the end text offset.
        """
        if startOffset > endOffset or endOffset != -1 or startOffset == -1:
            temp = endOffset
            endOffset = startOffset
            startOffset = temp
        
        phrase = self.getText(obj, startOffset, endOffset)
        if len(phrase):
            if phrase.isupper():
                voice = self.voices[settings.UPPERCASE_VOICE]
            else:
                voice = self.voices[settings.DEFAULT_VOICE]
            phrase = util.adjustForRepeats(phrase)
            speech.speak(phrase, voice)
            util.speakTextSelectionState(obj, startOffset, endOffset)
        

    
    def sayLine(self, obj):
        """Speaks the line of an AccessibleText object that contains the
        caret, unless the line is empty in which case it's ignored.

        Arguments:
        - obj: an Accessible object that implements the AccessibleText
               interface
        """
        (line, caretOffset, startOffset) = util.getTextLineAtCaret(obj)
        debug.println(debug.LEVEL_FINEST, 'sayLine: line=<%s>, len=%d, start=%d, caret=%d, speakBlankLines=%s' % (line, len(line), startOffset, caretOffset, settings.speakBlankLines))
        if len(line):
            if line.isupper():
                voice = self.voices[settings.UPPERCASE_VOICE]
            else:
                voice = self.voices[settings.DEFAULT_VOICE]
            if settings.enableSpeechIndentation:
                self.speakTextIndentation(obj, line)
            
            line = util.adjustForRepeats(line)
            speech.speak(line, voice)
            util.speakTextSelectionState(obj, startOffset, caretOffset)
        else:
            char = obj.text.getTextAtOffset(caretOffset, atspi.Accessibility.TEXT_BOUNDARY_CHAR)
            debug.println(debug.LEVEL_FINEST, 'sayLine: character=<%s>, start=%d, end=%d' % (char[0], char[1], char[2]))
            if char[0] == '\n' and startOffset == caretOffset and settings.speakBlankLines:
                speech.speak(_('blank'))
            

    
    def sayWord(self, obj):
        '''Speaks the word at the caret.  [[[TODO: WDW - what if there is no
        word at the caret?]]]

        Arguments:
        - obj: an Accessible object that implements the AccessibleText
               interface
        '''
        text = obj.text
        offset = text.caretOffset
        lastKey = orca_state.lastInputEvent.event_string
        lastWord = orca_state.lastWord
        (word, startOffset, endOffset) = text.getTextAtOffset(offset, atspi.Accessibility.TEXT_BOUNDARY_WORD_START)
        if lastKey == 'Right' and len(lastWord) > 0:
            lastChar = lastWord[len(lastWord) - 1]
            if lastChar == '\n' and lastWord != word:
                voice = self.voices[settings.DEFAULT_VOICE]
                speech.speak(chnames.getCharacterName('\n'), voice, False)
            
        
        if lastKey == 'Left' and len(word) > 0:
            lastChar = word[len(word) - 1]
            if lastChar == '\n' and lastWord != word:
                voice = self.voices[settings.DEFAULT_VOICE]
                speech.speak(chnames.getCharacterName('\n'), voice, False)
            
        
        if util.getLinkIndex(obj, offset) >= 0:
            voice = self.voices[settings.HYPERLINK_VOICE]
        elif word.isupper():
            voice = self.voices[settings.UPPERCASE_VOICE]
        else:
            voice = self.voices[settings.DEFAULT_VOICE]
        word = util.adjustForRepeats(word)
        orca_state.lastWord = word
        speech.speak(word, voice)
        util.speakTextSelectionState(obj, startOffset, endOffset)

    
    def speakTextIndentation(self, obj, line):
        '''Speaks a summary of the number of spaces and/or tabs at the
        beginning of the given line.

        Arguments:
        - obj: the text object.
        - line: the string to check for spaces and tabs.
        '''
        line = line.replace('\xc2\xa0', ' ')
        line = line.decode('UTF-8')
        spaceCount = 0
        tabCount = 0
        for offset in range(0, len(line)):
            if line[offset] == ' ':
                spaceCount += 1
                continue
            if line[offset] == '\t':
                tabCount += 1
                continue
        
        utterance = ''
        if spaceCount:
            if spaceCount == 1:
                utterance += _('1 space ')
            else:
                utterance += _('%d spaces ') % spaceCount
        
        if tabCount:
            if tabCount == 1:
                utterance += _('1 tab ')
            else:
                utterance += _('%d tabs ') % tabCount
        
        if len(utterance):
            speech.speak(utterance)
        

    
    def echoPreviousWord(self, obj):
        """Speaks the word prior to the caret, as long as there is
        a word prior to the caret and there is no intervening word
        delimiter between the caret and the end of the word.

        The entry condition for this method is that the character
        prior to the current caret position is a word delimiter,
        and it's what caused this method to be called in the first
        place.

        Arguments:
        - obj: an Accessible object that implements the AccessibleText
               interface.
        """
        text = obj.text
        if not text:
            return None
        
        offset = text.caretOffset - 1
        if offset < 0:
            return None
        
        (char, startOffset, endOffset) = text.getTextAtOffset(offset, atspi.Accessibility.TEXT_BOUNDARY_CHAR)
        if not util.isWordDelimiter(char):
            return None
        
        wordEndOffset = text.caretOffset - 2
        wordStartOffset = wordEndOffset
        while wordStartOffset >= 0:
            (char, startOffset, endOffset) = text.getTextAtOffset(wordStartOffset, atspi.Accessibility.TEXT_BOUNDARY_CHAR)
            if util.isWordDelimiter(char):
                break
                continue
            wordStartOffset -= 1
        if wordStartOffset == wordEndOffset:
            return None
        else:
            word = self.getText(obj, wordStartOffset + 1, wordEndOffset + 1)
        if util.getLinkIndex(obj, wordStartOffset + 1) >= 0:
            voice = self.voices[settings.HYPERLINK_VOICE]
        elif word.isupper():
            voice = self.voices[settings.UPPERCASE_VOICE]
        else:
            voice = self.voices[settings.DEFAULT_VOICE]
        word = util.adjustForRepeats(word)
        speech.speak(word, voice)

    
    def sayCharacter(self, obj):
        """Speak the character under the caret.  [[[TODO: WDW - isn't the
        caret between characters?]]]

        Arguments:
        - obj: an Accessible object that implements the AccessibleText
               interface
        """
        text = obj.text
        offset = text.caretOffset
        mods = orca_state.lastInputEvent.modifiers
        shiftMask = 1 << atspi.Accessibility.MODIFIER_SHIFT
        if mods & shiftMask and orca_state.lastInputEvent.event_string == 'Right':
            startOffset = offset - 1
            endOffset = offset
        else:
            startOffset = offset
            endOffset = offset + 1
        if endOffset > text.characterCount:
            character = '\n'
        else:
            character = self.getText(obj, startOffset, endOffset)
        if util.getLinkIndex(obj, offset) >= 0:
            voice = self.voices[settings.HYPERLINK_VOICE]
        elif character.isupper():
            voice = self.voices[settings.UPPERCASE_VOICE]
        else:
            voice = self.voices[settings.DEFAULT_VOICE]
        prevChar = self.getText(obj, startOffset - 1, endOffset - 1)
        debug.println(debug.LEVEL_FINEST, 'sayCharacter: prev=<%s>, char=<%s>, startOffset=%d, caretOffset=%d, endOffset=%d, speakBlankLines=%s' % (prevChar, character, startOffset, offset, endOffset, settings.speakBlankLines))
        if orca_state.lastInputEvent.event_string == 'Right':
            if prevChar == '\n':
                speech.speak(chnames.getCharacterName('\n'), voice, False)
            
        elif orca_state.lastInputEvent.event_string == 'Left':
            if character == '\n':
                speech.speak(chnames.getCharacterName('\n'), voice, False)
            
        
        if character == '\n':
            if prevChar == '\n':
                if settings.speakBlankLines:
                    speech.speak(_('blank'), voice, False)
                
            
        else:
            speech.speak(character, voice, False)
        util.speakTextSelectionState(obj, startOffset, endOffset)

    
    def whereAmI(self, inputEvent):
        '''
        Speaks information about the current object of interest.
        '''
        obj = orca_state.locusOfFocus
        self.updateBraille(obj)
        if inputEvent and orca_state.lastInputEvent and isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
            string = atspi.KeystrokeListener.keyEventToString(inputEvent)
            debug.println(debug.LEVEL_FINEST, 'default.whereAmI: %s' % string)
        else:
            context = self.speechGenerator.getSpeechContext(obj)
            return where_am_I.whereAmI(obj, context, False, False)
        orcaKey = False
        if settings.keyboardLayout == settings.GENERAL_KEYBOARD_LAYOUT_DESKTOP:
            orcaKey = inputEvent.modifiers & 1 << settings.MODIFIER_ORCA == 1 << settings.MODIFIER_ORCA
        else:
            orcaKey = inputEvent.event_string == '/'
        doubleClick = util.getClickCount(self.lastWhereAmIEvent, inputEvent) == 2
        self.lastWhereAmIEvent = inputEvent
        context = self.speechGenerator.getSpeechContext(obj)
        return where_am_I.whereAmI(obj, context, doubleClick, orcaKey)

    
    def findCommonAncestor(self, a, b):
        '''Finds the common ancestor between Accessible a and Accessible b.

        Arguments:
        - a: Accessible
        - b: Accessible
        '''
        debug.println(debug.LEVEL_FINEST, 'default.findCommonAncestor...')
        if not a or not b:
            return None
        
        if a == b:
            return a
        
        aParents = [
            a]
        
        try:
            parent = a.parent
            while parent and parent.parent != parent:
                aParents.append(parent)
                parent = parent.parent
            aParents.reverse()
        except:
            debug.printException(debug.LEVEL_FINEST)

        bParents = [
            b]
        
        try:
            parent = b.parent
            while parent and parent.parent != parent:
                bParents.append(parent)
                parent = parent.parent
            bParents.reverse()
        except:
            debug.printException(debug.LEVEL_FINEST)

        commonAncestor = None
        maxSearch = min(len(aParents), len(bParents))
        i = 0
        while i < maxSearch:
            if aParents[i] == bParents[i]:
                commonAncestor = aParents[i]
                i += 1
                continue
            break
        debug.println(debug.LEVEL_FINEST, '...default.findCommonAncestor')
        return commonAncestor

    
    def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
        '''Called when the visual object with focus changes.

        Arguments:
        - event: if not None, the Event that caused the change
        - oldLocusOfFocus: Accessible that is the old locus of focus
        - newLocusOfFocus: Accessible that is the new locus of focus
        '''
        
        try:
            if self.findCommandRun:
                return None
        except:
            pass

        if newLocusOfFocus:
            mag.magnifyAccessible(event, newLocusOfFocus)
        
        if self.flatReviewContext:
            self.toggleFlatReviewMode()
        
        if util.isSameObject(oldLocusOfFocus, newLocusOfFocus):
            return None
        
        if oldLocusOfFocus:
            oldParent = oldLocusOfFocus.parent
        else:
            oldParent = None
        if newLocusOfFocus:
            newParent = newLocusOfFocus.parent
        else:
            newParent = None
        if newLocusOfFocus:
            self.updateBraille(newLocusOfFocus)
            utterances = []
            commonAncestor = self.findCommonAncestor(oldLocusOfFocus, newLocusOfFocus)
            if commonAncestor:
                context = self.speechGenerator.getSpeechContext(newLocusOfFocus, commonAncestor)
                utterances.append(' '.join(context))
            
            oldNodeLevel = -1
            newNodeLevel = -1
            if newLocusOfFocus.role == rolenames.ROLE_TABLE_CELL:
                if oldParent and oldParent.table:
                    table = oldParent.table
                    oldRow = table.getRowAtIndex(oldLocusOfFocus.index)
                    oldCol = table.getColumnAtIndex(oldLocusOfFocus.index)
                else:
                    oldRow = -1
                    oldCol = -1
                if newParent and newParent.table:
                    table = newParent.table
                    newRow = table.getRowAtIndex(newLocusOfFocus.index)
                    newCol = table.getColumnAtIndex(newLocusOfFocus.index)
                    if newRow != oldRow or oldParent != newParent:
                        desc = newParent.table.getRowDescription(newRow)
                        if desc and len(desc):
                            text = desc
                            if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
                                text += ' ' + rolenames.rolenames[rolenames.ROLE_ROW_HEADER].speech
                            
                            utterances.append(text)
                        
                    
                    if newCol != oldCol or oldParent != newParent:
                        topName = util.getTopLevelName(newLocusOfFocus)
                        if not topName.endswith(' - Thunderbird'):
                            desc = newParent.table.getColumnDescription(newCol)
                            if desc and len(desc):
                                text = desc
                                if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
                                    text += ' ' + rolenames.rolenames[rolenames.ROLE_COLUMN_HEADER].speech
                                
                                utterances.append(text)
                            
                        
                    
                
            
            oldNodeLevel = util.getNodeLevel(oldLocusOfFocus)
            newNodeLevel = util.getNodeLevel(newLocusOfFocus)
            if newLocusOfFocus.role == rolenames.ROLE_RADIO_BUTTON:
                radioGroupLabel = None
                inSameGroup = False
                relations = newLocusOfFocus.relations
                for relation in relations:
                    if not radioGroupLabel and relation.getRelationType() == atspi.Accessibility.RELATION_LABELLED_BY:
                        radioGroupLabel = atspi.Accessible.makeAccessible(relation.getTarget(0))
                    
                    if not inSameGroup and relation.getRelationType() == atspi.Accessibility.RELATION_MEMBER_OF:
                        for i in range(0, relation.getNTargets()):
                            target = atspi.Accessible.makeAccessible(relation.getTarget(i))
                            if target == oldLocusOfFocus:
                                inSameGroup = True
                                break
                                continue
                        
                
                if not inSameGroup and radioGroupLabel:
                    utterances.append(util.getDisplayedText(radioGroupLabel))
                
            
            utterances.extend(self.speechGenerator.getSpeech(newLocusOfFocus, False))
            if oldNodeLevel != newNodeLevel and newNodeLevel >= 0:
                utterances.append(_('tree level %d') % (newNodeLevel + 1))
            
            if event and event.type == 'focus:' and self.windowActivateTime:
                pass
            shouldNotInterrupt = time.time() - self.windowActivateTime < 1.0
            if newLocusOfFocus.role == rolenames.ROLE_LINK:
                voice = self.voices[settings.HYPERLINK_VOICE]
            else:
                voice = self.voices[settings.DEFAULT_VOICE]
            speech.speakUtterances(utterances, voice, not shouldNotInterrupt)
            if newLocusOfFocus.role == rolenames.ROLE_TABLE_CELL:
                if newParent and newParent.table:
                    table = newParent.table
                    column = table.getColumnAtIndex(newLocusOfFocus.index)
                    newParent.lastColumn = column
                    row = table.getRowAtIndex(newLocusOfFocus.index)
                    newParent.lastRow = row
                
            
        else:
            orca_state.noFocusTimeStamp = time.time()

    
    def visualAppearanceChanged(self, event, obj):
        """Called when the visual appearance of an object changes.  This
        method should not be called for objects whose visual appearance
        changes solely because of focus -- setLocusOfFocus is used for that.
        Instead, it is intended mostly for objects whose notional 'value' has
        changed, such as a checkbox changing state, a progress bar advancing,
        a slider moving, text inserted, caret moved, etc.

        Arguments:
        - event: if not None, the Event that caused this to happen
        - obj: the Accessible whose visual appearance changed.
        """
        if False and obj.role == rolenames.ROLE_PANEL and event.detail1 == 1 and util.isInActiveApp(obj):
            reallyShowing = True
            parent = obj.parent
            while reallyShowing and parent and parent != parent.parent and parent.role != rolenames.ROLE_APPLICATION:
                debug.println(debug.LEVEL_FINEST, 'default.visualAppearanceChanged - ' + 'checking parent')
                reallyShowing = parent.state.count(atspi.Accessibility.STATE_SHOWING)
                parent = parent.parent
            if reallyShowing:
                utterances = []
                labels = util.findUnrelatedLabels(obj)
                for label in labels:
                    utterances.append(label.name)
                
                speech.speakUtterances(utterances)
                return None
            
        
        relations = obj.relations
        for relation in relations:
            if relation.getRelationType() == atspi.Accessibility.RELATION_CONTROLLED_BY:
                target = atspi.Accessible.makeAccessible(relation.getTarget(0))
                if target == orca_state.locusOfFocus:
                    self.updateBraille(target)
                    speech.speakUtterances(self.speechGenerator.getSpeech(target, True))
                    return None
                
            target == orca_state.locusOfFocus
        
        if obj.role == rolenames.ROLE_LABEL:
            for relation in relations:
                if relation.getRelationType() == atspi.Accessibility.RELATION_LABEL_FOR:
                    target = atspi.Accessible.makeAccessible(relation.getTarget(0))
                    if target == orca_state.locusOfFocus:
                        self.updateBraille(target)
                        speech.speakUtterances(self.speechGenerator.getSpeech(target, True))
                        return None
                    
                target == orca_state.locusOfFocus
            
        
        if obj != orca_state.locusOfFocus:
            return None
        
        if event:
            debug.println(debug.LEVEL_FINE, "VISUAL CHANGE: '%s' '%s' (event='%s')" % (obj.name, obj.role, event.type))
        else:
            debug.println(debug.LEVEL_FINE, "VISUAL CHANGE: '%s' '%s' (event=None)" % (obj.name, obj.role))
        mag.magnifyAccessible(event, obj)
        self.updateBraille(obj)
        speech.speakUtterances(self.speechGenerator.getSpeech(obj, True))

    
    def updateBraille(self, obj, extraRegion = None):
        '''Updates the braille display to show the give object.

        Arguments:
        - obj: the Accessible
        - extra: extra Region to add to the end
        '''
        if not obj:
            return None
        
        braille.clear()
        line = braille.Line()
        braille.addLine(line)
        if obj.text and self.isTextArea(obj):
            text = obj.text
            (string, startOffset, endOffset) = text.getTextAtOffset(text.caretOffset, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
            if startOffset == 0:
                line.addRegions(self.brailleGenerator.getBrailleContext(obj))
            
        else:
            line.addRegions(self.brailleGenerator.getBrailleContext(obj))
        result = self.brailleGenerator.getBrailleRegions(obj)
        line.addRegions(result[0])
        if extraRegion:
            line.addRegion(extraRegion)
        
        if extraRegion:
            braille.setFocus(extraRegion)
        else:
            braille.setFocus(result[1])
        braille.refresh(True)

    
    def onFocus(self, event):
        '''Called whenever an object gets focus.

        Arguments:
        - event: the Event
        '''
        role = event.source.role
        if role == rolenames.ROLE_MENU and role == rolenames.ROLE_MENU_ITEM and role == rolenames.ROLE_CHECK_MENU_ITEM or role == rolenames.ROLE_RADIO_MENU_ITEM:
            selection = event.source.selection
            if selection and selection.nSelectedChildren > 0:
                return None
            
        
        newFocus = event.source
        if event.source.role == rolenames.ROLE_LAYERED_PANE and event.source.role == rolenames.ROLE_TABLE and event.source.role == rolenames.ROLE_TREE_TABLE or event.source.role == rolenames.ROLE_TREE:
            if event.source.childCount:
                selection = event.source.selection
                if selection and selection.nSelectedChildren > 0:
                    newFocus = atspi.Accessible.makeAccessible(selection.getSelectedChild(0))
                elif event.source.__dict__.has_key('activeDescendantInfo'):
                    (parent, index) = event.source.activeDescendantInfo
                    newFocus = parent.child(index)
                
            
        
        orca.setLocusOfFocus(event, newFocus)

    
    def onNameChanged(self, event):
        '''Called whenever a property on an object changes.

        Arguments:
        - event: the Event
        '''
        if event.source and event.source.role == rolenames.ROLE_DIALOG and event.source == orca_state.locusOfFocus:
            return None
        
        
        try:
            if event.source.oldName == event.source.name:
                return None
        except:
            pass

        event.source.oldName = event.source.name
        orca.visualAppearanceChanged(event, event.source)

    
    def _presentTextAtNewCaretPosition(self, event):
        if event.source:
            mag.magnifyAccessible(event, event.source)
        
        brailleNeedsRepainting = True
        line = braille.getShowingLine()
        for region in line.regions:
            if isinstance(region, braille.Text) and region.accessible == event.source:
                if region.repositionCursor():
                    braille.refresh(True)
                    brailleNeedsRepainting = False
                
                break
                continue
        
        if brailleNeedsRepainting:
            self.updateBraille(event.source)
        
        if not orca_state.lastInputEvent:
            return None
        
        if isinstance(orca_state.lastInputEvent, input_event.MouseButtonEvent):
            if not orca_state.lastInputEvent.pressed:
                self.sayLine(event.source)
            
            return None
        
        if not isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
            return None
        
        string = orca_state.lastInputEvent.event_string
        mods = orca_state.lastInputEvent.modifiers
        isControlKey = mods & 1 << atspi.Accessibility.MODIFIER_CONTROL
        isShiftKey = mods & 1 << atspi.Accessibility.MODIFIER_SHIFT
        hasLastPos = event.source.__dict__.has_key('lastCursorPosition')
        if string == 'Up' or string == 'Down':
            if hasLastPos and isShiftKey and not isControlKey:
                self.sayPhrase(event.source, event.source.lastCursorPosition, event.source.text.caretOffset)
            else:
                self.sayLine(event.source)
        elif string == 'Left' or string == 'Right':
            if hasLastPos and isShiftKey and isControlKey:
                self.sayPhrase(event.source, event.source.lastCursorPosition, event.source.text.caretOffset)
            elif isControlKey:
                self.sayWord(event.source)
            else:
                self.sayCharacter(event.source)
        elif string == 'Page_Up':
            if hasLastPos and isShiftKey and isControlKey:
                self.sayPhrase(event.source, event.source.lastCursorPosition, event.source.text.caretOffset)
            elif isControlKey:
                self.sayCharacter(event.source)
            else:
                self.sayLine(event.source)
        elif string == 'Page_Down':
            if hasLastPos and isShiftKey and isControlKey:
                self.sayPhrase(event.source, event.source.lastCursorPosition, event.source.text.caretOffset)
            else:
                self.sayLine(event.source)
        elif string == 'Home' or string == 'End':
            if hasLastPos and isShiftKey and not isControlKey:
                self.sayPhrase(event.source, event.source.lastCursorPosition, event.source.text.caretOffset)
            elif isControlKey:
                self.sayLine(event.source)
            else:
                self.sayCharacter(event.source)
        elif string == 'A' and isControlKey:
            text = event.source.text
            charCount = text.characterCount
            for i in range(0, text.getNSelections()):
                (startSelOffset, endSelOffset) = text.getSelection(i)
                if text.caretOffset == 0 and startSelOffset == 0 and endSelOffset == charCount:
                    speech.speak(_('entire document selected'))
                    continue
            
        

    
    def onCaretMoved(self, event):
        '''Called whenever the caret moves.

        Arguments:
        - event: the Event
        '''
        if event and event.source and event.source != orca_state.locusOfFocus and event.source.state.count(atspi.Accessibility.STATE_FOCUSED):
            orca.setLocusOfFocus(event, event.source, False)
        
        if event.source != orca_state.locusOfFocus and event.source.parent != orca_state.locusOfFocus:
            return None
        
        if self.flatReviewContext:
            self.toggleFlatReviewMode()
        
        self._presentTextAtNewCaretPosition(event)

    
    def onTextDeleted(self, event):
        '''Called whenever text is deleted from an object.

        Arguments:
        - event: the Event
        '''
        if event and event.source and event.source != orca_state.locusOfFocus and event.source.state.count(atspi.Accessibility.STATE_FOCUSED):
            orca.setLocusOfFocus(event, event.source, False)
        
        if event.source != orca_state.locusOfFocus and event.source.parent != orca_state.locusOfFocus:
            return None
        
        if event.source.role == rolenames.ROLE_SLIDER:
            return None
        
        self.updateBraille(event.source)
        if not (orca_state.lastInputEvent) or not isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
            return None
        
        string = orca_state.lastInputEvent.event_string
        text = event.source.text
        if string == 'BackSpace':
            character = event.any_data
        elif string == 'Delete':
            offset = text.caretOffset
            (character, startOffset, endOffset) = event.source.text.getTextAtOffset(offset, atspi.Accessibility.TEXT_BOUNDARY_CHAR)
        else:
            return None
        if util.getLinkIndex(event.source, text.caretOffset) >= 0:
            voice = self.voices[settings.HYPERLINK_VOICE]
        elif character.isupper():
            voice = self.voices[settings.UPPERCASE_VOICE]
        else:
            voice = self.voices[settings.DEFAULT_VOICE]
        speech.speak(character, voice, False)

    
    def onTextInserted(self, event):
        '''Called whenever text is inserted into an object.

        Arguments:
        - event: the Event
        '''
        if event and event.source and event.source != orca_state.locusOfFocus and event.source.state.count(atspi.Accessibility.STATE_FOCUSED):
            orca.setLocusOfFocus(event, event.source, False)
        
        if event.source != orca_state.locusOfFocus and event.source.parent != orca_state.locusOfFocus:
            return None
        
        if event.source.role == rolenames.ROLE_SLIDER:
            return None
        
        self.updateBraille(event.source)
        text = event.any_data
        if event.source.role == rolenames.ROLE_SPIN_BUTTON:
            speech.speak(text)
            return None
        
        if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
            keyString = orca_state.lastInputEvent.event_string
            if event.source.role == rolenames.ROLE_TEXT:
                pass
            wasAutoComplete = event.source.text.getNSelections()
            wasCommand = orca_state.lastInputEvent.modifiers & (1 << atspi.Accessibility.MODIFIER_CONTROL | 1 << atspi.Accessibility.MODIFIER_ALT | 1 << atspi.Accessibility.MODIFIER_META | 1 << atspi.Accessibility.MODIFIER_META2 | 1 << atspi.Accessibility.MODIFIER_META3)
            if text == ' ' or keyString == 'space' or text == keyString:
                pass
            elif wasCommand and wasAutoComplete or event.source.role == rolenames.ROLE_PASSWORD_TEXT:
                if text.isupper():
                    speech.speak(text, self.voices[settings.UPPERCASE_VOICE])
                else:
                    speech.speak(text)
            
        
        if settings.enableEchoByWord and util.isWordDelimiter(text.decode('UTF-8')[-1:]):
            self.echoPreviousWord(event.source)
        

    
    def onActiveDescendantChanged(self, event):
        '''Called when an object who manages its own descendants detects a
        change in one of its children.

        Arguments:
        - event: the Event
        '''
        if not event.source.state.count(atspi.Accessibility.STATE_FOCUSED):
            return None
        
        child = event.any_data
        if child:
            orca.setLocusOfFocus(event, child)
        else:
            orca.setLocusOfFocus(event, event.source)
        if orca_state.locusOfFocus and orca_state.locusOfFocus != event.source:
            event.source.activeDescendantInfo = [
                orca_state.locusOfFocus.parent,
                orca_state.locusOfFocus.index]
        elif event.source.__dict__.has_key('activeDescendantInfo'):
            del event.source.__dict__['activeDescendantInfo']
        

    
    def onLinkSelected(self, event):
        '''Called when a hyperlink is selected in a text area.

        Arguments:
        - event: the Event
        '''
        orca.setLocusOfFocus(event, event.source)

    
    def onStateChanged(self, event):
        """Called whenever an object's state changes.

        Arguments:
        - event: the Event
        """
        if event.type == 'object:state-changed:active':
            if self.findCommandRun:
                self.findCommandRun = False
                self.find()
                return None
            
        
        if event.type == 'object:state-changed:focused':
            iconified = False
            
            try:
                window = util.getTopLevel(event.source)
                iconified = window.state.count(atspi.Accessibility.STATE_ICONIFIED)
            except:
                debug.println(debug.LEVEL_FINEST, 'onStateChanged: could not get frame of focused item')

            if not iconified:
                if event.detail1:
                    self.onFocus(event)
                
                return None
            
        
        if state_change_notifiers.has_key(event.source.role):
            notifiers = state_change_notifiers[event.source.role]
            found = False
            for state in notifiers:
                if state and event.type.endswith(state):
                    found = True
                    break
                    continue
            
            if found:
                orca.visualAppearanceChanged(event, event.source)
            
        

    
    def onSelectionChanged(self, event):
        """Called when an object's selection changes.

        Arguments:
        - event: the Event
        """
        if not event or not (event.source):
            return None
        
        if event.source.state.count(atspi.Accessibility.STATE_MANAGES_DESCENDANTS):
            return None
        
        if event.source.role == rolenames.ROLE_COMBO_BOX:
            orca.visualAppearanceChanged(event, event.source)
        elif event.source != orca_state.locusOfFocus and event.source.state.count(atspi.Accessibility.STATE_FOCUSED):
            newFocus = event.source
            if event.source.childCount:
                selection = event.source.selection
                if selection and selection.nSelectedChildren > 0:
                    child = selection.getSelectedChild(0)
                    if child:
                        newFocus = atspi.Accessible.makeAccessible(child)
                    
                
            
            orca.setLocusOfFocus(event, newFocus)
        

    
    def onValueChanged(self, event):
        """Called whenever an object's value changes.  Currently, the
        value changes for non-focused objects are ignored.

        Arguments:
        - event: the Event
        """
        if event.source.role == rolenames.ROLE_SPIN_BUTTON:
            return None
        
        if event.source.value and event.source.__dict__.has_key('oldValue') and event.source.value.currentValue == event.source.oldValue:
            return None
        
        orca.visualAppearanceChanged(event, event.source)
        event.source.oldValue = event.source.value.currentValue

    
    def onWindowActivated(self, event):
        '''Called whenever a toplevel window is activated.

        Arguments:
        - event: the Event
        '''
        self.windowActivateTime = time.time()
        orca.setLocusOfFocus(event, event.source)
        orca_state.activeWindow = event.source

    
    def onWindowDeactivated(self, event):
        '''Called whenever a toplevel window is deactivated.

        Arguments:
        - event: the Event
        '''
        if orca_state.locusOfFocus and orca_state.locusOfFocus.app == event.source.app:
            speech.stop()
        
        if event.source == orca_state.activeWindow:
            orca.setLocusOfFocus(event, None)
            orca_state.activeWindow = None
        

    
    def noOp(self, event):
        '''Just here to capture events.

        Arguments:
        - event: the Event
        '''
        pass

    
    def isLayoutOnly(self, obj):
        '''Returns True if the given object is a table and is for layout
        purposes only.'''
        layoutOnly = False
        if obj and obj.role == rolenames.ROLE_TABLE and obj.attributes:
            for attribute in obj.attributes:
                if attribute == 'layout-guess:true':
                    layoutOnly = True
                    break
                    continue
            
        elif obj and obj.role == rolenames.ROLE_PANEL:
            text = util.getDisplayedText(obj)
            label = util.getDisplayedLabel(obj)
            if not (label or len(label)) and text and len(text):
                layoutOnly = True
            
        
        if layoutOnly:
            debug.println(debug.LEVEL_FINEST, 'Object deemed to be for layout purposes only: ' + obj.toString('', True))
        
        return layoutOnly

    
    def toggleTableCellReadMode(self, inputEvent = None):
        '''Toggles an indicator for whether we should just read the current
        table cell or read the whole row.'''
        line = _('Speak ')
        settings.readTableCellRow = not (settings.readTableCellRow)
        if settings.readTableCellRow:
            line += _('row')
        else:
            line += _('cell')
        speech.speak(line)
        return True

    
    def textAttrsToDictionary(self, str):
        """Converts a string of text attribute tokens of the form
        <key>:<value>; into a dictionary of keys and values.
        Text before the colon is the key and text afterwards is the
        value. If there is a final semi-colon, then it's ignored.

        Arguments:
        - str: the string of tokens containing <key>:<value>; pairs.

        Returns a list containing two items:
        A list of the keys in the order they were extracted from the
        text attribute string and a dictionary of key/value items.
        """
        list = []
        dictionary = { }
        allTokens = str.split(';')
        for i in range(0, len(allTokens)):
            item = allTokens[i].split(':')
            if len(item) == 2:
                key = item[0].strip()
                attribute = item[1].strip()
                list.append(key)
                dictionary[key] = attribute
                continue
        
        return [
            list,
            dictionary]

    
    def outputCharAttributes(self, keys, attributes):
        '''Speak each of the text attributes given dictionary.

        Arguments:
        - attributes: a dictionary of text attributes to speak.
        '''
        for i in range(0, len(keys)):
            key = keys[i]
            if attributes.has_key(key):
                attribute = attributes[key]
                if attribute:
                    if key == 'weight' and int(attribute) > 400:
                        line = _('bold')
                    elif key == 'left-margin' or key == 'right-margin':
                        line = key + ' ' + attribute + _(' pixels')
                    else:
                        line = key + ' ' + attribute
                    speech.speak(line)
                
            attribute
        

    
    def readCharAttributes(self, inputEvent = None):
        '''Reads the attributes associated with the current text character.
        Calls outCharAttributes to speak a list of attributes. By default,
        a certain set of attributes will be spoken. If this is not desired,
        then individual application scripts should override this method to
        only speak the subset required.
        '''
        if orca_state.locusOfFocus and orca_state.locusOfFocus.text:
            caretOffset = orca_state.locusOfFocus.text.caretOffset
            text = orca_state.locusOfFocus.text
            defAttributes = text.getDefaultAttributes()
            debug.println(debug.LEVEL_FINEST, 'readCharAttributes: default text attributes: %s' % defAttributes)
            (defUser, defDict) = self.textAttrsToDictionary(defAttributes)
            allAttributes = defDict
            charAttributes = text.getAttributes(caretOffset)
            if charAttributes[0]:
                (charList, charDict) = self.textAttrsToDictionary(charAttributes[0])
                if allAttributes:
                    for key in charDict.keys():
                        allAttributes[key] = charDict[key]
                    
                else:
                    allAttributes = charDict
            
            (userAttrList, userAttrDict) = self.textAttrsToDictionary(settings.enabledTextAttributes)
            attributes = { }
            for i in range(0, len(userAttrList)):
                key = userAttrList[i]
                if allAttributes.has_key(key):
                    textAttr = allAttributes.get(key)
                    userAttr = userAttrDict.get(key)
                    if textAttr != userAttr or len(userAttr) == 0:
                        attributes[key] = textAttr
                    
                len(userAttr) == 0
            
            self.outputCharAttributes(userAttrList, attributes)
        
        return True

    
    def reportScriptInfo(self, inputEvent = None):
        '''Output useful information on the current script via speech
        and braille.  This information will be helpful to script writers.
        '''
        string = "SCRIPT INFO: Script name='%s'" % self.name
        if orca_state.locusOfFocus and orca_state.locusOfFocus.app:
            string += " Application name='%s'" % orca_state.locusOfFocus.app.name
            
            try:
                string += " Toolkit name='%s'" % orca_state.locusOfFocus.app.toolkitName
            except:
                string += ' Toolkit unknown'

            
            try:
                string += " Version='%s'" % orca_state.locusOfFocus.app.version
            except:
                string += ' Version unknown'

            debug.println(debug.LEVEL_INFO, string)
            speech.speak(string)
            braille.displayMessage(string)
        
        return True

    
    def enterLearnMode(self, inputEvent = None):
        '''Turns learn mode on.  The user must press the escape key to exit
        learn mode.

        Returns True to indicate the input event has been consumed.
        '''
        if settings.learnModeEnabled:
            return True
        
        self.exitLearnModeKeyBinding = keybindings.KeyBinding('Escape', 0, 0, self.inputEventHandlers['exitLearnModeHandler'])
        self.keyBindings.add(self.exitLearnModeKeyBinding)
        speech.speak(_('Entering learn mode.  Press any key to hear its function. ' + 'To exit learn mode, press the escape key.'))
        braille.displayMessage(_('Learn mode.  Press escape to exit.'))
        settings.learnModeEnabled = True
        return True

    
    def exitLearnMode(self, inputEvent = None):
        '''Turns learn mode off.

        Returns True to indicate the input event has been consumed.
        '''
        self.keyBindings.remove(self.exitLearnModeKeyBinding)
        speech.speak(_('Exiting learn mode.'))
        braille.displayMessage(_('Exiting learn mode.'))
        self.whereAmI(None)
        return True

    
    def pursueForFlatReview(self, obj):
        '''Determines if we should look any further at the object
        for flat review.'''
        return obj.state.count(atspi.Accessibility.STATE_SHOWING)

    
    def getFlatReviewContext(self):
        '''Returns the flat review context, creating one if necessary.'''
        if not self.flatReviewContext:
            self.flatReviewContext = self.flatReviewContextClass(self)
            self.justEnteredFlatReviewMode = True
            self.targetCursorCell = braille.cursorCell
        
        return self.flatReviewContext

    
    def toggleFlatReviewMode(self, inputEvent = None):
        '''Toggles between flat review mode and focus tracking mode.'''
        if self.flatReviewContext:
            util.drawOutline(-1, 0, 0, 0)
            self.flatReviewContext = None
            self.updateBraille(orca_state.locusOfFocus)
        else:
            context = self.getFlatReviewContext()
            (string, x, y, width, height) = context.getCurrent(flat_review.Context.WORD)
            util.drawOutline(x, y, width, height)
            self._reviewCurrentItem(inputEvent, self.targetCursorCell)
        return True

    
    def updateBrailleReview(self, targetCursorCell = 0):
        '''Obtains the braille regions for the current flat review line
        and displays them on the braille display.  If the targetCursorCell
        is non-0, then an attempt will be made to postion the review cursor
        at that cell.  Otherwise, we will pan in display-sized increments
        to show the review cursor.'''
        context = self.getFlatReviewContext()
        (regions, regionWithFocus) = context.getCurrentBrailleRegions()
        if not regions:
            regions = []
            regionWithFocus = None
        
        line = braille.Line()
        line.addRegions(regions)
        braille.setLines([
            line])
        braille.setFocus(regionWithFocus, False)
        if regionWithFocus:
            braille.panToOffset(regionWithFocus.brailleOffset + regionWithFocus.cursorOffset)
        
        if self.justEnteredFlatReviewMode:
            braille.refresh(True, self.targetCursorCell)
            self.justEnteredFlatReviewMode = False
        else:
            braille.refresh(True, targetCursorCell)

    
    def _setFlatReviewContextToBeginningOfBrailleDisplay(self):
        '''Sets the character of interest to be the first character showing
        at the beginning of the braille display.'''
        context = self.getFlatReviewContext()
        (regions, regionWithFocus) = context.getCurrentBrailleRegions()
        for region in regions:
            if region.brailleOffset + len(region.string.decode('UTF-8')) > braille._viewport[0]:
                if isinstance(region, braille.ReviewText) or isinstance(region, braille.ReviewComponent):
                    position = max(region.brailleOffset, braille._viewport[0])
                    offset = position - region.brailleOffset
                    self.targetCursorCell = region.brailleOffset - braille._viewport[0]
                    (word, charOffset) = region.zone.getWordAtOffset(offset)
                    if word:
                        self.flatReviewContext.setCurrent(word.zone.line.index, word.zone.index, word.index, charOffset)
                    else:
                        self.flatReviewContext.setCurrent(region.zone.line.index, region.zone.index, 0, 0)
                    break
                    continue
        

    
    def panBrailleLeft(self, inputEvent = None, panAmount = 0):
        '''Pans the braille display to the left.  If panAmount is non-zero,
        the display is panned by that many cells.  If it is 0, the display
        is panned one full display width.  In flat review mode, panning
        beyond the beginning will take you to the end of the previous line.

        In focus tracking mode, the cursor stays at its logical position.
        In flat review mode, the review cursor moves to character
        associated with cell 0.'''
        if self.flatReviewContext:
            if braille.beginningIsShowing:
                self.flatReviewContext.goBegin(flat_review.Context.LINE)
                self.reviewPreviousCharacter(inputEvent)
            else:
                braille.panLeft(panAmount)
            self._setFlatReviewContextToBeginningOfBrailleDisplay()
            (string, x, y, width, height) = self.flatReviewContext.getCurrent(flat_review.Context.CHAR)
            util.drawOutline(x, y, width, height)
            self.targetCursorCell = 1
            self.updateBrailleReview(self.targetCursorCell)
        elif braille.beginningIsShowing and orca_state.locusOfFocus and self.isTextArea(orca_state.locusOfFocus):
            text = orca_state.locusOfFocus.text
            (string, startOffset, endOffset) = text.getTextAtOffset(text.caretOffset, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
            if startOffset > 0:
                text.setCaretOffset(startOffset - 1)
            
        else:
            braille.panLeft(panAmount)
            braille.refresh(False)
        return True

    
    def panBrailleLeftOneChar(self, inputEvent = None):
        '''Nudges the braille display one character to the left.

        In focus tracking mode, the cursor stays at its logical position.
        In flat review mode, the review cursor moves to character
        associated with cell 0.'''
        self.panBrailleLeft(inputEvent, 1)

    
    def panBrailleRight(self, inputEvent = None, panAmount = 0):
        '''Pans the braille display to the right.  If panAmount is non-zero,
        the display is panned by that many cells.  If it is 0, the display
        is panned one full display width.  In flat review mode, panning
        beyond the end will take you to the begininng of the next line.

        In focus tracking mode, the cursor stays at its logical position.
        In flat review mode, the review cursor moves to character
        associated with cell 0.'''
        if self.flatReviewContext:
            if braille.endIsShowing:
                self.flatReviewContext.goEnd(flat_review.Context.LINE)
                self.reviewNextCharacter(inputEvent)
            else:
                braille.panRight(panAmount)
            self._setFlatReviewContextToBeginningOfBrailleDisplay()
            (string, x, y, width, height) = self.flatReviewContext.getCurrent(flat_review.Context.CHAR)
            util.drawOutline(x, y, width, height)
            self.targetCursorCell = 1
            self.updateBrailleReview(self.targetCursorCell)
        elif braille.endIsShowing and orca_state.locusOfFocus and self.isTextArea(orca_state.locusOfFocus):
            text = orca_state.locusOfFocus.text
            (string, startOffset, endOffset) = text.getTextAtOffset(text.caretOffset, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
            if endOffset < text.characterCount:
                text.setCaretOffset(endOffset)
            
        else:
            braille.panRight(panAmount)
            braille.refresh(False)
        return True

    
    def panBrailleRightOneChar(self, inputEvent = None):
        '''Nudges the braille display one character to the right.

        In focus tracking mode, the cursor stays at its logical position.
        In flat review mode, the review cursor moves to character
        associated with cell 0.'''
        self.panBrailleRight(inputEvent, 1)

    
    def goBrailleHome(self, inputEvent = None):
        '''Returns to the component with focus.'''
        if self.flatReviewContext:
            return self.toggleFlatReviewMode(inputEvent)
        else:
            return braille.returnToRegionWithFocus(inputEvent)

    
    def leftClickReviewItem(self, inputEvent = None):
        '''Performs a left mouse button click on the current item.'''
        self.getFlatReviewContext().clickCurrent(1)
        return True

    
    def rightClickReviewItem(self, inputEvent = None):
        '''Performs a right mouse button click on the current item.'''
        self.getFlatReviewContext().clickCurrent(3)
        return True

    
    def reviewCurrentLine(self, inputEvent):
        '''Presents the current flat review line via braille and speech.'''
        clickCount = util.getClickCount(self.lastReviewCurrentEvent, inputEvent)
        self._reviewCurrentLine(inputEvent, clickCount)
        self.lastReviewCurrentEvent = inputEvent
        return True

    
    def _reviewCurrentLine(self, inputEvent, clickCount = 1):
        '''Presents the current flat review line via braille and speech.

        Arguments:
        - inputEvent - the current input event.
        - clickCount - number of times the user has "clicked" the current key.
        '''
        context = self.getFlatReviewContext()
        (string, x, y, width, height) = context.getCurrent(flat_review.Context.LINE)
        util.drawOutline(x, y, width, height)
        if not isinstance(inputEvent, input_event.BrailleEvent):
            if not string and not len(string) or string == '\n':
                speech.speak(_('blank'))
            elif string.isspace():
                speech.speak(_('white space'))
            elif string.isupper():
                pass
            None if clickCount < 2 or clickCount > 3 else clickCount > 3
            if clickCount == 2:
                self.spellCurrentItem(string)
            elif clickCount == 3:
                util.phoneticSpellCurrentItem(string)
            else:
                string = util.adjustForRepeats(string)
                speech.speak(string)
        
        self.updateBrailleReview()
        return True

    
    def reviewPreviousLine(self, inputEvent):
        '''Moves the flat review context to the beginning of the
        previous line.'''
        context = self.getFlatReviewContext()
        moved = context.goPrevious(flat_review.Context.LINE, flat_review.Context.WRAP_LINE)
        if moved:
            self._reviewCurrentLine(inputEvent)
            self.targetCursorCell = braille.cursorCell
        
        return True

    
    def reviewHome(self, inputEvent):
        '''Moves the flat review context to the top left of the current
        window.'''
        context = self.getFlatReviewContext()
        context.goBegin()
        self._reviewCurrentLine(inputEvent)
        self.targetCursorCell = braille.cursorCell
        return True

    
    def reviewNextLine(self, inputEvent):
        '''Moves the flat review context to the beginning of the
        next line.  Places the flat review cursor at the beginning
        of the line.'''
        context = self.getFlatReviewContext()
        moved = context.goNext(flat_review.Context.LINE, flat_review.Context.WRAP_LINE)
        if moved:
            self._reviewCurrentLine(inputEvent)
            self.targetCursorCell = braille.cursorCell
        
        return True

    
    def reviewBottomLeft(self, inputEvent):
        '''Moves the flat review context to the beginning of the
        last line in the window.  Places the flat review cursor at
        the beginning of the line.'''
        context = self.getFlatReviewContext()
        context.goEnd(flat_review.Context.WINDOW)
        context.goBegin(flat_review.Context.LINE)
        self._reviewCurrentLine(inputEvent)
        self.targetCursorCell = braille.cursorCell
        return True

    
    def reviewEnd(self, inputEvent):
        '''Moves the flat review context to the end of the
        last line in the window.  Places the flat review cursor
        at the end of the line.'''
        context = self.getFlatReviewContext()
        context.goEnd()
        self._reviewCurrentLine(inputEvent)
        self.targetCursorCell = braille.cursorCell
        return True

    
    def reviewCurrentItem(self, inputEvent, targetCursorCell = 0):
        '''Speak/Braille the current item to the user. A "double-click"
        of this key will cause the word to be spelt. A "triple-click"
        will cause the word to be phonetically spelt.
        '''
        clickCount = util.getClickCount(self.lastReviewCurrentEvent, inputEvent)
        self._reviewCurrentItem(inputEvent, targetCursorCell, clickCount)
        self.lastReviewCurrentEvent = inputEvent
        return True

    
    def spellCurrentItem(self, string):
        '''Spell the current flat review word or line.

        Arguments:
        - string: the string to spell.
        '''
        for index, character in enumerate(string):
            if character.isupper():
                speech.speak(character, self.voices[settings.UPPERCASE_VOICE])
                continue
            speech.speak(character)
        

    
    def _reviewCurrentItem(self, inputEvent, targetCursorCell = 0, clickCount = 1):
        '''Presents the current item to the user.

        Arguments:
        - inputEvent - the current input event.
        - targetCursorCell - if non-zero, the target braille cursor cell.
        - clickCount - number of times the user has "clicked" the current key.
        '''
        context = self.getFlatReviewContext()
        (string, x, y, width, height) = context.getCurrent(flat_review.Context.WORD)
        util.drawOutline(x, y, width, height)
        if not isinstance(inputEvent, input_event.BrailleEvent):
            if not string and not len(string) or string == '\n':
                speech.speak(_('blank'))
            else:
                (lineString, x, y, width, height) = context.getCurrent(flat_review.Context.LINE)
                if lineString == '\n':
                    speech.speak(_('blank'))
                elif string.isspace():
                    speech.speak(_('white space'))
                elif string.isupper():
                    pass
                None if clickCount < 2 or clickCount > 3 else clickCount > 3
                if clickCount == 2:
                    self.spellCurrentItem(string)
                elif clickCount == 3:
                    util.phoneticSpellCurrentItem(string)
                else:
                    string = util.adjustForRepeats(string)
                    speech.speak(string)
        
        self.updateBrailleReview(targetCursorCell)
        return True

    
    def reviewCurrentAccessible(self, inputEvent):
        context = self.getFlatReviewContext()
        (string, x, y, width, height) = context.getCurrent(flat_review.Context.ZONE)
        util.drawOutline(x, y, width, height)
        if not isinstance(inputEvent, input_event.BrailleEvent):
            speech.speakUtterances(self.speechGenerator.getSpeech(context.getCurrentAccessible(), False))
        
        return True

    
    def reviewPreviousItem(self, inputEvent):
        '''Moves the flat review context to the previous item.  Places
        the flat review cursor at the beginning of the item.'''
        context = self.getFlatReviewContext()
        moved = context.goPrevious(flat_review.Context.WORD, flat_review.Context.WRAP_LINE)
        if moved:
            self._reviewCurrentItem(inputEvent)
            self.targetCursorCell = braille.cursorCell
        
        return True

    
    def reviewNextItem(self, inputEvent):
        '''Moves the flat review context to the next item.  Places
        the flat review cursor at the beginning of the item.'''
        context = self.getFlatReviewContext()
        moved = context.goNext(flat_review.Context.WORD, flat_review.Context.WRAP_LINE)
        if moved:
            self._reviewCurrentItem(inputEvent)
            self.targetCursorCell = braille.cursorCell
        
        return True

    
    def reviewCurrentCharacter(self, inputEvent):
        '''Presents the current flat review character via braille and speech.
        '''
        clickCount = util.getClickCount(self.lastReviewCurrentEvent, inputEvent)
        self._reviewCurrentCharacter(inputEvent, clickCount)
        self.lastReviewCurrentEvent = inputEvent
        return True

    
    def _reviewCurrentCharacter(self, inputEvent, clickCount = 1):
        '''Presents the current flat review character via braille and speech.

        Arguments:
        - inputEvent - the current input event.
        - clickCount - number of times the user has "clicked" the current key.
        '''
        context = self.getFlatReviewContext()
        (string, x, y, width, height) = context.getCurrent(flat_review.Context.CHAR)
        util.drawOutline(x, y, width, height)
        if not isinstance(inputEvent, input_event.BrailleEvent):
            if not string or not len(string):
                speech.speak(_('blank'))
            else:
                (lineString, x, y, width, height) = context.getCurrent(flat_review.Context.LINE)
                if lineString == '\n':
                    speech.speak(_('blank'))
                elif clickCount == 2:
                    self.spellCurrentItem(string)
                elif clickCount == 3:
                    util.phoneticSpellCurrentItem(string)
                elif string.isupper():
                    speech.speak(string, self.voices[settings.UPPERCASE_VOICE])
                else:
                    speech.speak(string)
        
        self.updateBrailleReview()
        return True

    
    def reviewPreviousCharacter(self, inputEvent):
        '''Moves the flat review context to the previous character.  Places
        the flat review cursor at character.'''
        context = self.getFlatReviewContext()
        moved = context.goPrevious(flat_review.Context.CHAR, flat_review.Context.WRAP_LINE)
        if moved:
            self._reviewCurrentCharacter(inputEvent)
            self.targetCursorCell = braille.cursorCell
        
        return True

    
    def reviewEndOfLine(self, inputEvent):
        '''Moves the flat review context to the end of the line.  Places
        the flat review cursor at the end of the line.'''
        context = self.getFlatReviewContext()
        context.goEnd(flat_review.Context.LINE)
        self.reviewCurrentCharacter(inputEvent)
        self.targetCursorCell = braille.cursorCell
        return True

    
    def reviewNextCharacter(self, inputEvent):
        '''Moves the flat review context to the next character.  Places
        the flat review cursor at character.'''
        context = self.getFlatReviewContext()
        moved = context.goNext(flat_review.Context.CHAR, flat_review.Context.WRAP_LINE)
        if moved:
            self._reviewCurrentCharacter(inputEvent)
            self.targetCursorCell = braille.cursorCell
        
        return True

    
    def reviewAbove(self, inputEvent):
        '''Moves the flat review context to the character most directly
        above the current flat review cursor.  Places the flat review
        cursor at character.'''
        context = self.getFlatReviewContext()
        moved = context.goAbove(flat_review.Context.CHAR, flat_review.Context.WRAP_LINE)
        if moved:
            self._reviewCurrentItem(inputEvent, self.targetCursorCell)
        
        return True

    
    def reviewBelow(self, inputEvent):
        '''Moves the flat review context to the character most directly
        below the current flat review cursor.  Places the flat review
        cursor at character.'''
        context = self.getFlatReviewContext()
        moved = context.goBelow(flat_review.Context.CHAR, flat_review.Context.WRAP_LINE)
        if moved:
            self._reviewCurrentItem(inputEvent, self.targetCursorCell)
        
        return True

    
    def showZones(self, inputEvent):
        '''Debug routine to paint rectangles around the discrete
        interesting (e.g., text)  zones in the active window for
        this application.
        '''
        flatReviewContext = self.getFlatReviewContext()
        lines = flatReviewContext.lines
        for line in lines:
            string = ''
            for zone in line.zones:
                string += " '%s' [%s]" % (zone.string, zone.accessible.role)
                util.drawOutline(zone.x, zone.y, zone.width, zone.height, False)
            
            debug.println(debug.LEVEL_OFF, string)
        
        self.flatReviewContext = None

    
    def find(self, query = None):
        '''Searches for the specified query.  If no query is specified,
        it searches for the query specified in the Orca Find dialog.

        Arguments:
        - query: The search query to find.
        '''
        if not query:
            query = find.getLastQuery()
        
        if query:
            context = self.getFlatReviewContext()
            location = query.findQuery(context, self.justEnteredFlatReviewMode)
            if not location:
                braille.displayMessage(_('string not found'))
                speech.speak(_('string not found'))
            else:
                context.setCurrent(location.lineIndex, location.zoneIndex, location.wordIndex, location.charIndex)
                self.reviewCurrentItem(None)
                self.targetCursorCell = braille.cursorCell
        

    
    def findNext(self, inputEvent):
        '''Searches forward for the next instance of the string
        searched for via the Orca Find dialog.  Other than direction
        and the starting point, the search options initially specified
        (case sensitivity, window wrap, and full/partial match) are
        preserved.
        '''
        lastQuery = find.getLastQuery()
        if lastQuery:
            lastQuery.searchBackwards = False
            lastQuery.startAtTop = False
            self.find(lastQuery)
        else:
            orca._showFindGUI()

    
    def findPrevious(self, inputEvent):
        '''Searches backwards for the next instance of the string
        searched for via the Orca Find dialog.  Other than direction
        and the starting point, the search options initially specified
        (case sensitivity, window wrap, and full/partial match) are
        preserved.
        '''
        lastQuery = find.getLastQuery()
        if lastQuery:
            lastQuery.searchBackwards = True
            lastQuery.startAtTop = False
            self.find(lastQuery)
        else:
            orca._showFindGUI()


state_change_notifiers = { }
state_change_notifiers[rolenames.ROLE_CHECK_MENU_ITEM] = ('checked', None)
state_change_notifiers[rolenames.ROLE_CHECK_BOX] = ('checked', None)
state_change_notifiers[rolenames.ROLE_PANEL] = ('showing', None)
state_change_notifiers[rolenames.ROLE_LABEL] = ('showing', None)
state_change_notifiers[rolenames.ROLE_TOGGLE_BUTTON] = ('checked', None)
state_change_notifiers[rolenames.ROLE_TABLE_CELL] = ('checked', 'expanded', None)
