# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''Provides support for synthesizing keyboard and mouse events.'''
__id__ = '$Id: eventsynthesizer.py 1533 2006-10-06 07:59:04Z richb $'
__version__ = '$Revision: 1533 $'
__date__ = '$Date: 2006-10-06 03:59:04 -0400 (Fri, 06 Oct 2006) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'
import atspi
import debug

def generateMouseEvent(x, y, eventName):
    '''Synthesize a mouse event at a specific screen coordinate.
    Most AT clients should use the #AccessibleAction interface when
    tempted to generate mouse events, rather than this method.

    Event names: b1p = button 1 press; b2r = button 2 release;
                 b3c = button 3 click; b2d = button 2 double-click;
                 abs = absolute motion; rel = relative motion.

    Arguments:
    - x: the x screen coordinate
    - y: the y screen coordinate
    - eventName: the event name string (as described above)
    '''
    debug.println(debug.LEVEL_FINER, 'SYNTHESIZING MOUSE EVENT: (%d, %d) %s' % (x, y, eventName))
    d = atspi.Registry().registry.getDeviceEventController()
    d.generateMouseEvent(x, y, eventName)


def clickObject(obj, button):
    '''Performs a button click on the given Accessible.

    Arguments:
    - obj: the Accessible
    - button: an integer representing the mouse button number
    '''
    extents = obj.extents
    x = extents.x + extents.width / 2
    y = extents.y + extents.height / 2
    generateMouseEvent(x, y, 'b%dc' % button)


def clickPoint(x, y, button):
    '''Performs a button click on the given point.

    Arguments:
    - obj: the Accessible
    - x, y: the point
    - button: an integer representing the mouse button number
    '''
    generateMouseEvent(x, y, 'b%dc' % button)


def generateKeyboardEvent(keycode, keystring, type):
    """Generates a keyboard event.

    Arguments:
    - keyval: a long integer indicating the keycode or keysym of the key event
              being synthesized.
    - keystring: an (optional) UTF-8 string which, if keyval is NULL,
              indicates a 'composed' keyboard input string which is
              being synthesized; this type of keyboard event synthesis does
              not emulate hardware keypresses but injects the string
              as though a composing input method (such as XIM) were used.
    - type:   an AccessibleKeySynthType flag indicating whether keyval
              is to be interpreted as a keysym rather than a keycode
              (atspi.Accessibility.KEY_SYM), or whether to synthesize
              KEY_PRESS, KEY_RELEASE, or both (KEY_PRESSRELEASE).
    """
    d = atspi.Registry().registry.getDeviceEventController()
    d.generateKeyboardEvent(keycode, keystring, type)

