# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''Provides the FocusTrackingPresenter for Orca.'''
__id__ = '$Id: focus_tracking_presenter.py 2064 2007-02-27 16:46:00Z wwalker $'
__version__ = '$Revision: 2064 $'
__date__ = '$Date: 2007-02-27 11:46:00 -0500 (Tue, 27 Feb 2007) $'
__copyright__ = 'Copyright (c) 2005-2007 Sun Microsystems Inc.'
__license__ = 'LGPL'
import gobject
import Queue
import threading
import time
import atspi
import braille
import default
import debug
import input_event
import orca_state
import presentation_manager
import rolenames
import settings
import speech
import util
from orca_i18n import _
import CORBA

class FocusTrackingPresenter(presentation_manager.PresentationManager):
    '''Maintain a set of scripts for all running applications, and
    also keeps the notion of an activeScript.  All object events are
    passed to the associated script for that application, regardless if
    the application has keyboard focus or not.  All keyboard events are
    passed to the active script only if it has indicated interest in the
    event.'''
    
    def __init__(self):
        self.registry = atspi.Registry()
        self._knownScripts = { }
        self._knownAppSettings = { }
        self._oldAppSettings = None
        self._eventQueue = Queue.Queue(0)
        self._gidleId = 0
        self._gidleLock = threading.Lock()
        self.noFocusTimestamp = 0.0
        orca_state.activeScript = None
        if settings.debugEventQueue:
            self._enqueueEventCount = 0
            self._dequeueEventCount = 0
        

    
    def _registerEventListener(self, eventType):
        '''Tells this module to listen for the given event type.

        Arguments:
        - eventType: the event type.
        '''
        if self._listenerCounts.has_key(eventType):
            self._listenerCounts[eventType] += 1
        else:
            self.registry.registerEventListener(self._enqueueEvent, eventType)
            self._listenerCounts[eventType] = 1

    
    def _deregisterEventListener(self, eventType):
        '''Tells this module to stop listening for the given event type.

        Arguments:
        - eventType: the event type.
        '''
        self._listenerCounts[eventType] -= 1
        if self._listenerCounts[eventType] == 0:
            self.registry.deregisterEventListener(self._enqueueEvent, eventType)
            del self._listenerCounts[eventType]
        

    
    def _registerEventListeners(self, script):
        '''Tells the FocusTrackingPresenter to listen for all
        the event types of interest to the script.

        Arguments:
        - script: the script.
        '''
        for eventType in script.listeners.keys():
            self._registerEventListener(eventType)
        

    
    def _deregisterEventListeners(self, script):
        '''Tells the FocusTrackingPresenter to stop listening for all the
        event types of interest to the script.

        Arguments:
        - script: the script.
        '''
        for eventType in script.listeners.keys():
            self._deregisterEventListener(eventType)
        

    
    def _createScript(self, app):
        """For the given application name, create a new script instance.
        We'll first see if a mapping from appName to module name exists.
        If it does, we use that.  If it doesn't, we try the app name.
        If all else fails, we fall back to the default script.
        """
        script = None
        if settings.enableCustomScripts:
            scriptPackages = settings.scriptPackages
            moduleName = settings.getScriptModuleName(app)
            module = None
            if moduleName and len(moduleName):
                for package in scriptPackages:
                    if len(package):
                        name = package + '.' + moduleName
                    else:
                        name = moduleName
                    
                    try:
                        debug.println(debug.LEVEL_FINEST, 'Looking for script at %s.py...' % name)
                        module = __import__(name, globals(), locals(), [
                            ''])
                        debug.println(debug.LEVEL_FINEST, '...found %s.py' % name)
                    continue
                    except ImportError:
                        debug.println(debug.LEVEL_FINEST, '...could not find %s.py' % name)
                        continue
                        debug.printException(debug.LEVEL_SEVERE)
                        debug.println(debug.LEVEL_SEVERE, 'While attempting to import %s' % name)
                        continue
                    

                
            
            if module:
                
                try:
                    script = module.Script(app)
                debug.printException(debug.LEVEL_SEVERE)

            
        
        if not script and app and app.__dict__.has_key('toolkitName') and app.toolkitName:
            
            try:
                debug.println(debug.LEVEL_FINE, 'Looking for toolkit script %s.py...' % app.toolkitName)
                module = __import__(app.toolkitName, globals(), locals(), [
                    ''])
                script = module.Script(app)
                debug.println(debug.LEVEL_FINE, '...found %s.py' % name)
            except ImportError:
                debug.println(debug.LEVEL_FINE, '...could not find %s.py' % app.toolkitName)
            except:
                None<EXCEPTION MATCH>ImportError
                debug.printException(debug.LEVEL_SEVERE)
                debug.println(debug.LEVEL_SEVERE, 'While attempting to import %s' % app.toolkitName)
            

        None<EXCEPTION MATCH>ImportError
        if not script:
            script = default.Script(app)
        
        return script

    
    def _getScript(self, app):
        """Get a script for an app (and make it if necessary).  This is used
        instead of a simple calls to Script's constructor.

        Arguments:
        - app: the Python app

        Returns an instance of a Script.
        """
        if not app:
            if not self._defaultScript:
                self._defaultScript = default.Script(None)
                self._registerEventListeners(self._defaultScript)
            
            script = self._defaultScript
        elif self._knownScripts.has_key(app):
            script = self._knownScripts[app]
        else:
            script = self._createScript(app)
            self._knownScripts[app] = script
            self._registerEventListeners(script)
        return script

    
    def _reclaimScripts(self):
        '''Compares the list of known scripts to the list of known apps,
        deleting any scripts as necessary.
        '''
        
        try:
            apps = []
            desktop = self.registry.desktop
            for i in range(0, desktop.childCount):
                acc = desktop.getChildAtIndex(i)
                app = atspi.Accessible.makeAccessible(acc)
                if app:
                    apps.insert(0, app)
                    continue
            
            for app in self._knownScripts.keys():
                if apps.count(app) == 0:
                    script = self._knownScripts[app]
                    self._deregisterEventListeners(script)
                    del self._knownScripts[app]
                    del script
                    continue
        except:
            debug.printException(debug.LEVEL_FINEST)


    
    def loadAppSettings(self, app):
        '''Load the users application specific settings for an app.

        Arguments:
        - app: the Python app
        '''
        settingsPackages = settings.settingsPackages
        moduleName = settings.getScriptModuleName(app)
        module = None
        if moduleName and len(moduleName):
            for package in settingsPackages:
                if len(package):
                    name = package + '.' + moduleName
                else:
                    name = moduleName
                
                try:
                    debug.println(debug.LEVEL_FINEST, 'Looking for settings at %s.py...' % name)
                    if self._knownAppSettings.has_key(name):
                        reload(self._knownAppSettings[name])
                    else:
                        self._knownAppSettings[name] = __import__(name, globals(), locals(), [
                            ''])
                    debug.println(debug.LEVEL_FINEST, '...found %s.py' % name)
                continue
                except ImportError:
                    debug.println(debug.LEVEL_FINEST, '...could not find %s.py' % name)
                    continue
                

            
        

    
    def _processKeyboardEvent(self, keyboardEvent):
        '''Processes the given keyboard event based on the keybinding from the
        currently active script.

        Arguments:
        - keyboardEvent: an instance of input_event.KeyboardEvent
        '''
        if orca_state.activeScript:
            
            try:
                orca_state.activeScript.processKeyboardEvent(keyboardEvent)
            debug.printException(debug.LEVEL_WARNING)
            debug.printStack(debug.LEVEL_WARNING)

        

    
    def _processBrailleEvent(self, brailleEvent):
        '''Called whenever a cursor key is pressed on the Braille display.

        Arguments:
        - brailleEvent: an instance of input_event.BrailleEvent
        '''
        if orca_state.activeScript:
            
            try:
                orca_state.activeScript.processBrailleEvent(brailleEvent)
            debug.printException(debug.LEVEL_WARNING)
            debug.printStack(debug.LEVEL_WARNING)

        

    
    def _processObjectEvent(self, event):
        '''Handles all events destined for scripts.

        Arguments:
        - e: an at-spi event.
        '''
        debug.printObjectEvent(debug.LEVEL_FINEST, event)
        
        try:
            if event.source.role == rolenames.ROLE_TOOL_TIP:
                return None
        except:
            pass

        if event.type == 'object:children-changed:remove':
            if event.source == atspi.Accessible.makeAccessible(self.registry.desktop):
                self._reclaimScripts()
            
            return None
        
        
        try:
            if event.source and event.source.state.count(atspi.Accessibility.STATE_DEFUNCT):
                debug.println(debug.LEVEL_FINEST, 'IGNORING DEFUNCT OBJECT')
                atspi.Accessible.deleteAccessible(event.source)
                return None
            
            if (not (debug.eventDebugFilter) or debug.eventDebugFilter) and debug.eventDebugFilter.match(event.type):
                debug.printDetails(debug.LEVEL_FINEST, '    ', event.source)
        except CORBA.COMM_FAILURE:
            debug.printException(debug.LEVEL_WARNING)
            debug.println(debug.LEVEL_FINEST, 'COMM_FAILURE above while processing event: ' + event.type)
        except CORBA.OBJECT_NOT_EXIST:
            debug.printException(debug.LEVEL_WARNING)
            debug.println(debug.LEVEL_WARNING, 'OBJECT_NOT_EXIST above while processing event: ' + event.type)
            atspi.Accessible.deleteAccessible(event.source)
            return None
        except:
            debug.printException(debug.LEVEL_WARNING)
            return None

        if not event.source:
            debug.println(debug.LEVEL_WARNING, 'ERROR: received an event with no source.')
            return None
        
        retryCount = 0
        oldLocusOfFocus = orca_state.locusOfFocus
        
        try:
            s = self._getScript(event.source.app)
        except:
            s = None
            debug.printException(debug.LEVEL_WARNING)
            debug.println(debug.LEVEL_WARNING, 'ERROR: received an event, but Script is None')

        while s and retryCount <= s.commFailureAttemptLimit:
            
            try:
                if not event.source.state.count(atspi.Accessibility.STATE_ICONIFIED):
                    if (event.type == 'window:activate' or event.type == 'focus:') and event.source.role == rolenames.ROLE_FRAME:
                        if not self._oldAppSettings:
                            self._oldAppSettings = util.saveOldAppSettings()
                        else:
                            util.restoreOldAppSettings(self._oldAppSettings)
                        orca_state.activeScript = self._getScript(event.source.app)
                        debug.println(debug.LEVEL_FINE, 'ACTIVE SCRIPT: ' + orca_state.activeScript.name)
                        appSettings = self.loadAppSettings(event.source.app)
                        braille.setupKeyRanges(orca_state.activeScript.brailleBindings.keys())
                    
                    s.processObjectEvent(event)
                    if retryCount:
                        debug.println(debug.LEVEL_WARNING, '  SUCCEEDED AFTER %d TRIES' % retryCount)
                    
                
            continue
            except CORBA.COMM_FAILURE:
                debug.printException(debug.LEVEL_WARNING)
                debug.println(debug.LEVEL_WARNING, 'COMM_FAILURE above while processing: ' + event.type)
                retryCount += 1
                if retryCount <= s.commFailureAttemptLimit:
                    orca_state.locusOfFocus = oldLocusOfFocus
                    debug.println(debug.LEVEL_WARNING, '  TRYING AGAIN (%d)' % retryCount)
                    time.sleep(s.commFailureWaitTime)
                else:
                    debug.println(debug.LEVEL_WARNING, '  GIVING UP AFTER %d TRIES' % (retryCount - 1))
                    atspi.Accessible.deleteAccessible(event.source)
                retryCount <= s.commFailureAttemptLimit
                debug.printException(debug.LEVEL_WARNING)
                break
                continue
            

            None<EXCEPTION MATCH>CORBA.COMM_FAILURE

    
    def _enqueueEvent(self, e):
        '''Handles all events destined for scripts.

        Arguments:
        - e: an at-spi event.
        '''
        if settings.debugEventQueue:
            if self._enqueueEventCount:
                debug.println(debug.LEVEL_ALL, 'focus_tracking_presenter._enqueueEvent has been entered before exiting (count = %d)' % self._enqueueEventCount)
            
            self._enqueueEventCount += 1
        
        event = None
        if isinstance(e, input_event.KeyboardEvent):
            if e.type == atspi.Accessibility.KEY_PRESSED_EVENT:
                debug.println(debug.LEVEL_FINEST, "----------> QUEUEING KEYPRESS '%s' (%d)" % (e.event_string, e.hw_code))
            elif e.type == atspi.Accessibility.KEY_RELEASED_EVENT:
                debug.println(debug.LEVEL_FINEST, "----------> QUEUEING KEYRELEASE '%s' (%d)" % (e.event_string, e.hw_code))
            
            event = e
        elif isinstance(e, input_event.BrailleEvent):
            debug.println(debug.LEVEL_FINEST, '----------> QUEUEING BRAILLE COMMAND %d' % e.event)
            event = e
        elif e.type == 'object:state-changed:defunct':
            if settings.debugEventQueue:
                self._enqueueEventCount -= 1
            
            return None
        
        if e.type == 'object:property-change:accessible-parent':
            if settings.debugEventQueue:
                self._enqueueEventCount -= 1
            
            return None
        
        if e.type == 'object:children-changed:remove' and e.source != self.registry.desktop:
            if settings.debugEventQueue:
                self._enqueueEventCount -= 1
            
            return None
        
        event = atspi.Event(e)
        if not (event.source) or not (event.source.valid):
            debug.println(debug.LEVEL_FINEST, '---------> IGNORING INVALID EVENT %s' % e.type)
            if settings.debugEventQueue:
                self._enqueueEventCount -= 1
            
            return None
        
        if (not (debug.eventDebugFilter) or debug.eventDebugFilter) and debug.eventDebugFilter.match(e.type):
            debug.println(debug.LEVEL_FINEST, '---------> QUEUEING EVENT %s' % e.type)
        
        if event:
            if settings.debugEventQueue:
                debug.println(debug.LEVEL_ALL, '           acquiring lock...')
            
            self._gidleLock.acquire()
            if settings.debugEventQueue:
                debug.println(debug.LEVEL_ALL, '           ...acquired')
            
            if settings.debugEventQueue:
                debug.println(debug.LEVEL_ALL, '           calling queue.put...')
            
            if settings.debugEventQueue:
                debug.println(debug.LEVEL_ALL, '           (full=%s)' % self._eventQueue.full())
            
            self._eventQueue.put(event)
            if settings.debugEventQueue:
                debug.println(debug.LEVEL_ALL, '           ...put complete')
            
            if not self._gidleId:
                if settings.gilSleepTime:
                    time.sleep(settings.gilSleepTime)
                
                self._gidleId = gobject.idle_add(self._dequeueEvent)
            
            if settings.debugEventQueue:
                debug.println(debug.LEVEL_ALL, '           releasing lock...')
            
            self._gidleLock.release()
            if settings.debugEventQueue:
                debug.println(debug.LEVEL_ALL, '           ...released')
            
        
        if settings.debugEventQueue:
            self._enqueueEventCount -= 1
        

    
    def _dequeueEvent(self):
        '''Handles all events destined for scripts.  Called by the GTK
        idle thread.
        '''
        rerun = True
        if settings.debugEventQueue:
            debug.println(debug.LEVEL_ALL, 'Entering focus_tracking_presenter._dequeueEvent' + ' %d' % self._dequeueEventCount)
            self._dequeueEventCount += 1
        
        
        try:
            event = self._eventQueue.get_nowait()
            if isinstance(event, input_event.KeyboardEvent):
                if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
                    debug.println(debug.LEVEL_FINEST, "DEQUEUED KEYPRESS '%s' (%d) <----------" % (event.event_string, event.hw_code))
                    pressRelease = 'PRESS'
                elif event.type == atspi.Accessibility.KEY_RELEASED_EVENT:
                    debug.println(debug.LEVEL_FINEST, "DEQUEUED KEYRELEASE '%s' (%d) <----------" % (event.event_string, event.hw_code))
                    pressRelease = 'RELEASE'
                
                debug.println(debug.eventDebugLevel, '\nvvvvv PROCESS KEY %s EVENT %s vvvvv' % (pressRelease, event.event_string))
                self._processKeyboardEvent(event)
                debug.println(debug.eventDebugLevel, '\n^^^^^ PROCESS KEY %s EVENT %s ^^^^^' % (pressRelease, event.event_string))
            elif isinstance(event, input_event.BrailleEvent):
                debug.println(debug.LEVEL_FINEST, 'DEQUEUED BRAILLE COMMAND %d <----------' % event.event)
                debug.println(debug.eventDebugLevel, '\nvvvvv PROCESS BRAILLE EVENT %d vvvvv' % event.event)
                self._processBrailleEvent(event)
                debug.println(debug.eventDebugLevel, '\n^^^^^ PROCESS BRAILLE EVENT %d ^^^^^' % event.event)
            elif (not (debug.eventDebugFilter) or debug.eventDebugFilter) and debug.eventDebugFilter.match(event.type):
                debug.println(debug.LEVEL_FINEST, 'DEQUEUED EVENT %s <----------' % event.type)
                debug.println(debug.eventDebugLevel, '\nvvvvv PROCESS OBJECT EVENT %s vvvvv' % event.type)
            
            self._processObjectEvent(event)
            if (not (debug.eventDebugFilter) or debug.eventDebugFilter) and debug.eventDebugFilter.match(event.type):
                debug.println(debug.eventDebugLevel, '^^^^^ PROCESS OBJECT EVENT %s ^^^^^\n' % event.type)
            
            if not (orca_state.activeScript) and not (orca_state.locusOfFocus):
                pass
            noFocus = self.noFocusTimestamp != orca_state.noFocusTimestamp
            self._gidleLock.acquire()
            if self._eventQueue.empty():
                if noFocus:
                    if settings.gilSleepTime:
                        time.sleep(settings.gilSleepTime)
                    
                    message = _('No focus')
                    if settings.brailleVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
                        braille.displayMessage(message)
                    
                    if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
                        speech.speak(message)
                    
                    self.noFocusTimestamp = orca_state.noFocusTimestamp
                
                self._gidleId = 0
                rerun = False
            
            self._gidleLock.release()
        except Queue.Empty:
            debug.println(debug.LEVEL_SEVERE, 'focus_tracking_presenter:_dequeueEvent: ' + ' the event queue is empty!')
        except:
            debug.printException(debug.LEVEL_SEVERE)

        if settings.debugEventQueue:
            self._dequeueEventCount -= 1
            debug.println(debug.LEVEL_ALL, 'Leaving focus_tracking_presenter._dequeueEvent' + ' %d' % self._dequeueEventCount)
        
        return rerun

    
    def processKeyboardEvent(self, keyboardEvent):
        '''Processes the given keyboard event based on the keybinding from the
        currently active script. This method is called synchronously from the
        at-spi registry and should be performant.  In addition, it must return
        True if it has consumed the event (and False if not).

        Arguments:
        - keyboardEvent: an instance of input_event.KeyboardEvent

        Returns True if the event should be consumed.
        '''
        if orca_state.activeScript and orca_state.activeScript.consumesKeyboardEvent(keyboardEvent):
            self._enqueueEvent(keyboardEvent)
            return True
        else:
            return False

    
    def processBrailleEvent(self, brailleEvent):
        '''Called whenever a cursor key is pressed on the Braille display.

        Arguments:
        - brailleEvent: an instance of input_event.BrailleEvent

        Returns True if the command was consumed; otherwise False
        '''
        if orca_state.activeScript and orca_state.activeScript.consumesBrailleEvent(brailleEvent):
            self._enqueueEvent(brailleEvent)
            return True
        else:
            return False

    
    def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
        '''Called when the visual object with focus changes.

        Arguments:
        - event: if not None, the Event that caused the change
        - oldLocusOfFocus: Accessible that is the old locus of focus
        - newLocusOfFocus: Accessible that is the new locus of focus
        '''
        if orca_state.activeScript:
            orca_state.activeScript.locusOfFocusChanged(event, oldLocusOfFocus, newLocusOfFocus)
        

    
    def visualAppearanceChanged(self, event, obj):
        """Called when the visual appearance of an object changes.
        This method should not be called for objects whose visual
        appearance changes solely because of focus -- locusOfFocusChanged
        is used for that.  Instead, it is intended mostly for objects
        whose notional 'value' has changed, such as a checkbox changing
        state, a progress bar advancing, a slider moving, text inserted,
        caret moved, etc.

        Arguments:
        - event: if not None, the Event that caused this to happen
        - obj: the Accessible whose visual appearance changed.
        """
        if orca_state.activeScript:
            orca_state.activeScript.visualAppearanceChanged(event, obj)
        

    
    def activate(self):
        '''Called when this presentation manager is activated.'''
        speech.speak(_('Switching to focus tracking mode.'))
        self._listenerCounts = { }
        self._knownScripts = { }
        self._knownAppSettings = { }
        self._oldAppSettings = None
        self._defaultScript = None
        orca_state.activeScript = self._getScript(None)
        braille.setupKeyRanges(orca_state.activeScript.brailleBindings.keys())
        self._registerEventListener('window:activate')
        self._registerEventListener('window:deactivate')
        self._registerEventListener('object:children-changed:remove')
        win = util.findActiveWindow()
        if win:
            e = atspi.Event()
            e.source = win._acc
            e.type = 'window:activate'
            e.detail1 = 0
            e.detail2 = 0
            e.any_data = None
            self._enqueueEvent(e)
        

    
    def deactivate(self):
        '''Called when this presentation manager is deactivated.'''
        for eventType in self._listenerCounts.keys():
            self.registry.deregisterEventListener(self._enqueueEvent, eventType)
        
        self._listenerCounts = { }
        self._knownScripts = { }
        self._knownAppSettings = { }
        self._oldAppSettings = None
        self._defaultScript = None
        orca_state.activeScript = None


