# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''Provides support for defining keybindings and matching them to input
events.'''
__id__ = '$Id: keybindings.py 1818 2006-12-13 01:21:31Z wwalker $'
__version__ = '$Revision: 1818 $'
__date__ = '$Date: 2006-12-12 20:21:31 -0500 (Tue, 12 Dec 2006) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'

try:
    import gtk
except:
    pass

import atspi
import debug
import settings
from orca_i18n import _
_keycodeCache = { }

def _getKeycode(keysym):
    """Converts an XKeysym string (e.g., 'KP_Enter') to a keycode that
    should match the event.hw_code for key events.

    This whole situation is caused by the fact that Solaris chooses
    to give us different keycodes for the same key, and the keypad
    is the primary place where this happens: if NumLock is not on,
    there is no telling the difference between keypad keys and the
    other navigation keys (e.g., arrows, page up/down, etc.).  One,
    for example, would expect to get KP_End for the '1' key on the
    keypad if NumLock were not on.  Instead, we get 'End' and the
    keycode for it matches the keycode for the other 'End' key.  Odd.
    If NumLock is on, we at least get KP_* keys.

    So...when setting up keybindings, we say we're interested in
    KeySyms, but those keysyms are carefully chosen so as to result
    in a keycode that matches the actual key on the keyboard.  This
    is why we use KP_1 instead of KP_End and so on in our keybindings.

    Arguments:
    - keysym: a string that is a valid representation of an XKeysym.

    Returns an integer representing a key code that should match the
    event.hw_code for key events.
    """
    if not _keycodeCache.has_key(keysym):
        keymap = gtk.gdk.keymap_get_default()
        keyval = gtk.gdk.keyval_from_name(keysym)
        if keyval == 0:
            return 0
        
        _keycodeCache[keysym] = 0
        entries = keymap.get_entries_for_keyval(keyval)
        if entries:
            for entry in entries:
                if entry[1] == 0:
                    _keycodeCache[keysym] = entry[0]
                    break
                
                if _keycodeCache[keysym] == 0:
                    _keycodeCache[keysym] = entries[0][0]
                    continue
            
        
    
    return _keycodeCache[keysym]


def getModifierNames(mods):
    '''Gets the modifier names of a numeric modifier mask as a human
    consumable string.
    '''
    text = ''
    if mods & 1 << settings.MODIFIER_ORCA:
        text += _('Orca') + '+'
    
    if mods & 128:
        text += _('Alt_R') + '+'
    
    if mods & 1 << atspi.Accessibility.MODIFIER_META3:
        text += _('Super') + '+'
    
    if mods & 1 << atspi.Accessibility.MODIFIER_META2:
        text += _('Meta2') + '+'
    
    if mods & 1 << atspi.Accessibility.MODIFIER_ALT:
        text += _('Alt_L') + '+'
    
    if mods & 1 << atspi.Accessibility.MODIFIER_CONTROL:
        text += _('Ctrl') + '+'
    
    if mods & 1 << atspi.Accessibility.MODIFIER_SHIFTLOCK:
        text += _('Caps_Lock') + '+'
    
    if mods & 1 << atspi.Accessibility.MODIFIER_SHIFT:
        text += _('Shift') + '+'
    
    return text


class KeyBinding:
    '''A single key binding, consisting of a keycode, a modifier mask,
    and the InputEventHandler.
    '''
    
    def __init__(self, keysymstring, modifier_mask, modifiers, handler):
        """Creates a new key binding.

        Arguments:
        - keysymstring: the keysymstring - this is typically a string
          from /usr/include/X11/keysymdef.h with the preceding 'XK_'
          removed (e.g., XK_KP_Enter becomes the string 'KP_Enter').
        - modifier_mask: bit mask where a set bit tells us what modifiers
          we care about (see atspi.Accessibility.MODIFIER_*)
        - modifiers: the state the modifiers we care about must be in for
          this key binding to match an input event (see also
          atspi.Accessibility.MODIFIER_*)
        - handler: the InputEventHandler for this key binding
        """
        self.keysymstring = keysymstring
        self.modifier_mask = modifier_mask
        self.modifiers = modifiers
        self.handler = handler
        self.keycode = None

    
    def matches(self, keycode, modifiers):
        '''Returns true if this key binding matches the given keycode and
        modifier state.
        '''
        if not self.keycode:
            self.keycode = _getKeycode(self.keysymstring)
        
        if self.keycode == keycode:
            result = modifiers & self.modifier_mask
            return result == self.modifiers
        else:
            return False



class KeyBindings:
    '''Structure that maintains a set of KeyBinding instances.
    '''
    
    def __init__(self):
        self.keyBindings = []

    
    def add(self, keyBinding):
        '''Adds the given KeyBinding instance to this set of keybindings.
        '''
        self.keyBindings.append(keyBinding)

    
    def remove(self, keyBinding):
        '''Removes the given KeyBinding instance from this set of keybindings.
        '''
        for i in range(0, len(self.keyBindings)):
            if keyBinding == self.keyBindings[i]:
                del self.keyBindings[i]
                continue
        

    
    def removeByHandler(self, handler):
        '''Removes the given KeyBinding instance from this set of keybindings.
        '''
        i = len(self.keyBindings)
        while i > 0:
            if self.keyBindings[i - 1].handler == handler:
                del self.keyBindings[i - 1]
            
            i = i - 1

    
    def hasKeyBinding(self, newKeyBinding, typeOfSearch = 'strict'):
        '''Return True if keyBinding is already in self.keyBindings.

           The typeOfSearch can be:
              "strict":      matches description, modifiers, key
              "description": matches only description.
              "keys":        matches only modifiers and key.
        '''
        hasIt = False
        for keyBinding in self.keyBindings:
            if typeOfSearch == 'strict':
                if keyBinding.handler._description == newKeyBinding.handler._description and keyBinding.keysymstring == newKeyBinding.keysymstring and keyBinding.modifier_mask == newKeyBinding.modifier_mask and keyBinding.modifiers == newKeyBinding.modifiers:
                    hasIt = True
                
            keyBinding.modifiers == newKeyBinding.modifiers
            if typeOfSearch == 'description':
                if keyBinding.handler._description == newKeyBinding.handler._description:
                    hasIt = True
                
            keyBinding.handler._description == newKeyBinding.handler._description
            if typeOfSearch == 'keys':
                if keyBinding.keysymstring == newKeyBinding.keysymstring and keyBinding.modifier_mask == newKeyBinding.modifier_mask and keyBinding.modifiers == newKeyBinding.modifiers:
                    hasIt = True
                
            keyBinding.modifiers == newKeyBinding.modifiers
        
        return hasIt

    
    def getInputHandler(self, keyboardEvent):
        '''Returns the input handler of the key binding that matches the
        given keycode and modifiers, or None if no match exists.
        '''
        handler = None
        for keyBinding in self.keyBindings:
            if keyBinding.matches(keyboardEvent.hw_code, keyboardEvent.modifiers):
                handler = keyBinding.handler
                break
                continue
        
        return handler

    
    def consumeKeyboardEvent(self, script, keyboardEvent):
        '''Attempts to consume the given keyboard event.  If these
        keybindings have a handler for the given keyboardEvent, it is
        assumed the event will always be consumed.
        '''
        consumed = False
        handler = self.getInputHandler(keyboardEvent)
        if handler:
            consumed = True
            if keyboardEvent.type == atspi.Accessibility.KEY_PRESSED_EVENT:
                
                try:
                    handler.processInputEvent(script, keyboardEvent)
                debug.printException(debug.LEVEL_SEVERE)

            
        
        return consumed


