# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''Custom script for Evolution.'''
__id__ = '$Id: Evolution.py 1866 2006-12-28 18:53:39Z richb $'
__version__ = '$Revision: 1866 $'
__date__ = '$Date: 2006-12-28 13:53:39 -0500 (Thu, 28 Dec 2006) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'
import orca.debug as debug
import orca.default as default
import orca.atspi as atspi
import orca.input_event as input_event
import orca.keybindings as keybindings
import orca.rolenames as rolenames
import orca.braille as braille
import orca.orca_state as orca_state
import orca.speech as speech
import orca.settings as settings
import orca.util as util
import orca.chnames as chnames
from orca.orca_i18n import _

class Script(default.Script):
    
    def __init__(self, app):
        '''Creates a new script for the given application.

        Arguments:
        - app: the application to create a script for.
        '''
        self.debugLevel = debug.LEVEL_FINEST
        default.Script.__init__(self, app)
        self.message_panel = None
        self.setupPanels = { }
        self.setupLabels = { }
        self.lastMessageColumn = -1
        self.lastMessageRow = -1
        self.presentIfInactive = False
        rolenames.ROLE_CALENDAR_VIEW = 'Calendar View'
        rolenames.rolenames[rolenames.ROLE_CALENDAR_VIEW] = rolenames.Rolename(rolenames.ROLE_CALENDAR_VIEW, _('calv'), _('CalendarView'), _('calendar view'))
        rolenames.ROLE_CALENDAR_EVENT = 'Calendar Event'
        rolenames.rolenames[rolenames.ROLE_CALENDAR_EVENT] = rolenames.Rolename(rolenames.ROLE_CALENDAR_EVENT, _('cale'), _('CalendarEvent'), _('calendar event'))

    
    def setupInputEventHandlers(self):
        '''Defines InputEventHandler fields for this script that can be
        called by the key and braille bindings. In this particular case,
        we just want to be able to define our own sayAll() method.
        '''
        default.Script.setupInputEventHandlers(self)
        self.inputEventHandlers['sayAllHandler'] = input_event.InputEventHandler(Script.sayAll, _('Speaks entire document.'))
        self.inputEventHandlers['toggleReadMailHandler'] = input_event.InputEventHandler(Script.toggleReadMail, _('Toggle whether we present new mail if we are not the active script.'))

    
    def getKeyBindings(self):
        '''Defines the new key binding for this script. Setup the default
        key bindings, then add one in for toggling whether we present new
        mail if we not not the active script.

        Returns an instance of keybindings.KeyBindings.
        '''
        debug.println(self.debugLevel, 'Evolution.getKeyBindings.')
        keyBindings = default.Script.getKeyBindings(self)
        keyBindings.add(keybindings.KeyBinding('n', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['toggleReadMailHandler']))
        return keyBindings

    
    def toggleReadMail(self, inputEvent):
        ''' Toggle whether we present new mail if we not not the active script.+
        Arguments:
        - inputEvent: if not None, the input event that caused this action.
        '''
        debug.println(self.debugLevel, 'Evolution.toggleReadMail.')
        line = _('present new mail if this script is not active.')
        self.presentIfInactive = not (self.presentIfInactive)
        if not self.presentIfInactive:
            line = _('do not present new mail if this script is not active.')
        
        speech.speak(line)
        return True

    
    def speakSetupAssistantLabel(self, label):
        '''Perform a variety of tests on this Setup Assistant label to see
        if we want to speak it.

        Arguments:
        - label: the Setup Assistant Label.
        '''
        if label.state.count(atspi.Accessibility.STATE_SHOWING):
            obj = label.parent
            while obj and obj.role != rolenames.ROLE_APPLICATION:
                if obj.role == rolenames.ROLE_PANEL:
                    state = obj.state
                    if not state.count(atspi.Accessibility.STATE_ENABLED) and not state.count(atspi.Accessibility.STATE_SHOWING) or not state.count(atspi.Accessibility.STATE_VISIBLE):
                        return None
                    
                
                obj = obj.parent
            if label.text:
                charAttributes = label.text.getAttributes(0)
                if charAttributes[0]:
                    (charKeys, charDict) = self.textAttrsToDictionary(charAttributes[0])
                    weight = charDict.get('weight')
                    if weight and weight == '800':
                        text = util.getDisplayedText(label)
                        if text and not self.setupLabels.has_key(label):
                            speech.speak(text + _(' screen'), None, False)
                            self.setupLabels[label] = True
                            if orca_state.locusOfFocus and orca_state.locusOfFocus.role == rolenames.ROLE_PUSH_BUTTON and orca_state.locusOfFocus.state.count(atspi.Accessibility.STATE_SENSITIVE) == 0:
                                self.updateBraille(orca_state.locusOfFocus)
                                speech.speakUtterances(self.speechGenerator.getSpeech(orca_state.locusOfFocus, False))
                            
                        
                    
                
            
            text = util.getDisplayedText(label)
            if text and not self.setupLabels.has_key(label):
                if text.startswith(_('Please')) and text.startswith(_('Welcome')) or text.startswith(_('Congratulations')):
                    speech.speak(text, None, False)
                    self.setupLabels[label] = True
                
            
        

    
    def handleSetupAssistantPanel(self, panel):
        '''Find all the labels in this Setup Assistant panel and see if
        we want to speak them.

        Arguments:
        - panel: the Setup Assistant panel.
        '''
        allLabels = util.findByRole(panel, rolenames.ROLE_LABEL)
        for i in range(0, len(allLabels)):
            self.speakSetupAssistantLabel(allLabels[i])
        

    
    def readPageTab(self, tab):
        '''Speak/Braille the given page tab. The speech verbosity is set
           to VERBOSITY_LEVEL_BRIEF for this operation and then restored
           to its previous value on completion.

        Arguments:
        - tab: the page tab to speak/braille.
        '''
        brailleGen = self.brailleGenerator
        speechGen = self.speechGenerator
        savedSpeechVerbosityLevel = settings.speechVerbosityLevel
        settings.speechVerbosityLevel = settings.VERBOSITY_LEVEL_BRIEF
        utterances = speechGen.getSpeech(tab, False)
        speech.speakUtterances(utterances)
        settings.speechVerbosityLevel = savedSpeechVerbosityLevel
        braille.displayRegions(brailleGen.getBrailleRegions(tab))

    
    def getTimeForCalRow(self, row, noIncs):
        '''Return a string equivalent to the time of the given row in
           the calendar day view. Each calendar row is equivalent to
           a certain time interval (from 5 minutes upto 1 hour), with
           time (row 0) starting at 12 am (midnight).

        Arguments:
        - row: the row number.
        - noIncs: the number of equal increments that the 24 hour period
                  is divided into.

        Returns the time as a string.
        '''
        totalMins = timeIncrements[noIncs] * row
        if totalMins < 720:
            suffix = 'A.M.'
        else:
            totalMins -= 720
            suffix = 'P.M.'
        hrs = hours[totalMins / 60]
        mins = minutes[totalMins % 60]
        return hrs + ' ' + mins + ' ' + suffix

    
    def textLines(self):
        '''This an iterator that produces elements of the form:
        [SayAllContext, acss], where SayAllContext has the text to be
        spoken and acss is an ACSS instance for speaking the text.  We
        do this because of the way Evolution lays out its messages:
        each paragraph tends to be in its own panel, each of which is
        in a higher level panel.  So, we just traverse through the
        children.
        '''
        panel = orca_state.locusOfFocus.parent
        htmlPanel = orca_state.locusOfFocus.parent.parent
        startIndex = panel.index
        for i in range(startIndex, htmlPanel.childCount):
            accPanel = htmlPanel.accessible.getChildAtIndex(i)
            panel = atspi.Accessible.makeAccessible(accPanel)
            accTextObj = panel.accessible.getChildAtIndex(0)
            textObj = atspi.Accessible.makeAccessible(accTextObj)
            for context, acss in util.textLines(textObj):
                yield [
                    context,
                    acss]
            
        

    
    def sayAll(self, inputEvent):
        '''Speak all the text associated with the text object that has
           focus. We have to define our own method here because Evolution
           does not implement the FLOWS_TO relationship and all the text
           are in an HTML panel which contains multiple panels, each
           containing a single text object.

        Arguments:
        - inputEvent: if not None, the input event that caused this action.
        '''
        debug.println(self.debugLevel, 'evolution.sayAll.')
        if orca_state.locusOfFocus and orca_state.locusOfFocus.text:
            speech.sayAll(self.textLines(), self._Script__sayAllProgressCallback)
        else:
            default.Script.sayAll(self, inputEvent)
        return True

    
    def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
        '''Called when the visual object with focus changes.

        Arguments:
        - event: if not None, the Event that caused the change
        - oldLocusOfFocus: Accessible that is the old locus of focus
        - newLocusOfFocus: Accessible that is the new locus of focus
        '''
        brailleGen = self.brailleGenerator
        speechGen = self.speechGenerator
        debug.printObjectEvent(self.debugLevel, event, event.source.toString())
        rolesList = [
            rolenames.ROLE_TEXT,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_UNKNOWN]
        if util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'evolution.locusOfFocusChanged - mail view: ' + 'current message pane: ' + 'individual lines of text.')
            (string, caretOffset, startOffset) = util.getTextLineAtCaret(event.source)
            braille.displayMessage(string)
            if settings.enableSpeechIndentation:
                self.speakTextIndentation(event.source, string)
            
            line = util.adjustForRepeats(string)
            if self.speakNewLine(event.source):
                speech.speak(chnames.getCharacterName('\n'), None, False)
            
            if self.speakBlankLine(event.source):
                speech.speak(_('blank'), None, False)
            else:
                speech.speak(line, None, False)
            return None
        
        rolesList = [
            rolenames.ROLE_TEXT,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_TABLE_CELL]
        if settings.readTableCellRow and util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'evolution.locusOfFocusChanged - mail view: ' + 'current message pane: ' + 'standard mail header lines.')
            obj = event.source.parent.parent
            parent = obj.parent
            if parent.role == rolenames.ROLE_TABLE:
                row = parent.table.getRowAtIndex(obj.index)
                utterances = []
                regions = []
                for i in range(0, parent.table.nColumns):
                    obj = parent.table.getAccessibleAt(row, i)
                    cell = atspi.Accessible.makeAccessible(obj)
                    while cell.childCount:
                        cell = cell.child(0)
                    if cell.role == rolenames.ROLE_TEXT:
                        regions.append(braille.Text(cell))
                        (string, caretOffset, startOffset) = util.getTextLineAtCaret(cell)
                        utterances.append(string)
                        continue
                
                braille.displayRegions([
                    regions,
                    regions[0]])
                speech.speakUtterances(utterances)
                return None
            
        
        rolesList = [
            rolenames.ROLE_TABLE_CELL,
            rolenames.ROLE_TREE_TABLE]
        if settings.readTableCellRow and util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'evolution.locusOfFocusChanged - mail view: ' + 'message header list.')
            settings.readTableCellRow = False
            parent = event.source.parent
            row = parent.table.getRowAtIndex(event.source.index)
            column = parent.table.getColumnAtIndex(event.source.index)
            if not self.lastMessageRow != row:
                if row == 0 or row == parent.table.nRows - 1:
                    pass
            speakAll = self.lastMessageColumn == column
            savedBrailleVerbosityLevel = settings.brailleVerbosityLevel
            savedSpeechVerbosityLevel = settings.speechVerbosityLevel
            brailleRegions = []
            cellWithFocus = None
            if orca_state.locusOfFocus.role != rolenames.ROLE_TABLE_CELL:
                speakAll = True
                message = '%d messages' % parent.table.nRows
                brailleRegions.append(braille.Region(message))
                speech.speak(message)
            
            for i in range(0, parent.table.nColumns):
                obj = parent.table.getAccessibleAt(row, i)
                if obj:
                    cell = atspi.Accessible.makeAccessible(obj)
                    verbose = cell.index == event.source.index
                    header_obj = parent.table.getColumnHeader(i)
                    header = atspi.Accessible.makeAccessible(header_obj)
                    checkbox = False
                    toRead = True
                    action = cell.action
                    if action:
                        for j in range(0, action.nActions):
                            if action.getName(j) == 'toggle':
                                checkbox = True
                                checked = cell.state.count(atspi.Accessibility.STATE_CHECKED)
                                if speakAll:
                                    if header.name == _('Status'):
                                        toRead = not checked
                                        break
                                    
                                    if not checked:
                                        toRead = False
                                    
                                
                                break
                                continue
                        
                    
                    if toRead:
                        if not checkbox and verbose:
                            settings.brailleVerbosityLevel = settings.VERBOSITY_LEVEL_BRIEF
                            settings.speechVerbosityLevel = settings.VERBOSITY_LEVEL_BRIEF
                            utterances = speechGen.getSpeech(header, False)
                            (headerRegions, focusedRegion) = brailleGen.getBrailleRegions(header)
                            brailleRegions.extend(headerRegions)
                            brailleRegions.append(braille.Region(' '))
                            if column == i:
                                cellWithFocus = focusedRegion
                            
                            if speakAll or column == i:
                                speech.speakUtterances(utterances)
                            
                        
                        if verbose:
                            settings.brailleVerbosityLevel = settings.VERBOSITY_LEVEL_VERBOSE
                        else:
                            settings.brailleVerbosityLevel = settings.VERBOSITY_LEVEL_BRIEF
                        settings.speechVerbosityLevel = savedSpeechVerbosityLevel
                        utterances = speechGen.getSpeech(cell, False)
                        (cellRegions, focusedRegion) = brailleGen.getBrailleRegions(cell)
                        if header.name == _('Status'):
                            text = _('unread')
                            utterances = [
                                text]
                            brailleRegions.append(braille.Region(text))
                            brailleRegions.append(braille.Region(' '))
                        elif header.name == _('Attachment'):
                            text = header.name
                            utterances = [
                                text]
                            brailleRegions.append(braille.Region(text))
                            brailleRegions.append(braille.Region(' '))
                        else:
                            brailleRegions.extend(cellRegions)
                            brailleRegions.append(braille.Region(' '))
                        if column == i and cellWithFocus == None:
                            cellWithFocus = focusedRegion
                        
                        if speakAll or column == i:
                            speech.speakUtterances(utterances)
                        
                    
                toRead
            
            if brailleRegions != []:
                braille.displayRegions([
                    brailleRegions,
                    cellWithFocus])
            
            settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
            settings.speechVerbosityLevel = savedSpeechVerbosityLevel
            self.lastMessageColumn = column
            self.lastMessageRow = row
            settings.readTableCellRow = True
            return None
        
        rolesList = [
            rolenames.ROLE_CALENDAR_EVENT,
            rolenames.ROLE_CALENDAR_VIEW]
        if util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'evolution.locusOfFocusChanged - calendar view: ' + 'day view: tabbing to day with appts.')
            parent = event.source.parent
            utterances = speechGen.getSpeech(parent, False)
            (brailleRegions, focusedRegion) = brailleGen.getBrailleRegions(parent)
            speech.speakUtterances(utterances)
            apptExtents = event.source.component.getExtents(0)
            for i in range(0, parent.childCount):
                child = parent.child(i)
                if child.role == rolenames.ROLE_TABLE:
                    noRows = child.table.nRows
                    for j in range(0, noRows):
                        row = child.table.getRowAtIndex(j)
                        obj = child.table.getAccessibleAt(row, 0)
                        appt = atspi.Accessible.makeAccessible(obj)
                        extents = appt.component.getExtents(0)
                        if extents.y == apptExtents.y:
                            utterances = speechGen.getSpeech(event.source, False)
                            (apptRegions, focusedRegion) = brailleGen.getBrailleRegions(event.source)
                            brailleRegions.extend(apptRegions)
                            speech.speakUtterances(utterances)
                            startTime = 'Start time ' + self.getTimeForCalRow(j, noRows)
                            brailleRegions.append(braille.Region(startTime))
                            speech.speak(startTime)
                            apptLen = apptExtents.height / extents.height
                            endTime = 'End time ' + self.getTimeForCalRow(j + apptLen, noRows)
                            brailleRegions.append(braille.Region(endTime))
                            speech.speak(endTime)
                            braille.displayRegions([
                                brailleRegions,
                                brailleRegions[0]])
                            return None
                            continue
                    
            
        
        rolesList = [
            rolenames.ROLE_UNKNOWN,
            rolenames.ROLE_TABLE,
            rolenames.ROLE_CALENDAR_VIEW]
        if util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'evolution.locusOfFocusChanged - calendar view: ' + 'day view: moving with arrow keys.')
            brailleRegions = []
            index = event.source.index
            parent = event.source.parent
            calendarView = event.source.parent.parent
            extents = event.source.component.getExtents(0)
            noRows = parent.table.nRows
            found = False
            for i in range(0, calendarView.childCount):
                child = calendarView.child(i)
                if child.role == rolenames.ROLE_CALENDAR_EVENT:
                    apptExtents = child.component.getExtents(0)
                    if extents.y == apptExtents.y:
                        utterances = speechGen.getSpeech(child, False)
                        (apptRegions, focusedRegion) = brailleGen.getBrailleRegions(child)
                        brailleRegions.extend(apptRegions)
                        speech.speakUtterances(utterances)
                        startTime = 'Start time ' + self.getTimeForCalRow(index, noRows)
                        brailleRegions.append(braille.Region(startTime))
                        speech.speak(startTime)
                        apptLen = apptExtents.height / extents.height
                        endTime = 'End time ' + self.getTimeForCalRow(index + apptLen, noRows)
                        brailleRegions.append(braille.Region(endTime))
                        speech.speak(endTime)
                        braille.displayRegions([
                            brailleRegions,
                            brailleRegions[0]])
                        found = True
                    
                extents.y == apptExtents.y
            
            if not found:
                startTime = 'Start time ' + self.getTimeForCalRow(index, noRows)
                brailleRegions.append(braille.Region(startTime))
                speech.speak(startTime)
                utterance = _('No appointments')
                speech.speak(utterance)
                brailleRegions.append(braille.Region(utterance))
                braille.displayRegions([
                    brailleRegions,
                    brailleRegions[0]])
            
            return None
        
        rolesList = [
            rolenames.ROLE_TABLE_CELL,
            rolenames.ROLE_TABLE,
            rolenames.ROLE_UNKNOWN,
            rolenames.ROLE_SCROLL_PANE]
        if util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'evolution.locusOfFocusChanged - preferences dialog: ' + 'table cell in options list.')
            index = event.source.index
            obj = event.source.parent.parent.parent
            parent = obj.parent
            if parent.role == rolenames.ROLE_FILLER:
                for i in range(0, parent.childCount):
                    child = parent.child(i)
                    if child.role == rolenames.ROLE_PAGE_TAB_LIST:
                        tabList = child
                        tab = tabList.child(index - 1)
                        if tab.role == rolenames.ROLE_PAGE_TAB:
                            self.readPageTab(tab)
                            return None
                        
                    tab.role == rolenames.ROLE_PAGE_TAB
                
            
        
        rolesList = [
            rolenames.ROLE_PUSH_BUTTON,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_SPLIT_PANE,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_DIALOG]
        if util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'evolution.locusOfFocusChanged - mail insert ' + 'attachment dialog: unlabelled button.')
            brailleRegions = []
            utterance = _('Directories button')
            speech.speak(utterance)
            brailleRegions.append(braille.Region(utterance))
            braille.displayRegions([
                brailleRegions,
                brailleRegions[0]])
            return None
        
        rolesList = [
            rolenames.ROLE_TEXT,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_SCROLL_PANE]
        if util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'evolution.locusOfFocusChanged - mail ' + 'compose window: message area.')
            self.message_panel = event.source.parent.parent
            if self.speakNewLine(event.source):
                speech.speak(chnames.getCharacterName('\n'), None, False)
            
            if self.speakBlankLine(event.source):
                speech.speak(_('blank'), None, False)
            
        
        rolesList = [
            rolenames.ROLE_TABLE,
            rolenames.ROLE_SCROLL_PANE,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_DIALOG]
        if util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'evolution.locusOfFocusChanged - spell checking dialog.')
            self.updateBraille(orca_state.locusOfFocus)
            panel = event.source.parent.parent
            allLabels = util.findByRole(panel, rolenames.ROLE_LABEL)
            found = False
            for i in range(0, len(allLabels)):
                if not found:
                    text = util.getDisplayedText(allLabels[i])
                    if text:
                        tokens = text.split()
                    else:
                        tokens = []
                    for j in range(0, len(tokens)):
                        if tokens[j].startswith("'"):
                            badWord = tokens[j]
                            badWord = badWord[1:len(badWord) - 1]
                            found = True
                            break
                            continue
                    
            
            if self.message_panel != None:
                allTokens = []
                panel = self.message_panel
                allText = util.findByRole(panel, rolenames.ROLE_TEXT)
                for i in range(0, len(allText)):
                    text = self.getText(allText[i], 0, -1)
                    tokens = text.split()
                    allTokens += tokens
                
                util.speakMisspeltWord(allTokens, badWord)
                return None
            
        
        rolesList = [
            rolenames.ROLE_PUSH_BUTTON,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_TABLE_CELL,
            rolenames.ROLE_TABLE,
            rolenames.ROLE_PANEL]
        if util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'evolution.locusOfFocusChanged - ' + 'mail message area attachments.')
            default.Script.locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus)
            table = event.source.parent.parent.parent.parent.parent
            cell = table.child(table.childCount - 1)
            allText = util.findByRole(cell, rolenames.ROLE_TEXT)
            utterance = 'for ' + self.getText(allText[0], 0, -1)
            speech.speak(utterance)
            return None
        
        obj = event.source.parent
        while obj and obj.role != rolenames.ROLE_APPLICATION:
            if obj.role == rolenames.ROLE_FRAME and obj.name.endswith(_('Assistant')):
                debug.println(self.debugLevel, 'evolution.locusOfFocusChanged - ' + 'setup assistant.')
                self.setupPanels = { }
                self.setupLabels = { }
                break
            
            obj = obj.parent
        default.Script.locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus)

    
    def speakNewLine(self, obj):
        '''Returns True if a newline should be spoken.
           Otherwise, returns False.
        '''
        text = obj.text
        if not text:
            return False
        
        if not orca_state.lastInputEvent and orca_state.lastInputEvent.__dict__.has_key('event_string'):
            return False
        
        lastKey = orca_state.lastInputEvent.event_string
        if lastKey != 'Left' and lastKey != 'Right':
            return False
        
        mods = orca_state.lastInputEvent.modifiers
        isControlKey = mods & 1 << atspi.Accessibility.MODIFIER_CONTROL
        caretOffset = text.caretOffset
        line = text.getTextAtOffset(caretOffset, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
        lineStart = line[1]
        lineEnd = line[2]
        if isControlKey:
            word = text.getTextAtOffset(caretOffset, atspi.Accessibility.TEXT_BOUNDARY_WORD_START)
            wordStart = word[1]
            wordEnd = word[2]
            if lastKey == 'Right':
                if wordStart == lineStart:
                    return True
                
            elif wordEnd == lineEnd:
                return True
            
        elif lastKey == 'Right':
            if caretOffset == lineStart:
                return True
            
        elif caretOffset == lineEnd:
            return True
        
        return False

    
    def speakBlankLine(self, obj):
        '''Returns True if a blank line should be spoken.
        Otherwise, returns False.
        '''
        text = obj.text
        if not text:
            return False
        
        caretOffset = text.caretOffset
        line = text.getTextAtOffset(caretOffset, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
        debug.println(debug.LEVEL_FINEST, 'speakBlankLine: start=%d, end=%d, line=<%s>' % (line[1], line[2], line[0]))
        if line[1] == 0 and line[2] == 0:
            return settings.speakBlankLines
        else:
            return False

    
    def onStateChanged(self, event):
        '''Called whenever an object\'s state changes.  We are only
        interested in "object:state-changed:showing" events for any
        object in the Setup Assistant.

        Arguments:
        - event: the Event
        '''
        if event.type.endswith('showing'):
            obj = event.source.parent
            while obj and obj.role != rolenames.ROLE_APPLICATION:
                if obj.role == rolenames.ROLE_FRAME and obj.name.endswith(_('Assistant')):
                    debug.println(self.debugLevel, 'evolution.onStateChanged - ' + 'setup assistant.')
                    if event.source.role == rolenames.ROLE_LABEL:
                        self.speakSetupAssistantLabel(event.source)
                        break
                    elif event.source.role == rolenames.ROLE_PANEL and not self.setupPanels.has_key(event.source):
                        self.handleSetupAssistantPanel(event.source)
                        self.setupPanels[event.source] = True
                        break
                    
                
                obj = obj.parent
        
        default.Script.onStateChanged(self, event)


timeIncrements = { }
timeIncrements[288] = 5
timeIncrements[144] = 10
timeIncrements[96] = 15
timeIncrements[48] = 30
timeIncrements[24] = 60
minutes = { }
minutes[0] = ''
minutes[5] = '5'
minutes[10] = '10'
minutes[15] = '15'
minutes[20] = '20'
minutes[25] = '25'
minutes[30] = '30'
minutes[35] = '35'
minutes[40] = '40'
minutes[45] = '45'
minutes[50] = '50'
minutes[55] = '55'
hours = [
    '12',
    '1',
    '2',
    '3',
    '4',
    '5',
    '6',
    '7',
    '8',
    '9',
    '10',
    '11']
