# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''Custom script for StarOffice and OpenOffice.'''
__id__ = '$Id: StarOffice.py 2068 2007-02-28 15:24:41Z richb $'
__version__ = '$Revision: 2068 $'
__date__ = '$Date: 2007-02-28 10:24:41 -0500 (Wed, 28 Feb 2007) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'
import orca.debug as debug
import orca.atspi as atspi
import orca.chnames as chnames
import orca.default as default
import orca.input_event as input_event
import orca.rolenames as rolenames
import orca.braille as braille
import orca.braillegenerator as braillegenerator
import orca.orca as orca
import orca.orca_state as orca_state
import orca.speech as speech
import orca.speechgenerator as speechgenerator
import orca.settings as settings
import orca.keybindings as keybindings
import orca.util as util
from orca.orca_i18n import _
inputLineForCell = None
dynamicColumnHeaders = { }
dynamicRowHeaders = { }

def adjustForWriterTable(obj):
    '''Check to see if we are in Writer, where the object with focus is a
    paragraph, and the parent is the table cell. If it is, then, return the
    parent table cell otherwise return the current object.

    Arguments:
    - obj: the accessible object to check.

    Returns parent table cell (if in a Writer table ) or the current object.
    '''
    if obj.role == rolenames.ROLE_PARAGRAPH and obj.parent.role == rolenames.ROLE_TABLE_CELL:
        return obj.parent
    else:
        return obj


def getTable(obj):
    """Get the table that this table cell is in.

    Arguments:
    - obj: the table cell.

    Return the table that this table cell is in, or None if this object
    isn't in a table.
    """
    table = None
    obj = adjustForWriterTable(obj)
    if obj.role == rolenames.ROLE_TABLE_CELL and obj.parent:
        table = obj.parent.table
    
    return table


def getDynamicColumnHeaderCell(obj, column):
    '''Given a table cell, return the dynamic column header cell associated
    with it.

    Arguments:
    - obj: the table cell.
    - column: the column that this dynamic header is on.

    Return the dynamic column header cell associated with the given table cell.
    '''
    obj = adjustForWriterTable(obj)
    accCell = None
    parent = obj.parent
    if parent and parent.table:
        row = parent.table.getRowAtIndex(obj.index)
        cell = parent.table.getAccessibleAt(row, column)
        accCell = atspi.Accessible.makeAccessible(cell)
    
    return accCell


def getDynamicRowHeaderCell(obj, row):
    '''Given a table cell, return the dynamic row header cell associated
    with it.

    Arguments:
    - obj: the table cell.
    - row: the row that this dynamic header is on.

    Return the dynamic row header cell associated with the given table cell.
    '''
    obj = adjustForWriterTable(obj)
    accCell = None
    parent = obj.parent
    if parent and parent.table:
        column = parent.table.getColumnAtIndex(obj.index)
        cell = parent.table.getAccessibleAt(row, column)
        accCell = atspi.Accessible.makeAccessible(cell)
    
    return accCell


def locateInputLine(obj):
    '''Return the spread sheet input line. This only needs to be found
    the very first time a spread sheet table cell gets focus. We use the
    table cell to work back up the component hierarchy until we have found
    the common panel that both it and the input line reside in. We then
    use that as the base component to search for a component which has a
    paragraph role. This will be the input line.

    Arguments:
    - obj: the spread sheet table cell that has just got focus.

    Returns the spread sheet input line component.
    '''
    inputLine = None
    panel = obj.parent.parent.parent.parent
    if panel and panel.role == rolenames.ROLE_PANEL:
        allParagraphs = util.findByRole(panel, rolenames.ROLE_PARAGRAPH)
        if len(allParagraphs) == 1:
            inputLine = allParagraphs[0]
        else:
            debug.println(debug.LEVEL_SEVERE, 'StarOffice: locateInputLine: incorrect paragraph count.')
    else:
        debug.println(debug.LEVEL_SEVERE, "StarOffice: locateInputLine: couldn't find common panel.")
    return inputLine


def isSpreadSheetCell(obj):
    '''Return an indication of whether the given obj is a spread sheet
    table cell.

    Arguments:
    - obj: the object to check.

    Returns True if this is a table cell, False otherwise.
    '''
    found = False
    rolesList = [
        rolenames.ROLE_TABLE_CELL,
        rolenames.ROLE_TABLE,
        rolenames.ROLE_UNKNOWN,
        rolenames.ROLE_SCROLL_PANE,
        rolenames.ROLE_PANEL,
        rolenames.ROLE_ROOT_PANE,
        rolenames.ROLE_FRAME,
        rolenames.ROLE_APPLICATION]
    if util.isDesiredFocusedItem(obj, rolesList):
        current = obj.parent
        while current.role != rolenames.ROLE_APPLICATION:
            if current.role == rolenames.ROLE_FRAME and current.name and current.name.endswith(_('Calc')):
                found = True
            
            current = current.parent
    
    return found


class BrailleGenerator(braillegenerator.BrailleGenerator):
    '''Overrides _getBrailleRegionsForTableCellRow so that , when we are
    in a spread sheet, we can braille the dynamic row and column headers
    (assuming they are set).
    Overrides _getBrailleRegionsForTableCell so that, when we are in
    a spread sheet, we can braille the location of the table cell as well
    as the contents.
    '''
    
    def __init__(self, script):
        braillegenerator.BrailleGenerator.__init__(self, script)

    
    def _getBrailleRegionsForTableCellRow(self, obj):
        '''Get the braille for a table cell row or a single table cell
        if settings.readTableCellRow is False.

        Arguments:
        - obj: the table cell

        Returns a list where the first element is a list of Regions to display
        and the second element is the Region which should get focus.
        '''
        regions = []
        table = getTable(obj)
        parent = obj.parent
        if parent.__dict__.has_key('lastColumn') and parent.lastColumn != parent.table.getColumnAtIndex(obj.index):
            if dynamicColumnHeaders.has_key(table):
                row = dynamicColumnHeaders[table]
                header = getDynamicRowHeaderCell(obj, row)
                if header.childCount > 0:
                    for i in range(0, header.childCount):
                        child = header.child(i)
                        text = self._script.getText(child, 0, -1)
                        if text:
                            regions.append(braille.Region(' ' + text + ' '))
                            continue
                    
                elif header.text:
                    text = self._script.getText(header, 0, -1)
                    if text:
                        regions.append(braille.Region(' ' + text + ' '))
                    
                
            
        
        if parent.__dict__.has_key('lastRow') and parent.lastRow != parent.table.getRowAtIndex(obj.index):
            if dynamicRowHeaders.has_key(table):
                column = dynamicRowHeaders[table]
                header = getDynamicColumnHeaderCell(obj, column)
                if header.childCount > 0:
                    for i in range(0, header.childCount):
                        child = header.child(i)
                        text = self._script.getText(child, 0, -1)
                        if text:
                            regions.append(braille.Region(' ' + text + ' '))
                            continue
                    
                elif header.text:
                    text = self._script.getText(header, 0, -1)
                    if text:
                        regions.append(braille.Region(' ' + text + ' '))
                    
                
            
        
        if isSpreadSheetCell(obj):
            if settings.readTableCellRow and obj.parent.table:
                rowRegions = []
                savedBrailleVerbosityLevel = settings.brailleVerbosityLevel
                settings.brailleVerbosityLevel = settings.VERBOSITY_LEVEL_BRIEF
                parent = obj.parent
                row = parent.table.getRowAtIndex(obj.index)
                column = parent.table.getColumnAtIndex(obj.index)
                speakAll = True
                if parent.__dict__.has_key('lastRow') and parent.__dict__.has_key('lastColumn'):
                    if not parent.lastRow != row:
                        if row == 0 or row == parent.table.nRows - 1:
                            pass
                    speakAll = parent.lastColumn == column
                
                if speakAll:
                    focusRowRegion = None
                    for i in range(0, parent.table.nColumns):
                        accRow = parent.table.getAccessibleAt(row, i)
                        cell = atspi.Accessible.makeAccessible(accRow)
                        showing = cell.state.count(atspi.Accessibility.STATE_SHOWING)
                        if showing:
                            (cellRegions, focusRegion) = self._getBrailleRegionsForTableCell(cell)
                            if len(rowRegions):
                                rowRegions.append(braille.Region(' '))
                            
                            rowRegions.append(cellRegions[0])
                            if i == column:
                                focusRowRegion = cellRegions[0]
                            
                        i == column
                    
                    regions.extend(rowRegions)
                    settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
                else:
                    (cellRegions, focusRegion) = self._getBrailleRegionsForTableCell(obj)
                    regions.extend(cellRegions)
            else:
                (cellRegions, focusRegion) = self._getBrailleRegionsForTableCell(obj)
                regions.extend(cellRegions)
            regions = [
                regions,
                focusRegion]
        else:
            brailleGen = braillegenerator.BrailleGenerator
            (cellRegions, focusRegion) = brailleGen._getBrailleRegionsForTableCellRow(self, obj)
            regions.extend(cellRegions)
            regions = [
                regions,
                focusRegion]
        return regions

    
    def _getBrailleRegionsForTableCell(self, obj):
        """Get the braille for a table cell. If this isn't inside a
        spread sheet, just return the regions returned by the default
        table cell braille handler.

        Arguments:
        - obj: the table cell

        Returns a list where the first element is a list of Regions to display
        and the second element is the Region which should get focus.
        """
        global inputLineForCell
        if isSpreadSheetCell(obj):
            if inputLineForCell == None:
                inputLineForCell = locateInputLine(obj)
            
            regions = []
            text = util.getDisplayedText(obj)
            componentRegion = braille.Component(obj, text)
            regions.append(componentRegion)
            if obj.text:
                objectText = self._script.getText(obj, 0, -1)
                if objectText and len(objectText) != 0:
                    regions.append(braille.Region(' ' + obj.name))
                
            
            return [
                regions,
                componentRegion]
        elif obj.childCount <= 1:
            brailleGen = braillegenerator.BrailleGenerator
            regions = brailleGen._getBrailleRegionsForTableCell(self, obj)
        else:
            regions = []
            for i in range(0, obj.childCount):
                child = obj.child(i)
                (cellRegions, focusRegion) = self._getBrailleRegionsForTableCell(child)
                regions.extend(cellRegions)
            
            return [
                regions,
                focusRegion]
        return regions



class SpeechGenerator(speechgenerator.SpeechGenerator):
    '''Overrides _getSpeechForComboBox so that we can provide a name for
    the Calc Name combo box.
    Overrides _getSpeechForTableCellRow so that , when we are in a
    spread sheet, we can speak the dynamic row and column headers
    (assuming they are set).
    Overrides _getSpeechForTableCell so that, when we are in a spread
    sheet, we can speak the location of the table cell as well as the
    contents.
    Overrides _getSpeechForToggleButton so that, when the toggle buttons
    on the Formatting toolbar change state, we provide both the name and
    the state (as "on" or "off")
    Overrides _getSpeechForPushButton because sometimes the toggle buttons
    on the Formatting toolbar claim to be push buttons.
    '''
    
    def __init__(self, script):
        speechgenerator.SpeechGenerator.__init__(self, script)

    
    def _getSpeechForComboBox(self, obj, already_focused):
        '''Get the speech for a combo box. If the combo box already has focus,
        then only the selection is spoken.
        Also provides a name for the OOo Calc Name combo box. This name is
        provided in clause 5) of locusOfFocusChanged() below.

        Arguments:
        - obj: the combo box
        - already_focused: False if object just received focus

        Returns a list of utterances to be spoken for the object.
        '''
        utterances = []
        if not already_focused:
            label = self._getSpeechForObjectLabel(obj)
            if not label:
                label = [
                    obj.name]
            
            utterances.extend(label)
        else:
            label = None
        name = self._getSpeechForObjectName(obj)
        if name != label:
            utterances.extend(name)
        
        if not already_focused:
            utterances.extend(self._getSpeechForObjectRole(obj))
        
        utterances.extend(self._getSpeechForObjectAvailability(obj))
        self._debugGenerator('_getSpeechForComboBox', obj, already_focused, utterances)
        return utterances

    
    def _getSpeechForTableCellRow(self, obj, already_focused):
        """Get the speech for a table cell row or a single table cell
        if settings.readTableCellRow is False. If this isn't inside a
        spread sheet, just return the utterances returned by the default
        table cell speech handler.

        Arguments:
        - obj: the table cell
        - already_focused: False if object just received focus

        Returns a list of utterances to be spoken for the object.
        """
        utterances = []
        if not already_focused:
            table = getTable(obj)
            parent = obj.parent
            if parent.__dict__.has_key('lastColumn') and parent.lastColumn != parent.table.getColumnAtIndex(obj.index):
                if dynamicColumnHeaders.has_key(table):
                    row = dynamicColumnHeaders[table]
                    header = getDynamicRowHeaderCell(obj, row)
                    if header.childCount > 0:
                        for i in range(0, header.childCount):
                            child = header.child(i)
                            text = self._script.getText(child, 0, -1)
                            if text:
                                utterances.append(text)
                                continue
                        
                    elif header.text:
                        text = self._script.getText(header, 0, -1)
                        if text:
                            utterances.append(text)
                        
                    
                
            
            if parent.__dict__.has_key('lastRow') and parent.lastRow != parent.table.getRowAtIndex(obj.index):
                if dynamicRowHeaders.has_key(table):
                    column = dynamicRowHeaders[table]
                    header = getDynamicColumnHeaderCell(obj, column)
                    if header.childCount > 0:
                        for i in range(0, header.childCount):
                            child = header.child(i)
                            text = self._script.getText(child, 0, -1)
                            if text:
                                utterances.append(text)
                                continue
                        
                    elif header.text:
                        text = self._script.getText(header, 0, -1)
                        if text:
                            utterances.append(text)
                        
                    
                
            
        
        if isSpreadSheetCell(obj):
            if not already_focused:
                if settings.readTableCellRow:
                    parent = obj.parent
                    row = parent.table.getRowAtIndex(obj.index)
                    column = parent.table.getColumnAtIndex(obj.index)
                    speakAll = True
                    if parent.__dict__.has_key('lastRow') and parent.__dict__.has_key('lastColumn'):
                        if not parent.lastRow != row:
                            if row == 0 or row == parent.table.nRows - 1:
                                pass
                        speakAll = parent.lastColumn == column
                    
                    if speakAll:
                        for i in range(0, parent.table.nColumns):
                            accRow = parent.table.getAccessibleAt(row, i)
                            cell = atspi.Accessible.makeAccessible(accRow)
                            showing = cell.state.count(atspi.Accessibility.STATE_SHOWING)
                            if showing:
                                utterances.extend(self._getSpeechForTableCell(cell, already_focused))
                                continue
                        
                    else:
                        utterances.extend(self._getSpeechForTableCell(obj, already_focused))
                else:
                    utterances.extend(self._getSpeechForTableCell(obj, already_focused))
            
        else:
            speechGen = speechgenerator.SpeechGenerator
            utterances.extend(speechGen._getSpeechForTableCellRow(self, obj, already_focused))
        return utterances

    
    def _getSpeechForTableCell(self, obj, already_focused):
        """Get the speech for a table cell. If this isn't inside a
        spread sheet, just return the utterances returned by the default
        table cell speech handler.

        Arguments:
        - obj: the table cell
        - already_focused: False if object just received focus

        Returns a list of utterances to be spoken for the object.
        """
        global inputLineForCell
        if isSpreadSheetCell(obj):
            utterances = []
            if inputLineForCell == None:
                inputLineForCell = locateInputLine(obj)
            
            if obj.text:
                objectText = self._script.getText(obj, 0, -1)
                utterances.append(objectText)
            
            nameList = obj.name.split()
            utterances.append(nameList[1])
        elif obj.childCount <= 1:
            speechGen = speechgenerator.SpeechGenerator
            utterances = speechGen._getSpeechForTableCell(self, obj, already_focused)
        else:
            utterances = []
            for i in range(0, obj.childCount):
                child = obj.child(i)
                utterances.extend(self._getSpeechForTableCell(child, already_focused))
            
        return utterances

    
    def _getSpeechForToggleButton(self, obj, already_focused):
        """Get the speech for a toggle button.  We always want to speak the
        state if it's on a toolbar.

        Arguments:
        - obj: the toggle button
        - already_focused: False if object just received focus

        Returns a list of utterances to be spoken for the object.
        """
        utterances = []
        if obj.parent.role == rolenames.ROLE_TOOL_BAR:
            if obj.state.count(atspi.Accessibility.STATE_CHECKED):
                checkedState = _('on')
            else:
                checkedState = _('off')
            utterances.append(obj.name)
            utterances.append(checkedState)
        else:
            speechGen = speechgenerator.SpeechGenerator
            utterances.extend(speechGen._getSpeechForToggleButton(self, obj, already_focused))
        return utterances

    
    def _getSpeechForPushButton(self, obj, already_focused):
        """Get the speech for a push button.  We always want to speak the
        state if it's on a toolbar.

        Arguments:
        - obj: the push button
        - already_focused: False if object just received focus

        Returns a list of utterances to be spoken for the object.
        """
        utterances = []
        if obj.parent.role == rolenames.ROLE_TOOL_BAR:
            if obj.state.count(atspi.Accessibility.STATE_CHECKED):
                checkedState = _('on')
            else:
                checkedState = _('off')
            utterances.append(obj.name)
            utterances.append(checkedState)
        else:
            speechGen = speechgenerator.SpeechGenerator
            utterances.extend(speechGen._getSpeechForPushButton(self, obj, already_focused))
        return utterances



class Script(default.Script):
    
    def __init__(self, app):
        '''Creates a new script for the given application.

        Arguments:
        - app: the application to create a script for.
        '''
        default.Script.__init__(self, app)
        self.debugLevel = debug.LEVEL_FINEST
        self.lastCell = None
        self.lastTextLength = -1
        self.lastBadWord = ''
        self.lastStartOff = -1
        self.lastEndOff = -1
        self.lastDynamicEvent = None
        self.currentParagraph = None
        self.commFailureAttemptLimit = 1
        settings.enabledTextAttributes = 'size:; family-name:; weight:400; indent:0; left-margin:0; right-margin:0; underline:none; strikethrough:false; justification:left; style:normal;'
        self.tickled = None

    
    def getBrailleGenerator(self):
        '''Returns the braille generator for this script.
        '''
        return BrailleGenerator(self)

    
    def getSpeechGenerator(self):
        '''Returns the speech generator for this script.
        '''
        return SpeechGenerator(self)

    
    def setupInputEventHandlers(self):
        '''Defines InputEventHandler fields for this script that can be
        called by the key and braille bindings. In this particular case,
        we just want to be able to add a handler to return the contents of
        the input line.
        '''
        default.Script.setupInputEventHandlers(self)
        self.inputEventHandlers['speakInputLineHandler'] = input_event.InputEventHandler(Script.speakInputLine, _('Speaks the contents of the input line.'))
        self.inputEventHandlers['setDynamicColumnHeadersHandler'] = input_event.InputEventHandler(Script.setDynamicColumnHeaders, _('Set the row to use as dynamic column headers when speaking calc cells.'))
        self.inputEventHandlers['setDynamicRowHeadersHandler'] = input_event.InputEventHandler(Script.setDynamicRowHeaders, _('Set the column to use as dynamic row headers to use when speaking calc cells.'))

    
    def getKeyBindings(self):
        '''Defines the key bindings for this script. Setup the default
        key bindings, then add one in for reading the input line.

        Returns an instance of keybindings.KeyBindings.
        '''
        keyBindings = default.Script.getKeyBindings(self)
        keyBindings.add(keybindings.KeyBinding('a', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['speakInputLineHandler']))
        keyBindings.add(keybindings.KeyBinding('r', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['setDynamicColumnHeadersHandler']))
        keyBindings.add(keybindings.KeyBinding('c', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['setDynamicRowHeadersHandler']))
        return keyBindings

    
    def checkForTableBoundry(self, oldFocus, newFocus):
        """Check to see if we've entered or left a table.
        When entering a table, announce the table dimensions.
        When leaving a table, announce that the table has been exited.

        Arguments:
        - oldFocus: Accessible that is the old locus of focus
        - newFocus: Accessible that is the new locus of focus
        """
        if oldFocus == None or newFocus == None:
            return None
        
        oldFocusIsTable = None
        while oldFocus.role != rolenames.ROLE_APPLICATION:
            if oldFocus.role == rolenames.ROLE_TABLE:
                oldFocusIsTable = oldFocus
                break
            
            oldFocus = oldFocus.parent
        newFocusIsTable = None
        while newFocus.role != rolenames.ROLE_APPLICATION:
            if newFocus.role == rolenames.ROLE_TABLE:
                newFocusIsTable = newFocus
                break
            
            newFocus = newFocus.parent
        if oldFocusIsTable == None and newFocusIsTable != None:
            rows = newFocusIsTable.table.nRows
            columns = newFocusIsTable.table.nColumns
            line = _('table with %d rows and %d columns.') % (rows, columns)
            speech.speak(line)
        elif oldFocusIsTable != None and newFocusIsTable == None:
            speech.speak(_('leaving table.'))
        

    
    def speakInputLine(self, inputEvent):
        '''Speak the contents of the spread sheet input line (assuming we
        have a handle to it - generated when we first focus on a spread
        sheet table cell.

        This will be either the contents of the table cell that has focus
        or the formula associated with it.

        Arguments:
        - inputEvent: if not None, the input event that caused this action.
        '''
        debug.println(self.debugLevel, 'StarOffice.speakInputLine.')
        if isSpreadSheetCell(orca_state.locusOfFocus):
            if inputLineForCell and inputLineForCell.text:
                inputLine = self.getText(inputLineForCell, 0, -1)
                if not inputLine:
                    inputLine = _('empty')
                
                debug.println(self.debugLevel, 'StarOffice.speakInputLine: contents: %s' % inputLine)
                speech.speak(inputLine)
            
        

    
    def getTableRow(self, cell):
        """Get the row number in the table that this table cell is on.

        Arguments:
        - cell: the table cell to get the row number for.

        Return the row number that this table cell is on, or None if
        this isn't a table cell.
        """
        row = None
        cell = adjustForWriterTable(cell)
        if cell.role == rolenames.ROLE_TABLE_CELL:
            parent = cell.parent
            if parent and parent.table:
                row = parent.table.getRowAtIndex(cell.index)
            
        
        return row

    
    def getTableColumn(self, cell):
        """Get the column number in the table that this table cell is on.

        Arguments:
        - cell: the table cell to get the column number for.

        Return the column number that this table cell is on, or None if
        this isn't a table cell.
        """
        column = None
        cell = adjustForWriterTable(cell)
        if cell.role == rolenames.ROLE_TABLE_CELL:
            parent = cell.parent
            if parent and parent.table:
                column = parent.table.getColumnAtIndex(cell.index)
            
        
        return column

    
    def setDynamicColumnHeaders(self, inputEvent):
        '''Set the row for the dynamic header columns to use when speaking
        calc cell entries. In order to set the row, the user should first set
        focus to the row that they wish to define and then press Insert-r.

        Once the user has defined the row, it will be used to first speak
        this header when moving between columns.

        A "double-click" of the Insert-c hotkey, will clear the dynamic
        header column.

        Arguments:
        - inputEvent: if not None, the input event that caused this action.
        '''
        debug.println(self.debugLevel, 'StarOffice.setDynamicColumnHeaders.')
        clickCount = util.getClickCount(self.lastDynamicEvent, inputEvent)
        table = getTable(orca_state.locusOfFocus)
        if table:
            row = self.getTableRow(orca_state.locusOfFocus)
            if clickCount == 2:
                
                try:
                    del dynamicColumnHeaders[table]
                    line = _('Dynamic column header cleared.')
                    speech.speak(line)
                    braille.displayMessage(line)

            else:
                dynamicColumnHeaders[table] = row
                line = _('Dynamic column header set for row ') + str(row + 1)
                speech.speak(line)
                braille.displayMessage(line)
        
        self.lastDynamicEvent = inputEvent
        return True

    
    def columnConvert(self, column):
        """ Convert a spreadsheet column into it's column label

        Arguments:
        - column: the column number to convert.

        Returns a string representing the spread sheet column.
        """
        BASE26 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
        if column <= len(BASE26):
            return BASE26[column - 1]
        
        res = ''
        while column > 0:
            digit = column % len(BASE26)
            res = ' ' + BASE26[digit - 1] + res
            column /= len(BASE26)
        return res

    
    def setDynamicRowHeaders(self, inputEvent):
        '''Set the column for the dynamic header rows to use when speaking
        calc cell entries. In order to set the column, the user should first
        set focus to the column that they wish to define and then press
        Insert-c.

        Once the user has defined the column, it will be used to first speak
        this header when moving between rows.

        A "double-click" of the Insert-r hotkey, will clear the dynamic
        header row.

        Arguments:
        - inputEvent: if not None, the input event that caused this action.
        '''
        debug.println(self.debugLevel, 'StarOffice.setDynamicRowHeaders.')
        clickCount = util.getClickCount(self.lastDynamicEvent, inputEvent)
        table = getTable(orca_state.locusOfFocus)
        if table:
            column = self.getTableColumn(orca_state.locusOfFocus)
            if clickCount == 2:
                
                try:
                    del dynamicRowHeaders[table]
                    line = _('Dynamic row header cleared.')
                    speech.speak(line)
                    braille.displayMessage(line)

            else:
                dynamicRowHeaders[table] = column
                line = _('Dynamic row header set for column %s') % self.columnConvert(column + 1)
                speech.speak(line)
                braille.displayMessage(line)
        
        self.lastDynamicEvent = inputEvent
        return True

    
    def readMisspeltWord(self, event, pane):
        '''Speak/braille the current misspelt word plus its context.
           The spell check dialog contains a "paragraph" which shows the
           context for the current spelling mistake. After speaking/brailling
           the default action for this component, that a selection of the
           surronding text from that paragraph with the misspelt word is also
           spoken.

        Arguments:
        - event: the event.
        - pane: the option pane in the spell check dialog.
        '''
        paragraph = util.findByRole(pane, rolenames.ROLE_PARAGRAPH)
        textLength = paragraph[0].text.characterCount
        startFound = False
        startOff = 0
        endOff = textLength
        for i in range(0, textLength):
            attributes = paragraph[0].text.getAttributes(i)
            if len(attributes[0]) != 0:
                if not startFound:
                    startOff = i
                    startFound = True
                
            startFound
            if startFound:
                endOff = i
                break
                continue
        
        badWord = self.getText(paragraph[0], startOff, endOff - 1)
        debug.println(self.debugLevel, 'StarOffice.readMisspeltWord: type=%s  word=%s(%d,%d)  len=%d' % (event.type, badWord, startOff, endOff, textLength))
        if textLength == self.lastTextLength and badWord == self.lastBadWord and startOff == self.lastStartOff and endOff == self.lastEndOff:
            return None
        
        text = self.getText(paragraph[0], 0, -1)
        allTokens = text.split()
        util.speakMisspeltWord(allTokens, badWord)
        self.lastTextLength = textLength
        self.lastBadWord = badWord
        self.lastStartOff = startOff
        self.lastEndOff = endOff

    
    def endOfLink(self, obj, word, startOffset, endOffset):
        '''Return an indication of whether the given word contains the
           end of a hypertext link.

        Arguments:
        - obj: an Accessible object that implements the AccessibleText
               interface
        - word: the word to check
        - startOffset: the start offset for this word
        - endOffset: the end offset for this word

        Returns True if this word contains the end of a hypertext link.
        '''
        nLinks = obj.hypertext.getNLinks()
        links = []
        for i in range(0, nLinks):
            links.append(obj.hypertext.getLink(i))
        
        for i in range(0, len(links)):
            if links[i].endIndex > startOffset and links[i].endIndex <= endOffset:
                return True
                continue
        
        return False

    
    def sayWriterWord(self, obj, word, startOffset, endOffset):
        '''Speaks the given word in the appropriate voice. If this word is
        a hypertext link and it is also at the end offset for one of the
        links, then the word "link" is also spoken.

        Arguments:
        - obj: an Accessible object that implements the AccessibleText
               interface
        - word: the word to speak
        - startOffset: the start offset for this word
        - endOffset: the end offset for this word
        '''
        voices = settings.voices
        for i in range(startOffset, endOffset):
            if util.getLinkIndex(obj, i) >= 0:
                voice = voices[settings.HYPERLINK_VOICE]
                break
                continue
            if word.isupper():
                voice = voices[settings.UPPERCASE_VOICE]
                continue
            voice = voices[settings.DEFAULT_VOICE]
        
        speech.speak(word, voice)
        if self.endOfLink(obj, word, startOffset, endOffset):
            speech.speak(_('link'))
        

    
    def isSetupDialog(self, obj):
        ''' Check to see if this object is in the Setup dialog by walking
        back up the object hierarchy until we get to the dialog object and
        checking to see if it has a name that starts with "Welcome to
        StarOffice".

        Arguments:
        - obj: an Accessible object that implements the AccessibleText
               interface

        Returns an indication of whether this object is in the Setup dialog.
        '''
        found = False
        while obj and obj.role != rolenames.ROLE_APPLICATION:
            if obj.role == rolenames.ROLE_DIALOG and obj.name and obj.name.startswith(_('Welcome to StarOffice')):
                debug.println(self.debugLevel, 'StarOffice.isSetupDialog: True.')
                found = True
            
            obj = obj.parent
        return found

    
    def speakSetupLabel(self, label):
        '''Speak this Setup dialog label.

        Arguments:
        - label: the Setup dialog Label.
        '''
        text = util.getDisplayedText(label)
        if text:
            speech.speak(text)
        

    
    def handleSetupPanel(self, panel):
        '''Find all the labels in this Setup panel and speak them.

        Arguments:
        - panel: the Setup panel.
        '''
        allLabels = util.findByRole(panel, rolenames.ROLE_LABEL)
        for i in range(0, len(allLabels)):
            self.speakSetupLabel(allLabels[i])
        

    
    def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
        '''Called when the visual object with focus changes.

        Arguments:
        - event: if not None, the Event that caused the change
        - oldLocusOfFocus: Accessible that is the old locus of focus
        - newLocusOfFocus: Accessible that is the new locus of focus
        '''
        brailleGen = self.brailleGenerator
        speechGen = self.speechGenerator
        debug.printObjectEvent(self.debugLevel, event, event.source.toString())
        if self.findCommandRun and event.type == 'object:state-changed:focused':
            self.findCommandRun = False
            self.find()
            return None
        
        rolesList = [
            rolenames.ROLE_PARAGRAPH,
            rolenames.ROLE_UNKNOWN,
            rolenames.ROLE_SCROLL_PANE,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_ROOT_PANE,
            rolenames.ROLE_FRAME]
        if util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'StarOffice.locusOfFocusChanged - Writer: text paragraph.')
            result = util.getTextLineAtCaret(event.source)
            result[0] = result[0].decode('UTF-8')
            if oldLocusOfFocus.role == rolenames.ROLE_MENU_ITEM and oldLocusOfFocus.name == _('Text Document') and len(result[0]) == 0:
                self.whereAmI(None)
            
            hypertext = event.source.hypertext
            if not hypertext or hypertext.getNLinks() == 0:
                if settings.enableSpeechIndentation:
                    self.speakTextIndentation(event.source, result[0].encode('UTF-8'))
                
                speech.speak(result[0].encode('UTF-8'), None, False)
            else:
                started = False
                startOffset = 0
                for i in range(0, len(result[0])):
                    if result[0][i] == ' ':
                        if started:
                            endOffset = i
                            self.sayWriterWord(event.source, result[0][startOffset:endOffset + 1].encode('UTF-8'), startOffset, endOffset)
                            startOffset = i
                            started = False
                        
                    started
                    if not started:
                        startOffset = i
                        started = True
                        continue
                
                if started:
                    endOffset = len(result[0])
                    self.sayWriterWord(event.source, result[0][startOffset:endOffset].encode('UTF-8'), startOffset, endOffset)
                
            braille.displayRegions(brailleGen.getBrailleRegions(event.source))
            return None
        
        rolesList = [
            rolenames.ROLE_PUSH_BUTTON,
            rolenames.ROLE_OPTION_PANE,
            rolenames.ROLE_DIALOG,
            rolenames.ROLE_APPLICATION]
        if util.isDesiredFocusedItem(event.source, rolesList):
            pane = event.source.parent
            if pane.name.startswith(_('Spellcheck:')):
                debug.println(self.debugLevel, 'StarOffice.locusOfFocusChanged - ' + 'Writer: spell check dialog.')
                self.readMisspeltWord(event, pane)
            
        
        if self.isSetupDialog(event.source):
            rolesList = [
                rolenames.ROLE_PUSH_BUTTON,
                rolenames.ROLE_PANEL,
                rolenames.ROLE_OPTION_PANE,
                rolenames.ROLE_DIALOG,
                rolenames.ROLE_APPLICATION]
            if util.isDesiredFocusedItem(event.source, rolesList):
                debug.println(self.debugLevel, 'StarOffice.locusOfFocusChanged - Setup dialog: ' + 'License Agreement screen: Scroll Down button.')
                self.handleSetupPanel(event.source.parent)
                speech.speak(_('Note that the Scroll Down button has to be pressed numerous times.'))
            
            rolesList = [
                rolenames.ROLE_UNKNOWN,
                rolenames.ROLE_SCROLL_PANE,
                rolenames.ROLE_PANEL,
                rolenames.ROLE_OPTION_PANE,
                rolenames.ROLE_DIALOG,
                rolenames.ROLE_APPLICATION]
            if util.isDesiredFocusedItem(event.source, rolesList):
                debug.println(self.debugLevel, 'StarOffice.locusOfFocusChanged - Setup dialog: ' + 'License Agreement screen: accept button.')
                speech.speak(_('License Agreement Accept button now has focus.'))
            
            rolesList = [
                rolenames.ROLE_CHECK_BOX,
                rolenames.ROLE_PANEL,
                rolenames.ROLE_OPTION_PANE,
                rolenames.ROLE_DIALOG,
                rolenames.ROLE_APPLICATION]
            if util.isDesiredFocusedItem(event.source, rolesList):
                debug.println(self.debugLevel, 'StarOffice.locusOfFocusChanged - Setup dialog: ' + 'Personal Data: Transfer Personal Data check box.')
                self.handleSetupPanel(event.source.parent)
            
            rolesList = [
                rolenames.ROLE_TEXT,
                rolenames.ROLE_PANEL,
                rolenames.ROLE_OPTION_PANE,
                rolenames.ROLE_DIALOG,
                rolenames.ROLE_APPLICATION]
            if util.isDesiredFocusedItem(event.source, rolesList) and event.source.name == _('First name'):
                debug.println(self.debugLevel, 'StarOffice.locusOfFocusChanged - Setup dialog: ' + 'User name: First Name text field.')
                panel = event.source.parent
                allLabels = util.findByRole(panel, rolenames.ROLE_LABEL)
                for i in range(0, len(allLabels)):
                    relations = allLabels[i].relations
                    hasLabelFor = False
                    for relation in relations:
                        if relation.getRelationType() == atspi.Accessibility.RELATION_LABEL_FOR:
                            hasLabelFor = True
                            continue
                    
                    if not hasLabelFor:
                        self.speakSetupLabel(allLabels[i])
                        continue
                
            
            rolesList = [
                rolenames.ROLE_RADIO_BUTTON,
                rolenames.ROLE_PANEL,
                rolenames.ROLE_OPTION_PANE,
                rolenames.ROLE_DIALOG,
                rolenames.ROLE_APPLICATION]
            if util.isDesiredFocusedItem(event.source, rolesList):
                debug.println(self.debugLevel, 'StarOffice.locusOfFocusChanged - Setup dialog: ' + 'Registration: Register Now radio button.')
                self.handleSetupPanel(event.source.parent)
            
        
        rolesList = [
            rolenames.ROLE_PARAGRAPH,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_UNKNOWN,
            rolenames.ROLE_SCROLL_PANE,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_ROOT_PANE,
            rolenames.ROLE_FRAME,
            rolenames.ROLE_APPLICATION]
        if util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'StarOffice.locusOfFocusChanged - ' + 'Calc: cell editor.')
            return None
        
        rolesList = [
            rolenames.ROLE_COMBO_BOX,
            rolenames.ROLE_TOOL_BAR,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_ROOT_PANE,
            rolenames.ROLE_FRAME,
            rolenames.ROLE_APPLICATION]
        if util.isDesiredFocusedItem(event.source, rolesList):
            if not (event.source.name) or len(event.source.name) == 0:
                debug.println(self.debugLevel, 'StarOffice.locusOfFocusChanged - ' + 'Calc: name box.')
                event.source.name = _('Move to cell')
            
        default.Script.locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus)

    
    def onWindowActivated(self, event):
        '''Called whenever a property on an object changes.

        Arguments:
        - event: the Event
        '''
        debug.printObjectEvent(self.debugLevel, event, event.source.toString())
        if self.isSetupDialog(event.source):
            debug.println(self.debugLevel, 'StarOffice.onWindowActivated - Setup dialog: Welcome screen.')
            allPanels = util.findByRole(event.source.parent, rolenames.ROLE_PANEL)
            for i in range(0, len(allPanels)):
                if not allPanels[i].name:
                    allLabels = util.findByRole(allPanels[i], rolenames.ROLE_LABEL)
                    for i in range(0, len(allLabels)):
                        self.speakSetupLabel(allLabels[i])
                    
            
        else:
            default.Script.onWindowActivated(self, event)

    
    def onNameChanged(self, event):
        '''Called whenever a property on an object changes.

        Arguments:
        - event: the Event
        '''
        brailleGen = self.brailleGenerator
        speechGen = self.speechGenerator
        debug.printObjectEvent(self.debugLevel, event, event.source.toString())
        rolesList = [
            rolenames.ROLE_OPTION_PANE,
            rolenames.ROLE_DIALOG,
            rolenames.ROLE_APPLICATION]
        if util.isDesiredFocusedItem(event.source, rolesList):
            pane = event.source
            if pane.name.startswith(_('Spellcheck:')):
                debug.println(self.debugLevel, 'StarOffice.onNameChanged - Writer: spell check dialog.')
                self.readMisspeltWord(event, pane)
            
        
        default.Script.onNameChanged(self, event)

    
    def onFocus(self, event):
        '''Called whenever an object gets focus. Overridden in this script 
        so that we can adjust "focus:" events for children of a combo-box 
        to just set the focus to the combo box. This is needed to help 
        reduce the verbosity of focusing on the Calc Name combo box (see 
        bug #364407).

        Arguments:
        - event: the Event
        '''
        if event.source.parent.role == rolenames.ROLE_COMBO_BOX:
            orca.setLocusOfFocus(None, event.source.parent, False)
            return None
        
        if event.source.role == rolenames.ROLE_PARAGRAPH and event.source.parent.role == rolenames.ROLE_TABLE_CELL and event.source.state.count(atspi.Accessibility.STATE_FOCUSED):
            if self.lastCell != event.source.parent:
                default.Script.locusOfFocusChanged(self, event, None, event.source.parent)
                self.lastCell = event.source.parent
            
            return None
        
        default.Script.onFocus(self, event)

    
    def onStateChanged(self, event):
        """Called whenever an object's state changes.

        Arguments:
        - event: the Event
        """
        if event.type == 'object:state-changed:focused' and event.detail1 == 1:
            current = event.source.parent
            while current.role != rolenames.ROLE_APPLICATION:
                if current.role == rolenames.ROLE_FRAME and current.name and current.name.endswith(_('Writer')):
                    self.checkForTableBoundry(orca_state.locusOfFocus, event.source)
                    break
                
                current = current.parent
        
        if event.type == 'object:state-changed:active':
            if self.findCommandRun:
                return None
            
            if not self.tickled:
                self.tickled = self.flatReviewContextClass(self)
            
        
        if event.type == 'object:state-changed:checked':
            if event.source.role == rolenames.ROLE_TOGGLE_BUTTON or event.source.role == rolenames.ROLE_PUSH_BUTTON:
                weToggledIt = False
                if isinstance(orca_state.lastInputEvent, input_event.MouseButtonEvent):
                    x = orca_state.lastInputEvent.x
                    y = orca_state.lastInputEvent.y
                    weToggledIt = event.source.component.contains(x, y, 0)
                elif isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
                    keyString = orca_state.lastInputEvent.event_string
                    navKeys = [
                        'Up',
                        'Down',
                        'Page_Up',
                        'Page_Down',
                        'Home',
                        'End']
                    wasCommand = orca_state.lastInputEvent.modifiers & (1 << atspi.Accessibility.MODIFIER_CONTROL | 1 << atspi.Accessibility.MODIFIER_ALT | 1 << atspi.Accessibility.MODIFIER_META | 1 << atspi.Accessibility.MODIFIER_META2 | 1 << atspi.Accessibility.MODIFIER_META3)
                    if wasCommand:
                        pass
                    weToggledIt = keyString not in navKeys
                
                if weToggledIt:
                    speech.speakUtterances(self.speechGenerator.getSpeech(event.source, False))
                
            
        if event.type == 'object:state-changed:focused' and event.source.role == rolenames.ROLE_PARAGRAPH and event.source.parent.role == rolenames.ROLE_TABLE_CELL and event.detail1 == 0 and event.source.state.count(atspi.Accessibility.STATE_FOCUSED):
            event_string = None
            if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
                event_string = orca_state.lastInputEvent.event_string
            
            if (event_string == 'Up' or event_string == 'Down') and event.source.parent == self.lastCell and event.source.parent.childCount > 1:
                default.Script.locusOfFocusChanged(self, event, None, event.source)
            else:
                default.Script.locusOfFocusChanged(self, event, None, event.source.parent)
            self.lastCell = event.source.parent
            return None
        
        if event.type == 'object:state-changed:focused' and event.source.role == rolenames.ROLE_PARAGRAPH and event.source != self.currentParagraph:
            self.currentParagraph = event.source
            orca.setLocusOfFocus(event, event.source, False)
            return None
        
        if event.source.parent and event.source.parent.role == rolenames.ROLE_COMBO_BOX:
            orca.setLocusOfFocus(None, event.source.parent, False)
            return None
        
        default.Script.onStateChanged(self, event)

    
    def onSelectionChanged(self, event):
        """Called when an object's selection changes.

        Arguments:
        - event: the Event
        """
        debug.printObjectEvent(self.debugLevel, event, event.source.toString())
        rolesList = [
            rolenames.ROLE_LIST,
            rolenames.ROLE_COMBO_BOX,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_TOOL_BAR,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_ROOT_PANE,
            rolenames.ROLE_FRAME,
            rolenames.ROLE_APPLICATION]
        if util.isDesiredFocusedItem(event.source, rolesList):
            if orca_state.locusOfFocus.role == rolenames.ROLE_TABLE_CELL:
                cell = orca_state.locusOfFocus
                if cell != self.lastCell:
                    self.lastCell = cell
                    if cell.text:
                        cellText = self.getText(cell, 0, -1)
                        if cellText and len(cellText):
                            if inputLineForCell and inputLineForCell.text:
                                inputLine = self.getText(inputLineForCell, 0, -1)
                                if inputLine and len(inputLine) > 1 and inputLine[0] == '=':
                                    hf = _(' has formula')
                                    speech.speak(hf, None, False)
                                    line = braille.getShowingLine()
                                    line.addRegion(braille.Region(hf))
                                    braille.refresh()
                                
                            
                        
                    
                
            
        
        default.Script.onSelectionChanged(self, event)

    
    def getText(self, obj, startOffset, endOffset):
        """Returns the substring of the given object's text specialization.

        NOTE: This is here to handle the problematic implementation of
        getText by OpenOffice.  See the bug discussion at:

           http://bugzilla.gnome.org/show_bug.cgi?id=356425)

        Once the OpenOffice issue has been resolved, this method probably
        should be removed.

        Arguments:
        - obj: an accessible supporting the accessible text specialization
        - startOffset: the starting character position
        - endOffset: the ending character position
        """
        text = obj.text.getText(0, -1).decode('UTF-8')
        if startOffset >= len(text):
            startOffset = len(text) - 1
        
        if endOffset == -1:
            endOffset = len(text)
        elif startOffset >= endOffset:
            endOffset = startOffset + 1
        
        string = text[max(0, startOffset):min(len(text), endOffset)]
        string = string.encode('UTF-8')
        return string

    
    def speakCellName(self, name):
        '''Speaks the given cell name.

        Arguments:
        - name: the name of the cell
        '''
        line = _('Cell ') + name
        speech.speak(line)

    
    def onCaretMoved(self, event):
        '''Called whenever the caret moves.

        Arguments:
        - event: the Event
        '''
        if event.source.role == rolenames.ROLE_PARAGRAPH and event.source.parent.role == rolenames.ROLE_TABLE_CELL and event.source.state.count(atspi.Accessibility.STATE_FOCUSED):
            event_string = orca_state.lastInputEvent.event_string
            if event_string == 'Up' or event_string == 'Down':
                if settings.readTableCellRow == False:
                    if event.detail1 != -1:
                        self.speakCellName(event.source.parent.name)
                    
                
                return None
            
            if (event_string == 'Left' or event_string == 'Right') and self.lastCell != event.source.parent:
                return None
            
            caretOffset = event.source.text.caretOffset
            len = event.source.text.characterCount
            if event_string == 'Right' and caretOffset == len:
                return None
            
            if not event_string == 'Left' or caretOffset == len:
                if (event_string == 'Right' or caretOffset == 0) and event_string == 'Tab' or event_string == 'ISO_Left_Tab':
                    if settings.readTableCellRow == False:
                        if event.detail1 != -1:
                            self.speakCellName(event.source.parent.name)
                        
                    
                    if self.speakBlankLine(event.source):
                        speech.speak(_('blank'), None, False)
                    
                    return None
                
            
        if self.speakNewLine(event.source):
            speech.speak(chnames.getCharacterName('\n'), None, False)
        
        if self.speakBlankLine(event.source):
            speech.speak(_('blank'), None, False)
        
        default.Script.onCaretMoved(self, event)

    
    def speakNewLine(self, obj):
        '''Returns True if a newline should be spoken.
           Otherwise, returns False.
        '''
        text = obj.text
        if not text:
            return False
        
        if not orca_state.lastInputEvent and orca_state.lastInputEvent.__dict__.has_key('event_string'):
            return False
        
        lastKey = orca_state.lastInputEvent.event_string
        if lastKey != 'Left' and lastKey != 'Right':
            return False
        
        mods = orca_state.lastInputEvent.modifiers
        isControlKey = mods & 1 << atspi.Accessibility.MODIFIER_CONTROL
        caretOffset = text.caretOffset
        line = text.getTextAtOffset(caretOffset, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
        lineStart = line[1]
        lineEnd = line[2]
        if isControlKey:
            word = text.getTextAtOffset(caretOffset, atspi.Accessibility.TEXT_BOUNDARY_WORD_START)
            wordStart = word[1]
            wordEnd = word[2]
            if lastKey == 'Right':
                if wordStart == lineStart:
                    return True
                
            elif wordEnd == lineEnd:
                return True
            
        elif lastKey == 'Right':
            if caretOffset == lineStart:
                return True
            
        elif caretOffset == lineEnd:
            return True
        
        return False

    
    def speakBlankLine(self, obj):
        '''Returns True if a blank line should be spoken.
        Otherwise, returns False.
        '''
        text = obj.text
        if not text:
            return False
        
        caretOffset = text.caretOffset
        line = text.getTextAtOffset(caretOffset, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
        if line[1] == 0 and line[2] == 0:
            return settings.speakBlankLines
        


