# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

''' Custom script for Thunderbird 3.
'''
__id__ = '$Id: $'
__version__ = '$Revision: $'
__date__ = '$Date: $'
__copyright__ = 'Copyright (c) 2005-2007 Sun Microsystems Inc.'
__license__ = 'LGPL'
import orca.orca as orca
import orca.atspi as atspi
import orca.debug as debug
import orca.default as default
import orca.rolenames as rolenames
import orca.settings as settings
import orca.braille as braille
import orca.braillegenerator as braillegenerator
import orca.speech as speech
import orca.speechgenerator as speechgenerator
import orca.util as util
import orca.Gecko as Gecko
from orca.orca_i18n import _

class SpeechGenerator(Gecko.SpeechGenerator):
    '''Provides a speech generator specific to Thunderbird.
    '''
    
    def __init__(self, script):
        self.debugLevel = debug.LEVEL_FINEST
        self._debug('__init__')
        Gecko.SpeechGenerator.__init__(self, script)

    
    def _debug(self, msg):
        ''' Convenience method for printing debug messages
        '''
        debug.println(self.debugLevel, 'Thunderbird.SpeechGenerator: ' + msg)

    
    def getSpeechContext(self, obj, stopAncestor = None):
        '''Get the speech that describes the names and role of
        the container hierarchy of the object, stopping at and
        not including the stopAncestor.

        This code is almost identical to the getSpeechContext
        in Gecko.py. If Gecko.getSpeechContext changes, this
        method may need to be modified too.

        Arguments:
        - obj: the object
        - stopAncestor: the ancestor to stop at and not include (None
          means include all ancestors)

        Returns a list of utterances to be spoken.
        '''
        utterances = []
        if not obj:
            return utterances
        
        if obj is stopAncestor:
            return utterances
        
        parent = obj.parent
        while parent and parent.parent != parent:
            if parent == stopAncestor:
                break
            
            if parent.role == rolenames.ROLE_FILLER and parent.role == rolenames.ROLE_AUTOCOMPLETE and parent.role == rolenames.ROLE_PANEL and parent.role == rolenames.ROLE_FORM and parent.role == rolenames.ROLE_LIST_ITEM and parent.role == rolenames.ROLE_LIST and parent.role == rolenames.ROLE_PARAGRAPH and parent.role == rolenames.ROLE_SECTION and parent.role == rolenames.ROLE_LAYERED_PANE and parent.role == rolenames.ROLE_SPLIT_PANE and parent.role == rolenames.ROLE_SCROLL_PANE and parent.role == rolenames.ROLE_UNKNOWN or self._script.isLayoutOnly(parent):
                parent = parent.parent
                continue
            
            text = util.getDisplayedText(parent)
            label = util.getDisplayedLabel(parent)
            if text and text != label and len(text) and not text.startswith('chrome://'):
                utterances.append(text)
            
            if label and len(label):
                utterances.append(label)
            
            parent = parent.parent
        utterances.reverse()
        self._debug("'%s'" % utterances)
        return utterances



class Script(Gecko.Script):
    _containingPanelName = ''
    
    def __init__(self, app):
        ''' Creates a new script for the given application.

        Arguments:
        - app: the application to create a script for.
        '''
        self.debugLevel = debug.LEVEL_FINEST
        Gecko.Script.__init__(self, app)

    
    def getSpeechGenerator(self):
        '''Returns the speech generator for this script.
        '''
        return SpeechGenerator(self)

    
    def _debug(self, msg):
        ''' Convenience method for printing debug messages
        '''
        debug.println(self.debugLevel, 'Thunderbird.py: ' + msg)

    
    def onFocus(self, event):
        ''' Called whenever an object gets focus.

        Arguments:
        - event: the Event

        '''
        obj = event.source
        parent = obj.parent
        top = util.getTopLevel(obj)
        consume = False
        self._debug("onFocus: name='%s', role='%s'" % (obj.name, obj.role))
        if parent and parent.role == rolenames.ROLE_AUTOCOMPLETE:
            return None
        
        if obj.name.startswith(_('chrome://')):
            return None
        
        if obj.role == rolenames.ROLE_TABLE_CELL:
            table = parent.table
            row = table.getRowAtIndex(obj.index)
            cell = table.getAccessibleAt(row, 0)
            acc = atspi.Accessible.makeAccessible(cell)
            orca.setLocusOfFocus(event, acc)
            consume = True
        
        if top.role == rolenames.ROLE_DIALOG:
            self._speakEnclosingPanel(obj)
        
        if not consume:
            Gecko.Script.onFocus(self, event)
        

    
    def onStateChanged(self, event):
        """Called whenever an object's state changes.

        Arguments:
        - event: the Event
        """
        default.Script.onStateChanged(self, event)

    
    def onTextInserted(self, event):
        '''Called whenever text is inserted into an object.

        Arguments:
        - event: the Event
        '''
        obj = event.source
        self._debug("onTextInserted: name='%s', role='%s', parent='%s'" % (obj.name, obj.role, obj.parent.role))
        parent = obj.parent
        if parent.role == rolenames.ROLE_AUTOCOMPLETE:
            speech.stop()
            utterances = []
            (text, caretOffset, startOffset) = util.getTextLineAtCaret(obj)
            utterances.append(text)
            self._debug("onTextInserted: utterances='%s'" % utterances)
            speech.speakUtterances(utterances)
        else:
            Gecko.Script.onTextInserted(self, event)

    
    def _speakEnclosingPanel(self, obj):
        """Speak the enclosing panel for the object, if it is
        named. Going two containers up the hierarchy appears to be far
        enough to find a named panel, if there is one.  Don't speak
        panels whose name begins with 'chrome://'"""
        self._debug('_speakEnclosingPanel')
        parent = obj.parent
        if not parent:
            return None
        
        if parent.name != '' and not parent.name.startswith(_('chrome://')) and parent.role == rolenames.ROLE_PANEL:
            if parent.name != self._containingPanelName:
                self._containingPanelName = parent.name
                utterances = []
                text = _('%s panel') % parent.name
                utterances.append(text)
                speech.speakUtterances(utterances)
            
        else:
            grandparent = parent.parent
            if grandparent and grandparent.name != '' and not grandparent.name.startswith(_('chrome://')) and grandparent.role == rolenames.ROLE_PANEL:
                if grandparent.name != self._containingPanelName:
                    self._containingPanelName = grandparent.name
                    utterances = []
                    text = _('%s panel') % grandparent.name
                    utterances.append(text)
                    speech.speakUtterances(utterances)
                
            


