# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''Custom script for gaim.  This provides the ability for Orca to
monitor both the IM input and IM output text areas at the same time.

The following script specific key sequences are supported:

  Insert-h      -  Toggle whether we prefix chat room messages with
                   the name of the chat room.
  Insert-[1-9]  -  Speak and braille a previous chat room message.
'''
__id__ = '$Id: gaim.py 1584 2006-10-19 18:16:54Z richb $'
__version__ = '$Revision: 1584 $'
__date__ = '$Date: 2006-10-19 14:16:54 -0400 (Thu, 19 Oct 2006) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'
import orca.atspi as atspi
import orca.braille as braille
import orca.debug as debug
import orca.default as default
import orca.input_event as input_event
import orca.keybindings as keybindings
import orca.rolenames as rolenames
import orca.settings as settings
import orca.speech as speech
import orca.util as util
from orca.orca_i18n import _

class RingList:
    
    def __init__(self, length):
        self.__data__ = []
        self.__full__ = 0
        self.__max__ = length
        self.__cur__ = 0

    
    def append(self, x):
        if self.__full__ == 1:
            for i in range(0, self.__cur__ - 1):
                self.__data__[i] = self.__data__[i + 1]
            
            self.__data__[self.__cur__ - 1] = x
        else:
            self.__data__.append(x)
            self.__cur__ += 1
            if self.__cur__ == self.__max__:
                self.__full__ = 1
            

    
    def get(self):
        return self.__data__

    
    def remove(self):
        if self.__cur__ > 0:
            del self.__data__[self.__cur__ - 1]
            self.__cur__ -= 1
        

    
    def size(self):
        return self.__cur__

    
    def maxsize(self):
        return self.__max__

    
    def __str__(self):
        return ''.join(self.__data__)



class Script(default.Script):
    
    def __init__(self, app):
        '''Creates a new script for the given application.

        Arguments:
        - app: the application to create a script for.
        '''
        self.debugLevel = debug.LEVEL_FINEST
        self.prefixChatMessage = False
        self.MESSAGE_LIST_LENGTH = 9
        self.previousMessages = RingList(self.MESSAGE_LIST_LENGTH)
        self.previousChatRoomNames = RingList(self.MESSAGE_LIST_LENGTH)
        for i in range(0, self.previousMessages.maxsize()):
            self.previousMessages.append('')
        
        for i in range(0, self.previousChatRoomNames.maxsize()):
            self.previousChatRoomNames.append('')
        
        self.chatAreas = { }
        default.Script.__init__(self, app)

    
    def setupInputEventHandlers(self):
        '''Defines InputEventHandler fields for this script that can be
        called by the key and braille bindings. In this particular case,
        we just want to be able to add a handler to toggle whether we
        prefix chat room messages with the name of the chat room.
        '''
        debug.println(self.debugLevel, 'gaim.setupInputEventHandlers.')
        default.Script.setupInputEventHandlers(self)
        self.inputEventHandlers['togglePrefixHandler'] = input_event.InputEventHandler(Script.togglePrefix, _('Toggle whether we prefix chat room messages with the name of the chat room.'))
        self.inputEventHandlers['readPreviousMessageHandler'] = input_event.InputEventHandler(Script.readPreviousMessage, _('Speak and braille a previous chat room message.'))

    
    def getKeyBindings(self):
        '''Defines the key bindings for this script. Setup the default
        key bindings, then add one in for toggling whether we prefix
        chat room messages with the name of the chat room.

        Returns an instance of keybindings.KeyBindings.
        '''
        debug.println(self.debugLevel, 'gaim.getKeyBindings.')
        keyBindings = default.Script.getKeyBindings(self)
        keyBindings.add(keybindings.KeyBinding('h', 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['togglePrefixHandler']))
        messageKeys = [
            '1',
            '2',
            '3',
            '4',
            '5',
            '6',
            '7',
            '8',
            '9']
        for i in range(0, len(messageKeys)):
            keyBindings.add(keybindings.KeyBinding(messageKeys[i], 1 << settings.MODIFIER_ORCA, 1 << settings.MODIFIER_ORCA, self.inputEventHandlers['readPreviousMessageHandler']))
        
        return keyBindings

    
    def togglePrefix(self, inputEvent):
        ''' Toggle whether we prefix chat room messages with the name of
        the chat room.

        Arguments:
        - inputEvent: if not None, the input event that caused this action.
        '''
        debug.println(self.debugLevel, 'gaim.togglePrefix.')
        line = _('speak chat room name.')
        self.prefixChatMessage = not (self.prefixChatMessage)
        if not self.prefixChatMessage:
            line = _('Do not speak chat room name.')
        
        speech.speak(line)
        return True

    
    def utterMessage(self, chatRoomName, message):
        ''' Speak/braille a chat room message.
        messages are kept.

        Arguments:
        - chatRoomName: name of the chat room this message came from.
        - message: the chat room message.
        '''
        text = ''
        if self.prefixChatMessage:
            if chatRoomName and chatRoomName != '':
                text += _('Message from chat room ') + chatRoomName + ' '
            
        
        if message and message != '':
            text += message
        
        if text != '':
            braille.displayMessage(text)
            speech.speak(text)
        

    
    def readPreviousMessage(self, inputEvent):
        ''' Speak/braille a previous chat room message. Up to nine
        previous messages are kept.

        Arguments:
        - inputEvent: if not None, the input event that caused this action.
        '''
        debug.println(self.debugLevel, 'gaim.readPreviousMessage.')
        i = int(inputEvent.event_string)
        messageNo = self.MESSAGE_LIST_LENGTH - i
        chatRoomNames = self.previousChatRoomNames.get()
        chatRoomName = chatRoomNames[messageNo]
        messages = self.previousMessages.get()
        message = messages[messageNo]
        self.utterMessage(chatRoomName, message)

    
    def getChatRoomTab(self, obj):
        """Walk up the hierarchy until we've found the page tab for this
        chat room, and return that object.

        Arguments:
        - obj: the accessible component to start from.

        Returns the page tab component for the chat room.
        """
        if obj:
            while True:
                if obj:
                    if obj.role == rolenames.ROLE_APPLICATION:
                        break
                    elif obj.role == rolenames.ROLE_PAGE_TAB:
                        return obj
                    else:
                        obj = obj.parent
                obj.role == rolenames.ROLE_APPLICATION
                break
        

    
    def onTextInserted(self, event):
        """Called whenever text is inserted into one of Gaim's text
        objects.  If the object is an instant message or chat, speak
        the text. If we're not watching anything, do the default
        behavior.

        Arguments:
        - event: the text inserted Event
        """
        chatRoomTab = self.getChatRoomTab(event.source)
        if not chatRoomTab:
            default.Script.onTextInserted(self, event)
            return None
        
        chatArea = None
        if not self.chatAreas.has_key(chatRoomTab):
            allTextFields = util.findByRole(chatRoomTab, rolenames.ROLE_TEXT)
            index = len(allTextFields) - 2
            if index >= 0:
                self.chatAreas[chatRoomTab] = allTextFields[index]
                chatArea = self.chatAreas[chatRoomTab]
            
        else:
            chatArea = self.chatAreas[chatRoomTab]
        if event.source and event.source == chatArea:
            if self.flatReviewContext:
                self.toggleFlatReviewMode()
            
            message = self.getText(event.source, event.detail1, event.detail1 + event.detail2)
            if message and message[0] == '\n':
                message = message[1:]
            
            chatRoomName = util.getDisplayedText(chatRoomTab)
            self.utterMessage(chatRoomName, message)
            self.previousMessages.append(message)
            self.previousChatRoomNames.append(chatRoomName)
        else:
            default.Script.onTextInserted(self, event)


