# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''Custom script for gedit.'''
__id__ = '$Id: gedit.py 1664 2006-11-06 16:41:04Z wwalker $'
__version__ = '$Revision: 1664 $'
__date__ = '$Date: 2006-11-06 11:41:04 -0500 (Mon, 06 Nov 2006) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'
import orca.debug as debug
import orca.atspi as atspi
import orca.braille as braille
import orca.default as default
import orca.orca as orca
import orca.orca_state as orca_state
import orca.rolenames as rolenames
import orca.speech as speech
import orca.speechgenerator as speechgenerator
import orca.util as util
from orca.orca_i18n import _

class SpeechGenerator(speechgenerator.SpeechGenerator):
    '''Overrides _getSpeechForFrame so as to avoid digging into the
    gedit hierarchy and tickling a bug in gedit.
    '''
    
    def __init__(self, script):
        speechgenerator.SpeechGenerator.__init__(self, script)

    
    def _getSpeechForFrame(self, obj, already_focused):
        '''Get the speech for a frame.  [[[TODO: WDW - This avoids
        digging into the component hierarchy so as to avoid tickling
        a bug in GEdit (see module comment above).]]]

        Arguments:
        - obj: the frame
        - already_focused: if False, the obj just received focus

        Returns a list of utterances to be spoken for the object.
        '''
        utterances = self._getDefaultSpeech(obj, already_focused)
        self._debugGenerator('GEditSpeechGenerator._getSpeechForFrame', obj, already_focused, utterances)
        return utterances



class Script(default.Script):
    
    def __init__(self, app):
        '''Creates a new script for the given application.

        Arguments:
        - app: the application to create a script for.
        '''
        default.Script.__init__(self, app)
        self.debugLevel = debug.LEVEL_FINEST
        self.textArea = None
        self.lastCaretPosition = -1
        self.lastBadWord = ''
        self.lastEventType = ''

    
    def getSpeechGenerator(self):
        '''Returns the speech generator for this script.
        '''
        return SpeechGenerator(self)

    
    def readMisspeltWord(self, event, panel):
        '''Speak/braille the current misspelt word plus its context.
           The spell check dialog contains a "paragraph" which shows the
           context for the current spelling mistake. After speaking/brailling
           the default action for this component, that a selection of the
           surronding text from that paragraph with the misspelt word is also
           spoken.

        Arguments:
        - event: the event.
        - panel: the panel in the check spelling dialog containing the label
                 with the misspelt word.
        '''
        self.updateBraille(orca_state.locusOfFocus)
        allLabels = util.findByRole(panel, rolenames.ROLE_LABEL)
        for i in range(0, len(allLabels)):
            if allLabels[i].name.startswith(_('Change to:')) or allLabels[i].name.startswith(_('Misspelled word:')):
                continue
                continue
            badWord = allLabels[i].name
        
        if self.textArea != None:
            allText = util.findByRole(self.textArea, rolenames.ROLE_TEXT)
            caretPosition = allText[0].text.caretOffset
            debug.println(self.debugLevel, 'gedit.readMisspeltWord: type=%s  word=%s caret position=%d' % (event.type, badWord, caretPosition))
            if caretPosition == self.lastCaretPosition and badWord == self.lastBadWord and event.type == self.lastEventType:
                return None
            
            if badWord == _('Completed spell checking'):
                utterance = _('Spell checking is complete.')
                speech.speak(utterance)
                utterance = _('Press Tab and Return to terminate.')
                speech.speak(utterance)
                return None
            
            allTokens = []
            for i in range(0, len(allText)):
                text = self.getText(allText[i], 0, -1)
                tokens = text.split()
                allTokens += tokens
            
            util.speakMisspeltWord(allTokens, badWord)
            self.lastCaretPosition = caretPosition
            self.lastBadWord = badWord
            self.lastEventType = event.type
        

    
    def isFocusOnFindDialog(self):
        '''Return an indication of whether the current locus of focus is on
        the Find button or the combo box in the Find dialog.
        '''
        obj = orca_state.locusOfFocus
        rolesList1 = [
            rolenames.ROLE_PUSH_BUTTON,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_DIALOG,
            rolenames.ROLE_APPLICATION]
        rolesList2 = [
            rolenames.ROLE_COMBO_BOX,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_DIALOG,
            rolenames.ROLE_APPLICATION]
        if (util.isDesiredFocusedItem(obj, rolesList1) or obj.name == _('Find') or util.isDesiredFocusedItem(obj, rolesList2)) and obj.parent.parent.parent.parent.name == _('Find'):
            return True
        else:
            return False

    
    def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
        '''Called when the visual object with focus changes.

        Arguments:
        - event: if not None, the Event that caused the change
        - oldLocusOfFocus: Accessible that is the old locus of focus
        - newLocusOfFocus: Accessible that is the new locus of focus
        '''
        brailleGen = self.brailleGenerator
        speechGen = self.speechGenerator
        debug.printObjectEvent(self.debugLevel, event, event.source.toString())
        rolesList = [
            rolenames.ROLE_TEXT,
            rolenames.ROLE_SCROLL_PANE,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_PAGE_TAB,
            rolenames.ROLE_PAGE_TAB_LIST,
            rolenames.ROLE_SPLIT_PANE]
        if util.isDesiredFocusedItem(event.source, rolesList):
            debug.println(self.debugLevel, 'gedit.locusOfFocusChanged - text area.')
            self.textArea = event.source.parent
        
        rolesList = [
            rolenames.ROLE_TEXT,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_FRAME]
        if util.isDesiredFocusedItem(event.source, rolesList):
            frame = event.source.parent.parent.parent.parent
            if frame.name.startswith(_('Check Spelling')):
                debug.println(self.debugLevel, 'gedit.locusOfFocusChanged - check spelling dialog.')
                self.readMisspeltWord(event, event.source.parent.parent)
            
        
        default.Script.locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus)

    
    def onNameChanged(self, event):
        '''Called whenever a property on an object changes.

        Arguments:
        - event: the Event
        '''
        brailleGen = self.brailleGenerator
        speechGen = self.speechGenerator
        debug.printObjectEvent(self.debugLevel, event, event.source.toString())
        rolesList = [
            rolenames.ROLE_LABEL,
            rolenames.ROLE_PANEL,
            rolenames.ROLE_FILLER,
            rolenames.ROLE_FRAME]
        if util.isDesiredFocusedItem(event.source, rolesList):
            frame = event.source.parent.parent.parent
            if frame.name.startswith(_('Check Spelling')):
                debug.println(self.debugLevel, 'gedit.onNameChanged - check spelling dialog.')
                self.readMisspeltWord(event, event.source.parent)
            
        
        if event.source.role == rolenames.ROLE_STATUSBAR and self.isFocusOnFindDialog() and orca_state.lastInputEvent.event_string == 'Return' and event.source.name == _('Phrase not found'):
            debug.println(self.debugLevel, 'gedit.onNameChanged - phrase not found.')
            speech.speak(event.source.name)
        
        default.Script.onNameChanged(self, event)

    
    def onStateChanged(self, event):
        """Called whenever an object's state changes.

        Arguments:
        - event: the Event
        """
        if event.type == 'object:state-changed:focused' and event.detail1:
            orca.setLocusOfFocus(event, event.source)
        
        default.Script.onStateChanged(self, event)

    
    def onCaretMoved(self, event):
        '''Called whenever the caret moves.

        Arguments:
        - event: the Event
        '''
        debug.printObjectEvent(self.debugLevel, event, event.source.toString())
        if self.isFocusOnFindDialog() and orca_state.lastInputEvent.event_string == 'Return':
            debug.println(self.debugLevel, 'gedit.onCaretMoved - find dialog.')
            allComboBoxes = util.findByRole(orca_state.locusOfFocus.app, rolenames.ROLE_COMBO_BOX)
            phrase = util.getDisplayedText(allComboBoxes[0])
            (text, caretOffset, startOffset) = util.getTextLineAtCaret(event.source)
            if text.lower().find(phrase) != -1:
                speech.speak(_('Phrase found.'))
                utterances = self.speechGenerator.getSpeech(event.source, True)
                speech.speakUtterances(utterances)
            
        
        default.Script.onCaretMoved(self, event)


