# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''The gnome-terminal script mainly handles the unique interaction model
of manipulating text - both the user and the system put text into the
system and we try to determine which was which and why.'''
__id__ = '$Id: gnome-terminal.py 1925 2007-01-15 02:32:55Z wwalker $'
__version__ = '$Revision: 1925 $'
__date__ = '$Date: 2007-01-14 21:32:55 -0500 (Sun, 14 Jan 2007) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'
import orca.atspi as atspi
import orca.debug as debug
import orca.default as default
import orca.input_event as input_event
import orca.util as util
import orca.orca_state as orca_state
import orca.rolenames as rolenames
import orca.settings as settings
import orca.speech as speech

class Script(default.Script):
    
    def __init__(self, app):
        '''Creates a new script for the given application.

        Arguments:
        - app: the application to create a script for.
        '''
        default.Script.__init__(self, app)
        self.presentIfInactive = False

    
    def onTextDeleted(self, event):
        '''Called whenever text is deleted from an object.

        Arguments:
        - event: the Event
        '''
        if orca_state.lastInputEvent and isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
            event_string = orca_state.lastInputEvent.event_string
        else:
            event_string = None
        if event.source.role != rolenames.ROLE_TERMINAL or event_string != 'BackSpace':
            default.Script.onTextDeleted(self, event)
            return None
        
        if event.source != orca_state.locusOfFocus and event.source.parent != orca_state.locusOfFocus:
            return None
        
        self.updateBraille(event.source)
        character = event.any_data.decode('UTF-8')[0].encode('UTF-8')
        if character.isupper():
            speech.speak(character, self.voices[settings.UPPERCASE_VOICE])
        else:
            speech.speak(character)

    
    def onTextInserted(self, event):
        '''Called whenever text is inserted into an object.

        Arguments:
        - event: the Event
        '''
        if event.source.role != rolenames.ROLE_TERMINAL:
            default.Script.onTextInserted(self, event)
            return None
        
        if event.source != orca_state.locusOfFocus and event.source.parent != orca_state.locusOfFocus:
            return None
        
        self.updateBraille(event.source)
        text = event.any_data
        if not (orca_state.lastInputEvent) or not isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
            return None
        
        keyString = orca_state.lastInputEvent.event_string
        controlPressed = orca_state.lastInputEvent.modifiers & 1 << atspi.Accessibility.MODIFIER_CONTROL
        if keyString == 'Delete' or keyString == 'BackSpace':
            return None
        elif keyString == 'D' and controlPressed:
            text = text.decode('UTF-8')[0].decode('UTF-8')
        
        matchFound = False
        if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
            wasCommand = orca_state.lastInputEvent.modifiers & (1 << atspi.Accessibility.MODIFIER_CONTROL | 1 << atspi.Accessibility.MODIFIER_ALT | 1 << atspi.Accessibility.MODIFIER_META | 1 << atspi.Accessibility.MODIFIER_META2 | 1 << atspi.Accessibility.MODIFIER_META3)
            if not wasCommand and keyString == 'Return':
                pass
            wasCommand = keyString == 'Tab'
            if text == ' ' or keyString == 'space' or text == keyString:
                matchFound = True
            elif wasCommand or len(text) > 5:
                if text.isupper():
                    speech.speak(text, self.voices[settings.UPPERCASE_VOICE])
                else:
                    speech.speak(text)
            
        
        if settings.enableEchoByWord and util.isWordDelimiter(text.decode('UTF-8')[-1:]):
            if matchFound:
                self.echoPreviousWord(event.source)
            
        


