# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

"""Manages the settings for Orca.  This will defer to user settings first, but
fallback to local settings if the user settings doesn't exist (e.g., in the
case of gdm) or doesn't have the specified attribute."""
__id__ = '$Id: settings.py 2077 2007-03-02 23:42:02Z wwalker $'
__version__ = '$Revision: 2077 $'
__date__ = '$Date: 2007-03-02 18:42:02 -0500 (Fri, 02 Mar 2007) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'
import os
import re
import sys
screenWidth = 640
screenHeight = 480
tty = 7

try:
    import gtk.gdk as gtk
    _display = gtk.gdk.display_get_default()
    _screen = _display.get_default_screen()
    _root_window = _screen.get_root_window()
    screenWidth = _screen.get_width()
    screenHeight = _screen.get_height()
    (atom, format, data) = _root_window.property_get('XFree86_VT')
    tty = data[0]
except:
    pass

import debug
from acss import ACSS
from orca_i18n import _
userCustomizableSettings = [
    'orcaModifierKeys',
    'enableSpeech',
    'speechServerFactory',
    'speechServerInfo',
    'voices',
    'speechVerbosityLevel',
    'readTableCellRow',
    'enableSpeechIndentation',
    'enableEchoByWord',
    'enableKeyEcho',
    'enablePrintableKeys',
    'enableModifierKeys',
    'enableLockingKeys',
    'enableFunctionKeys',
    'enableActionKeys',
    'enableBraille',
    'enableBrailleGrouping',
    'brailleVerbosityLevel',
    'brailleRolenameStyle',
    'enableBrailleMonitor',
    'enableMagnifier',
    'enableMagCursor',
    'enableMagCursorExplicitSize',
    'magCursorSize',
    'magCursorColor',
    'enableMagCrossHair',
    'enableMagCrossHairClip',
    'magCrossHairSize',
    'magZoomerLeft',
    'magZoomerRight',
    'magZoomerTop',
    'magZoomerBottom',
    'magZoomFactor',
    'enableMagZoomerColorInversion',
    'magSmoothingMode',
    'magMouseTrackingMode',
    'magSourceDisplay',
    'magTargetDisplay',
    'verbalizePunctuationStyle',
    'showMainWindow',
    'keyboardLayout',
    'speakBlankLines']
mainWindowModule = 'orca_gui_main'
guiPreferencesModule = 'orca_gui_prefs'
consolePreferencesModule = 'orca_console_prefs'
quitModule = 'orca_quit'
findModule = 'orca_gui_find'
DESKTOP_MODIFIER_KEYS = [
    'Insert',
    'KP_Insert']
LAPTOP_MODIFIER_KEYS = [
    'Caps_Lock']
orcaModifierKeys = DESKTOP_MODIFIER_KEYS
MODIFIER_ORCA = 8
VERBOSITY_LEVEL_BRIEF = 0
VERBOSITY_LEVEL_VERBOSE = 1
speechVerbosityLevel = VERBOSITY_LEVEL_VERBOSE
brailleVerbosityLevel = VERBOSITY_LEVEL_VERBOSE
BRAILLE_ROLENAME_STYLE_SHORT = 0
BRAILLE_ROLENAME_STYLE_LONG = 1
brailleRolenameStyle = BRAILLE_ROLENAME_STYLE_LONG
PUNCTUATION_STYLE_NONE = 3
PUNCTUATION_STYLE_SOME = 2
PUNCTUATION_STYLE_MOST = 1
PUNCTUATION_STYLE_ALL = 0
verbalizePunctuationStyle = PUNCTUATION_STYLE_MOST
speechRateDelta = 5
speechPitchDelta = 0.5
httpServerPort = 20433
enableSpeech = True
enableSpeechCallbacks = True
silenceSpeech = False
speechFactoryModules = [
    'espeechfactory',
    'gnomespeechfactory']
speechServerFactory = 'gnomespeechfactory'
speechServerInfo = None
DEFAULT_VOICE = 'default'
UPPERCASE_VOICE = 'uppercase'
HYPERLINK_VOICE = 'hyperlink'
voices = {
    DEFAULT_VOICE: ACSS({ }),
    UPPERCASE_VOICE: ACSS({
        ACSS.AVERAGE_PITCH: 6 }),
    HYPERLINK_VOICE: ACSS({ }) }
enableSpeechIndentation = False
enableBraille = True
enableBrailleGrouping = False
enableBrailleMonitor = False
enableMagnifier = False
enableMagCursor = True
enableMagCursorExplicitSize = False
magCursorSize = 32
magCursorColor = '#000000'
enableMagCrossHair = True
enableMagCrossHairClip = False
magCrossHairSize = 16
magZoomerLeft = screenWidth / 2
magZoomerRight = screenWidth
magZoomerTop = 0
magZoomerBottom = screenHeight
magZoomFactor = 4.0
enableMagZoomerColorInversion = False
MAG_SMOOTHING_MODE_BILINEAR = 0
MAG_SMOOTHING_MODE_NONE = 1
magSmoothingMode = MAG_SMOOTHING_MODE_BILINEAR
MAG_MOUSE_TRACKING_MODE_CENTERED = 0
MAG_MOUSE_TRACKING_MODE_NONE = 1
MAG_MOUSE_TRACKING_MODE_PROPORTIONAL = 2
MAG_MOUSE_TRACKING_MODE_PUSH = 3
magMouseTrackingMode = MAG_MOUSE_TRACKING_MODE_CENTERED
magSourceDisplay = ''
magTargetDisplay = ''
enableEchoByWord = False
enableKeyEcho = False
enablePrintableKeys = True
enableModifierKeys = True
enableLockingKeys = True
enableFunctionKeys = True
enableActionKeys = True
showMainWindow = True
GENERAL_KEYBOARD_LAYOUT_DESKTOP = 1
GENERAL_KEYBOARD_LAYOUT_LAPTOP = 2
keyboardLayout = GENERAL_KEYBOARD_LAYOUT_DESKTOP
speakBlankLines = True
readTableCellRow = True
enabledTextAttributes = 'size:; family-name:; weight:400; indent:0; underline:none; strikethrough:false; justification:left; style:normal;'
repeatCharacterLimit = 4
enableCustomScripts = True
keyBindingsMap = { }
brailleBindingsMap = { }
cacheValues = True
cacheDescriptions = True
cacheAccessibles = True
learnModeEnabled = False
userPrefsDir = os.path.join(os.environ['HOME'], '.orca')
gilSleepTime = 1.0000000000000001e-05
useBlockPreventor = False
useBonoboMain = True
debugEventQueue = False
timeoutTime = 10
timeoutCallback = None
doubleClickTimeout = 0.5

def overrideKeyBindings(script, keyBindings):
    return keyBindings

settingsPackages = [
    'app-settings']
scriptPackages = [
    'orca-scripts',
    'scripts']
_scriptMappings = []

def setScriptMapping(regExpression, moduleName):
    '''Tells this module what script module to look for a given
    application name.  The mappings are stored as a list and each
    new mapping is added to the beginning of the list, meaning it
    takes precedence over all other mappings.

    Arguments:
    - regExpression: a regular expression used to match against an
                     application name
    - moduleName:    the name of the Python module containing the script
                     class definition for the application
    '''
    _scriptMappings.insert(0, [
        regExpression,
        moduleName])


def getScriptModuleName(app):
    '''Returns the module name of the script to use for a given
    application.  Any script mapping set via the setScriptMapping
    method is searched first, with the ultimate fallback being the
    name of the application itself.

    Arguments:
    - app: the application to find a script module name for
    '''
    if not app.name:
        return None
    
    for mapping in _scriptMappings:
        regExpression = mapping[0]
        moduleName = mapping[1]
        if regExpression.match(app.name):
            debug.println(debug.LEVEL_FINEST, 'Script mapping for %s is %s' % (app.name, moduleName))
            return moduleName
            continue
    
    return app.name

setScriptMapping(re.compile(_('[\\S\\s]*StarOffice[\\s\\S]*')), 'StarOffice')
setScriptMapping(re.compile(_('soffice.bin')), 'StarOffice')
setScriptMapping(re.compile(_('[Ee]volution')), 'Evolution')
setScriptMapping(re.compile(_('Deer Park')), 'Mozilla')
setScriptMapping(re.compile(_('yelp')), 'Mozilla')
setScriptMapping(re.compile(_('Bon Echo')), 'Mozilla')
setScriptMapping(re.compile(_('Minefield')), 'Mozilla')
setScriptMapping(re.compile(_('Mail/News')), 'Thunderbird')
setScriptMapping(re.compile(_('bug-buddy')), 'gnome_segv2')
setScriptMapping(re.compile(_('vte')), 'gnome-terminal')
setScriptMapping(re.compile(_('time-admin')), 'users-admin')
