# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''Manages the default speech server for orca.  A script can use this
as its speech server, or it can feel free to create one of its own.'''
__id__ = '$Id: speech.py 1973 2007-02-02 23:22:35Z wwalker $'
__version__ = '$Revision: 1973 $'
__date__ = '$Date: 2007-02-02 18:22:35 -0500 (Fri, 02 Feb 2007) $'
__copyright__ = 'Copyright (c) 2005-2006 Sun Microsystems Inc.'
__license__ = 'LGPL'
import logging
log = logging.getLogger('speech')
import time
import debug
import orca_state
import platform
import settings
from acss import ACSS
from orca_i18n import _
_speechserver = None

def getSpeechServerFactories():
    '''Imports all known SpeechServer factory modules.  Returns a list
    of modules that implement the getSpeechServers method, which
    returns a list of speechserver.SpeechServer instances.
    '''
    factories = []
    moduleNames = settings.speechFactoryModules
    for moduleName in moduleNames:
        
        try:
            module = __import__(moduleName, globals(), locals(), [
                ''])
            factories.append(module)
        continue
        debug.printException(debug.LEVEL_OFF)
        continue

    
    return factories


def init():
    global _speechserver, _speechserver
    if _speechserver:
        return None
    
    moduleName = settings.speechServerFactory
    if moduleName:
        debug.println(debug.LEVEL_CONFIGURATION, 'Using speech server factory: %s' % moduleName)
    else:
        debug.println(debug.LEVEL_CONFIGURATION, 'Speech not available.')
        return None
    factory = None
    
    try:
        factory = __import__(moduleName, globals(), locals(), [
            ''])
    except:
        
        try:
            moduleName = moduleName.replace('orca.', '', 1)
            factory = __import__(moduleName, globals(), locals(), [
                ''])
        debug.printException(debug.LEVEL_SEVERE)


    speechServerInfo = settings.speechServerInfo
    if speechServerInfo:
        _speechserver = factory.SpeechServer.getSpeechServer(speechServerInfo)
    else:
        _speechserver = factory.SpeechServer.getSpeechServer()


def __resolveACSS(acss = None):
    if acss:
        return acss
    else:
        voices = settings.voices
        return voices[settings.DEFAULT_VOICE]


def sayAll(utteranceIterator, progressCallback):
    if settings.silenceSpeech:
        return None
    
    if _speechserver:
        _speechserver.sayAll(utteranceIterator, progressCallback)
    


def speak(text, acss = None, interrupt = True):
    '''Speaks all queued text immediately.  If text is not None,
    it is added to the queue before speaking.

    Arguments:
    - text:      optional text to add to the queue before speaking
    - acss:      acss.ACSS instance; if None,
                 the default voice settings will be used.
                 Otherwise, the acss settings will be
                 used to augment/override the default
                 voice settings.
    - interrupt: if True, stops any speech in progress before
                 speaking the text
    '''
    if orca_state.lastKeyEchoTime:
        if interrupt:
            pass
        interrupt = time.time() - orca_state.lastKeyEchoTime > 0.5
    
    if settings.silenceSpeech:
        return None
    
    debug.println(debug.LEVEL_INFO, "SPEECH OUTPUT: '" + text + "'")
    log.info("'%s'" % text)
    if _speechserver:
        _speechserver.speak(text, __resolveACSS(acss), interrupt)
    


def isSpeaking():
    '''"Returns True if the system is currently speaking.'''
    if _speechserver:
        return _speechserver.isSpeaking()
    else:
        return False


def speakUtterances(utterances, acss = None, interrupt = True):
    '''Speaks the given list of utterances immediately.

    Arguments:
    - list:      list of strings to be spoken
    - acss:      acss.ACSS instance; if None,
                 the default voice settings will be used.
                 Otherwise, the acss settings will be
                 used to augment/override the default
                 voice settings.
    - interrupt: if True, stop any speech currently in progress.
    '''
    if orca_state.lastKeyEchoTime:
        if interrupt:
            pass
        interrupt = time.time() - orca_state.lastKeyEchoTime > 0.5
    
    if settings.silenceSpeech:
        return None
    
    for utterance in utterances:
        debug.println(debug.LEVEL_INFO, "SPEECH OUTPUT: '" + utterance + "'")
        log.info("'%s'" % utterance)
    
    if _speechserver:
        _speechserver.speakUtterances(utterances, __resolveACSS(acss), interrupt)
    


def stop():
    if _speechserver:
        _speechserver.stop()
    


def increaseSpeechRate(script = None, inputEvent = None):
    if _speechserver:
        _speechserver.increaseSpeechRate()
    
    return True


def decreaseSpeechRate(script = None, inputEvent = None):
    if _speechserver:
        _speechserver.decreaseSpeechRate()
    
    return True


def increaseSpeechPitch(script = None, inputEvent = None):
    if _speechserver:
        _speechserver.increaseSpeechPitch()
    
    return True


def decreaseSpeechPitch(script = None, inputEvent = None):
    if _speechserver:
        _speechserver.decreaseSpeechPitch()
    
    return True


def shutdown():
    global _speechserver
    if _speechserver:
        _speechserver.shutdownActiveServers()
        _speechserver = None
    


def reset(text = None, acss = None):
    if _speechserver:
        _speechserver.reset(text, acss)
    


def testNoSettingsInit():
    init()
    speak('testing')
    speak('this is higher', ACSS({
        'average-pitch': 7 }))
    speak('this is slower', ACSS({
        'rate': 3 }))
    speak('this is faster', ACSS({
        'rate': 80 }))
    speak('this is quiet', ACSS({
        'gain': 2 }))
    speak('this is loud', ACSS({
        'gain': 10 }))
    speak('this is normal')


def test():
    import speechserver
    factories = getSpeechServerFactories()
    for factory in factories:
        print factory.__name__
        servers = factory.SpeechServer.getSpeechServers()
        for server in servers:
            
            try:
                print '    ', server.getInfo()
                for family in server.getVoiceFamilies():
                    name = family[speechserver.VoiceFamily.NAME]
                    print '      ', name
                    acss = ACSS({
                        ACSS.FAMILY: family })
                    server.speak(name, acss)
                    server.speak('testing')
                
                server.shutdown()
            continue
            debug.printException(debug.LEVEL_OFF)
            continue

        
    

