# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''Store, load, and handle problem reports.

Copyright (C) 2006 Canonical Ltd.
Author: Martin Pitt <martin.pitt@ubuntu.com>

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
the full text of the license.
'''
import bz2
import zlib
import base64
import time
import UserDict
import sys

class ProblemReport(UserDict.IterableUserDict):
    
    def __init__(self, type = 'Crash', date = None):
        """Initialize a fresh problem report.
\t
\ttype can be 'Crash', 'Packaging', or 'Kernel'. date is the desired
\tdate/time string; if None (default), the current local time is used. """
        if date == None:
            date = time.asctime()
        
        self.data = {
            'ProblemType': type,
            'Date': date }

    
    def load(self, file, binary = True):
        '''Initialize problem report from a file-like object, using Debian
\tcontrol file format.
\t
\tif binary is False, binary data is not loaded; the dictionary key is
\tcreated, but its value will be an empty string.'''
        self.data.clear()
        key = None
        value = None
        b64_block = False
        bd = None
        for line in file:
            if line.startswith(' '):
                if b64_block and not binary:
                    continue
                
                if not key != None or value != None:
                    raise AssertionError
                if b64_block:
                    l = base64.b64decode(line)
                None if bd else None<EXCEPTION MATCH>zlib.error
                if len(value) > 0:
                    value += '\n'
                
                value += line[1:-1]
                continue
            if b64_block:
                
                try:
                    value += bd.flush()
                except AttributeError:
                    pass

                b64_block = False
                bd = None
            
            if key:
                if not value != None:
                    raise AssertionError
                self.data[key] = value
            
            (key, value) = line.split(':', 1)
            value = value.strip()
            if value == 'base64':
                value = ''
                b64_block = True
                continue
        
        if key != None:
            self.data[key] = value
        

    
    def has_removed_fields(self):
        '''Check whether the report has any keys which were not loaded in load()
\tdue to being compressed binary.'''
        return '' in self.itervalues()

    
    def _is_binary(self, string):
        '''Check if the given strings contains binary data.'''
        for c in string:
            if c < ' ' and not c.isspace():
                return True
                continue
        
        return False

    
    def write(self, file):
        """Write information into the given file-like object, using Debian
\tcontrol file format.

\tIf a value is a string, it is written directly. Otherwise it must be a
\ttuple containing a string, and an optional boolean value (in that
\torder); the first argument is interpreted as a file name, which will be
\tread and its content will become the value of this key.

\tThe second argument specifies whether the contents will be
\tbzip2'ed and base64-encoded (this defaults to True).
\t"""
        asckeys = []
        binkeys = []
        for k in self.data.keys():
            v = self.data[k]
            if hasattr(v, 'find'):
                if self._is_binary(v):
                    binkeys.append(k)
                else:
                    asckeys.append(k)
            self._is_binary(v)
            if len(v) >= 2 and not v[1]:
                asckeys.append(k)
                continue
            binkeys.append(k)
        
        asckeys.sort()
        if 'ProblemType' in asckeys:
            asckeys.remove('ProblemType')
            asckeys.insert(0, 'ProblemType')
        
        binkeys.sort()
        for k in asckeys:
            v = self.data[k]
            if not hasattr(v, 'find'):
                v = open(v[0]).read()
            
            if v.find('\n') >= 0:
                if not v.find('\n\n') < 0:
                    raise AssertionError
                print >>file, k + ':'
                print >>file, '', v.replace('\n', '\n ')
                continue
            print >>file, k + ':', v
        
        for k in binkeys:
            v = self.data[k]
            file.write(k + ': base64\n ')
            bc = zlib.compressobj()
            if hasattr(v, 'find'):
                outblock = bc.compress(v)
                if outblock:
                    file.write(base64.b64encode(outblock))
                    file.write('\n ')
                
            else:
                f = open(v[0])
                while True:
                    block = f.read(1048576)
                    if block:
                        outblock = bc.compress(block)
                        if outblock:
                            file.write(base64.b64encode(outblock))
                            file.write('\n ')
                        
                    outblock
                    break
            file.write(base64.b64encode(bc.flush()))
            file.write('\n')
        

    
    def add_to_existing(self, reportfile, keep_times = False):
        """Add the fields of this report to an already existing report
\tfile.
\t
\tThe file will be temporarily chmod'ed to 000 to prevent frontends
\tfrom picking up a hal-updated report file. If keep_times
\tis True, then the file's atime and mtime restored after updating."""
        st = os.stat(reportfile)
        
        try:
            f = open(reportfile, 'a')
            os.chmod(reportfile, 0)
            self.write(f)
            f.close()
        finally:
            if keep_times:
                os.utime(reportfile, (st.st_atime, st.st_mtime))
            
            os.chmod(reportfile, st.st_mode)


    
    def __setitem__(self, k, v):
        if not hasattr(k, 'isalnum'):
            raise AssertionError
        if not k.isalnum():
            raise AssertionError
        if not hasattr(v, 'isalnum'):
            if hasattr(v, '__getitem__'):
                if not len(v) == 1 or len(v) == 2 or v[1] in (True, False) or hasattr(v[0], 'isalnum'):
                    raise AssertionError
        return self.data.__setitem__(k, v)


import unittest
import StringIO
import tempfile
import os

class _ProblemReportTest(unittest.TestCase):
    
    def test_basic_operations(self):
        '''Test basic creation and operation.'''
        pr = ProblemReport()
        pr['foo'] = 'bar'
        pr['bar'] = ' foo   bar\nbaz\n   blip  '
        self.assertEqual(pr['foo'], 'bar')
        self.assertEqual(pr['bar'], ' foo   bar\nbaz\n   blip  ')
        self.assertEqual(pr['ProblemType'], 'Crash')
        self.assert_(time.strptime(pr['Date']))

    
    def test_ctor_arguments(self):
        '''Test non-default constructor arguments.'''
        pr = ProblemReport('Kernel')
        self.assertEqual(pr['ProblemType'], 'Kernel')
        pr = ProblemReport(date = '19801224 12:34')
        self.assertEqual(pr['Date'], '19801224 12:34')

    
    def test_sanity_checks(self):
        '''Test various error conditions.'''
        pr = ProblemReport()
        self.assertRaises(AssertionError, pr.__setitem__, 'a b', '1')
        self.assertRaises(AssertionError, pr.__setitem__, 'a', 1)
        self.assertRaises(AssertionError, pr.__setitem__, 'a', 1)
        self.assertRaises(AssertionError, pr.__setitem__, 'a', (1,))
        self.assertRaises(AssertionError, pr.__setitem__, 'a', ('/tmp/nonexistant', ''))
        self.assertRaises(KeyError, pr.__getitem__, 'Nonexistant')

    
    def test_write(self):
        '''Test write() and proper formatting.'''
        pr = ProblemReport(date = 'now!')
        pr['Simple'] = 'bar'
        pr['WhiteSpace'] = ' foo   bar\nbaz\n  blip  '
        io = StringIO.StringIO()
        pr.write(io)
        self.assertEqual(io.getvalue(), 'ProblemType: Crash\nDate: now!\nSimple: bar\nWhiteSpace:\n  foo   bar\n baz\n   blip  \n')

    
    def test_write_append(self):
        '''Test write() with appending to an existing file.'''
        pr = ProblemReport(date = 'now!')
        pr['Simple'] = 'bar'
        pr['WhiteSpace'] = ' foo   bar\nbaz\n  blip  '
        io = StringIO.StringIO()
        pr.write(io)
        pr.clear()
        pr['Extra'] = 'appended'
        pr.write(io)
        self.assertEqual(io.getvalue(), 'ProblemType: Crash\nDate: now!\nSimple: bar\nWhiteSpace:\n  foo   bar\n baz\n   blip  \nExtra: appended\n')
        temp = tempfile.NamedTemporaryFile()
        temp.write('AB' * 10 + '\x00' * 10 + 'Z')
        temp.flush()
        pr = ProblemReport(date = 'now!')
        pr['File'] = (temp.name,)
        io = StringIO.StringIO()
        pr.write(io)
        temp.close()
        pr.clear()
        pr['Extra'] = 'appended'
        pr.write(io)
        self.assertEqual(io.getvalue(), 'ProblemType: Crash\nDate: now!\nFile: base64\n eJw=\n c3RyxIAMcBAFAG55BXk=\nExtra: appended\n')

    
    def test_load(self):
        '''Test load() with various formatting.'''
        pr = ProblemReport()
        pr.load(StringIO.StringIO('ProblemType: Crash\nDate: now!\nSimple: bar\nWhiteSpace:\n  foo   bar\n baz\n   blip  \n'))
        self.assertEqual(pr['ProblemType'], 'Crash')
        self.assertEqual(pr['Date'], 'now!')
        self.assertEqual(pr['Simple'], 'bar')
        self.assertEqual(pr['WhiteSpace'], ' foo   bar\nbaz\n  blip  ')
        pr.load(StringIO.StringIO('ProblemType: Crash\nDate: now!\nSimple: bar\nWhiteSpace:\n  foo   bar\n baz\n   blip  \n \n'))
        self.assertEqual(pr['ProblemType'], 'Crash')
        self.assertEqual(pr['Date'], 'now!')
        self.assertEqual(pr['Simple'], 'bar')
        self.assertEqual(pr['WhiteSpace'], ' foo   bar\nbaz\n  blip  \n')
        pr = ProblemReport()
        pr.load(StringIO.StringIO('ProblemType: Crash\nWhiteSpace:\n  foo   bar\n baz\n   blip  \nLast: foo\n'))
        self.assertEqual(pr['WhiteSpace'], ' foo   bar\nbaz\n  blip  ')
        self.assertEqual(pr['Last'], 'foo')
        pr.load(StringIO.StringIO('ProblemType: Crash\nWhiteSpace:\n  foo   bar\n baz\n   blip  \nLast: foo\n \n'))
        self.assertEqual(pr['WhiteSpace'], ' foo   bar\nbaz\n  blip  ')
        self.assertEqual(pr['Last'], 'foo\n')
        pr.load(StringIO.StringIO('ProblemType: Crash'))
        self.assertEqual(pr.keys(), [
            'ProblemType'])

    
    def test_write_file(self):
        '''Test writing a report with binary file data.'''
        temp = tempfile.NamedTemporaryFile()
        temp.write('AB' * 10 + '\x00' * 10 + 'Z')
        temp.flush()
        pr = ProblemReport(date = 'now!')
        pr['File'] = (temp.name,)
        pr['Afile'] = (temp.name,)
        io = StringIO.StringIO()
        pr.write(io)
        temp.close()
        self.assertEqual(io.getvalue(), 'ProblemType: Crash\nDate: now!\nAfile: base64\n eJw=\n c3RyxIAMcBAFAG55BXk=\nFile: base64\n eJw=\n c3RyxIAMcBAFAG55BXk=\n')
        temp = tempfile.NamedTemporaryFile()
        temp.write('foo\x00bar')
        temp.flush()
        pr = ProblemReport(date = 'now!')
        pr['File'] = (temp.name, False)
        io = StringIO.StringIO()
        pr.write(io)
        self.assertEqual(io.getvalue(), 'ProblemType: Crash\nDate: now!\nFile: foo\x00bar\n')
        pr['File'] = (temp.name, True)
        io = StringIO.StringIO()
        pr.write(io)
        self.assertEqual(io.getvalue(), 'ProblemType: Crash\nDate: now!\nFile: base64\n eJw=\n S8vPZ0hKLAIACfACeg==\n')
        temp.close()

    
    def test_read_file(self):
        '''Test reading a report with binary data.'''
        bin_report = 'ProblemType: Crash\nDate: now!\nFile: base64\n eJw=\n c3RyxIAMcBAFAG55BXk=\nFoo: Bar\n'
        pr = ProblemReport()
        pr.load(StringIO.StringIO(bin_report))
        self.assertEqual(pr['File'], 'AB' * 10 + '\x00' * 10 + 'Z')
        self.assertEqual(pr.has_removed_fields(), False)
        pr.load(StringIO.StringIO(bin_report), binary = False)
        self.assertEqual(pr['File'], '')
        self.assertEqual(pr.has_removed_fields(), True)

    
    def test_read_file_bzip2(self):
        '''Test reading a report with binary data (legacy bzip2 compression).'''
        bin_report = 'ProblemType: Crash\nDate: now!\nFile: base64\n QlpoOTFBWSZTWc5ays4AAAdGAEEAMAAAECAAMM0AkR6fQsBSDhdyRThQkM5ays4=\nFoo: Bar\n'
        pr = ProblemReport()
        pr.load(StringIO.StringIO(bin_report))
        self.assertEqual(pr['File'], 'AB' * 10 + '\x00' * 10 + 'Z')
        self.assertEqual(pr.has_removed_fields(), False)
        pr.load(StringIO.StringIO(bin_report), binary = False)
        self.assertEqual(pr['File'], '')
        self.assertEqual(pr.has_removed_fields(), True)

    
    def test_big_file(self):
        '''Test writing and re-decoding a big random file.'''
        temp = tempfile.NamedTemporaryFile()
        data = os.urandom(1048576)
        temp.write(data)
        temp.flush()
        pr = ProblemReport()
        pr['File'] = (temp.name,)
        pr['Before'] = 'xtestx'
        pr['ZAfter'] = 'ytesty'
        io = StringIO.StringIO()
        pr.write(io)
        temp.close()
        io.seek(0)
        pr = ProblemReport()
        pr.load(io)
        self.assert_(pr['File'] == data)
        self.assertEqual(pr['Before'], 'xtestx')
        self.assertEqual(pr['ZAfter'], 'ytesty')
        io2 = StringIO.StringIO()
        pr.write(io2)
        self.assert_(io.getvalue() == io2.getvalue())

    
    def test_iter(self):
        '''Test ProblemReport iteration.'''
        pr = ProblemReport()
        pr['foo'] = 'bar'
        keys = []
        for k in pr:
            keys.append(k)
        
        keys.sort()
        self.assertEqual(' '.join(keys), 'Date ProblemType foo')
        []([](_[1]), 2)

    
    def test_modify(self):
        '''Test reading, modifying fields, and writing back.'''
        report = 'ProblemType: Crash\nDate: now!\nLong:\n xxx\n .\n yyy\nShort: Bar\nFile: base64\n eJw=\n c3RyxIAMcBAFAG55BXk=\n'
        pr = ProblemReport()
        pr.load(StringIO.StringIO(report))
        self.assertEqual(pr['Long'], 'xxx\n.\nyyy')
        io = StringIO.StringIO()
        pr.write(io)
        self.assertEqual(io.getvalue(), report)
        pr['Short'] = 'aaa\nbbb'
        pr['Long'] = '123'
        io = StringIO.StringIO()
        pr.write(io)
        self.assertEqual(io.getvalue(), 'ProblemType: Crash\nDate: now!\nLong: 123\nShort:\n aaa\n bbb\nFile: base64\n eJw=\n c3RyxIAMcBAFAG55BXk=\n')

    
    def test_add_to_existing(self):
        '''Test adding information to an existing report.'''
        pr = ProblemReport()
        pr['old1'] = '11'
        pr['old2'] = '22'
        (fd, rep) = tempfile.mkstemp()
        os.close(fd)
        pr.write(open(rep, 'w'))
        origstat = os.stat(rep)
        pr = ProblemReport()
        pr.clear()
        pr['new1'] = '33'
        pr.add_to_existing(rep, keep_times = True)
        newstat = os.stat(rep)
        self.assertEqual(origstat.st_mode, newstat.st_mode)
        self.assertEqual(origstat.st_atime, newstat.st_atime)
        self.assertEqual(origstat.st_mtime, newstat.st_mtime)
        newpr = ProblemReport()
        newpr.load(open(rep))
        self.assertEqual(newpr['old1'], '11')
        self.assertEqual(newpr['old2'], '22')
        self.assertEqual(newpr['new1'], '33')
        time.sleep(1)
        open(rep).read()
        time.sleep(1)
        pr = ProblemReport()
        pr.clear()
        pr['new2'] = '44'
        pr.add_to_existing(rep)
        newstat = os.stat(rep)
        self.assertEqual(origstat.st_mode, newstat.st_mode)
        self.assertNotEqual(origstat.st_mtime, newstat.st_mtime)
        self.assertNotEqual(origstat.st_atime, newstat.st_atime)
        newpr = ProblemReport()
        newpr.load(open(rep))
        self.assertEqual(newpr['old1'], '11')
        self.assertEqual(newpr['old2'], '22')
        self.assertEqual(newpr['new1'], '33')
        self.assertEqual(newpr['new2'], '44')
        os.unlink(rep)


if __name__ == '__main__':
    unittest.main()

