# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

'''
This is the window widget which will contain the audio mastering widget
defined in audio_widgets.AudioMastering. 
'''
import os
import os.path as os
import gtk
import gtk.glade as gtk
import gobject
import statvfs
from gettext import gettext as _
import operations
import urlutil
from mastering import GtkMusicList, MusicListGateway
from mastering import ErrorTrapper
from recording import ConvertAndWrite
from preferences import RecordingPreferences
from components import Component
from mainwindow import SerpentineWindow
from common import SerpentineNotSupportedError, SerpentineCacheError
from plugins import plugins
PATTERN_SEPARATOR = ';'

def _validate_music_list(music_list, preferences):
    secs = 0
    missing_musics = []
    for music in music_list:
        if not urlutil.exists(music['location']):
            missing_musics.append('%s - %s' % (music['title'], music['artist']))
        
        if not preferences.pool.is_available(music['location']):
            secs += music['duration']
            continue
    
    if len(missing_musics) > 0:
        raise SerpentineCacheError(SerpentineCacheError.INVALID, '%s\n%s' % (_('The following musics have missing files:'), '\n'.join(missing_musics)))
    
    size_needed = secs * 0x2B110L
    for tmp in preferences.get_temporary_dirs():
        
        try:
            s = os.statvfs(tmp)
        except OSError:
            continue

        size_avail = s[statvfs.F_BAVAIL] * long(s[statvfs.F_BSIZE])
        if size_avail - size_needed < 0:
            preferences.pool.clear()
        
        size_avail = s[statvfs.F_BAVAIL] * long(s[statvfs.F_BSIZE])
        if size_avail - size_needed >= 0:
            preferences.temporary_dir = tmp
            return None
            continue
    
    raise SerpentineCacheError(SerpentineCacheError.INVALID, _('Please check if the cache location exists and has writable permissions.'))


class SavePlaylistRegistry(Component):
    
    def __init__(self, parent):
        super(SavePlaylistRegistry, self).__init__(parent)
        all_files = gtk.FileFilter()
        all_files.set_name(_('All files'))
        all_files.add_pattern('*.*')
        self._SavePlaylistRegistry__global_filter = gtk.FileFilter()
        self._SavePlaylistRegistry__global_filter.set_name(_('Supported playlists'))
        self._SavePlaylistRegistry__file_filters = [
            self._SavePlaylistRegistry__global_filter,
            all_files]
        self._SavePlaylistRegistry__factories = { }
        self.listeners = []

    file_filters = property((lambda self: self._SavePlaylistRegistry__file_filters))
    
    def register(self, factory, extension, description):
        self._SavePlaylistRegistry__global_filter.add_pattern('*' + extension)
        filter = gtk.FileFilter()
        filter.set_name(description)
        filter.add_pattern('*' + extension)
        self.file_filters.append(filter)
        for listener in self.listeners:
            listener.on_registred(factory, extension, description)
        
        self._SavePlaylistRegistry__factories[extension] = factory
        self._SavePlaylistRegistry__global_filter.add_pattern('*' + extension)

    
    def save(self, filename, extension = None):
        if extension is None:
            (fname, extension) = os.path.splitext(filename)
        
        
        try:
            return self._SavePlaylistRegistry__factories[extension](self.parent.music_list, filename)
        except KeyError:
            raise SerpentineNotSupportedError(extension)




class Application(operations.Operation, Component):
    components = ()
    
    def __init__(self, locations):
        operations.Operation.__init__(self)
        Component.__init__(self, None)
        self.savePlaylist = SavePlaylistRegistry(self)
        self.locations = locations
        self._Application__preferences = RecordingPreferences(locations)
        self._Application__running_ops = []
        self._music_file_patterns = { }
        self._playlist_file_patterns = { }
        self._music_file_filters = None
        self._playlist_file_filters = None
        self.register_music_file_pattern('MPEG Audio Stream, Layer III', '*.mp3')
        self.register_music_file_pattern('Ogg Vorbis Codec Compressed WAV File', '*.ogg')
        self.register_music_file_pattern('Free Lossless Audio Codec', '*.flac')
        self.register_music_file_pattern('MPEG 4 Audio', '*.m4a;*.mp4')
        self.register_music_file_pattern('PCM Wave audio', '*.wav')

    
    def _load_plugins(self):
        self._Application__plugins = []
        for plug in plugins:
            self._Application__plugins.append(plugins[plug].create_plugin(self))
        

    cache_pool = property((lambda self: self._Application__preferences.pool))
    preferences = property((lambda self: self._Application__preferences))
    running_ops = property((lambda self: self._Application__running_ops))
    can_stop = property((lambda self: len(self.running_ops) == 0))
    can_start = property((lambda self: self._Application__window is not None))
    
    def on_finished(self, event):
        self._Application__running_ops.remove(event.source)
        if self.can_stop:
            self.stop()
        

    
    def stop(self):
        if not self.can_stop:
            raise AssertionError, 'Check if you can stop the operation first.'
        self.preferences.savePlaylist(self.music_list)
        self.preferences.pool.clear()
        self._send_finished_event(operations.SUCCESSFUL)
        del self._Application__plugins

    validated = False
    
    def validate_files(self):
        '''Call this before calling write files'''
        music_list = self.music_list_gw.music_list
        _validate_music_list(music_list, self.preferences)
        self.validated = True

    
    def write_files(self, window = None):
        if not self.validated:
            raise ValueError('Call validate_files first')
        
        r = ConvertAndWrite(self.music_list, self.preferences, window)
        self.running_ops.append(r)
        r.listeners.append(self)
        self.validated = False
        return r

    write_files = operations.async(write_files)
    
    def add_hints_filter(self, location_filter):
        self.music_list_gw.add_hints_filter(location_filter)

    
    def remove_hints_filter(self, location_filter):
        self.music_list_gw.remove_hints_filter(location_filter)

    
    def register_music_file_pattern(self, name, pattern):
        """
        Music patterns are meant to be used in the file dialog for adding
        musics to the playlist.
        
        To associate more then one pattern to the same name use the ';' char
        as the pattern separator.
        """
        self._music_file_patterns[pattern.strip(PATTERN_SEPARATOR)] = name
        self._music_file_filters = None

    
    def register_playlist_file_pattern(self, name, pattern):
        '''Playlist patterns are meant to be used in the file dialog for adding
        playlist contents to the current playlist.'''
        self._playlist_file_patterns[pattern] = name
        self._playlist_file_filters = None

    
    def __gen_file_filters(self, patterns, all_filters_name):
        all_files = gtk.FileFilter()
        all_files.set_name(_('All files'))
        all_files.add_pattern('*.*')
        all_musics = gtk.FileFilter()
        all_musics.set_name(all_filters_name)
        filters = [
            all_musics]
        for pattern, name in patterns.iteritems():
            patterns = pattern.split(PATTERN_SEPARATOR)
            for patt in patterns:
                all_musics.add_pattern(patt)
            
            filter = gtk.FileFilter()
            filter.set_name(name)
            for patt in patterns:
                filter.add_pattern(patt)
            
            filters.append(filter)
        
        filters.append(all_files)
        return filters

    
    def __get_file_filter(self, filter_attr, pattern_attr, name):
        file_filter = getattr(self, filter_attr)
        if file_filter is not None:
            return file_filter
        
        setattr(self, filter_attr, self._Application__gen_file_filters(getattr(self, pattern_attr), name))
        return getattr(self, filter_attr)

    music_file_filters = property((lambda self: self._Application__get_file_filter('_music_file_filters', '_music_file_patterns', _('Common media files'))))
    playlist_file_filters = property((lambda self: self._Application__get_file_filter('_playlist_file_filters', '_playlist_file_patterns', _('Supported playlists'))))
    
    def add_files(self, files):
        '''Returns an operation'''
        return self.music_list_gw.add_files(files)

    add_files = operations.async(add_files)
    
    def clear_files(self):
        self.music_list.clear()



class HeadlessApplication(Application):
    
    class Gateway(MusicListGateway):
        
        def __init__(self, app):
            MusicListGateway.__init__(self, app)
            self.music_list = GtkMusicList()

        
        class Handler:
            
            def prepare_queue(self, gateway, queue):
                self.trapper = ErrorTrapper(None)

            
            def prepare_add_file(self, gateway, add_file):
                add_file.listeners.append(self.trapper)

            
            def finish_queue(self, gateway, queue):
                queue.append(self.trapper)
                del self.trapper



    
    def __init__(self, locations):
        Application.__init__(self, locations)
        self.music_list_gw = HeadlessApplication.Gateway(self)
        self._load_plugins()

    music_list = property((lambda self: self.music_list_gw.music_list))


class SerpentineApplication(Application):
    '''When no operations are left in SerpentineApplication it will exit.
    An operation can be writing the files to cd or showing the main window.
    This enables us to close the main window and continue showing the progress
    dialog. This object should be simple enough for D-Bus export.
    '''
    
    def __init__(self, locations):
        Application.__init__(self, locations)
        self._SerpentineApplication__window = SerpentineWindow(self)
        self.preferences.dialog.set_transient_for(self.window_widget)
        self._SerpentineApplication__window.listeners.append(self)
        self._load_plugins()

    window_widget = property((lambda self: self._SerpentineApplication__window))
    music_list = property((lambda self: self.window_widget.music_list))
    music_list_gw = property((lambda self: self.window_widget.masterer.music_list_gateway))
    
    def write_files(self):
        return Application.write_files(self, self.window_widget)

    
    def show_window(self):
        self._SerpentineApplication__window.start()
        self.running_ops.append(self._SerpentineApplication__window)

    
    def close_window(self):
        if self._SerpentineApplication__window.running:
            self._SerpentineApplication__window.stop()
        

    
    def stop(self):
        Application.stop(self)
        self._SerpentineApplication__window.destroy()
        del self._SerpentineApplication__window


gobject.type_register(SerpentineWindow)
