# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

import nautilusburn
import gtk
import gobject
import operations
import gtkutil
import gst
from nautilusburn import AudioTrack
from operations import OperationsQueueListener, MeasurableOperation
from converting import FetchMusicList
from gettext import gettext as _

class ConvertingError:
    
    def __init__(self, win):
        self.win = win

    
    def on_finished(self, evt):
        if evt.id != operations.ERROR:
            return None
        
        err = evt.error
        if isinstance(err, gst.GError) and err.code == gst.RESOURCE_ERROR_NOT_FOUND:
            gtkutil.dialog_error(_('Converting files failed'), _('Some of the files were missing. The disc is still usable.'), parent = self.win)
            return None
        
        gtkutil.dialog_error(_('Converting files failed'), _("Writing to disc didn't start so it is still usable."), parent = self.win)



class WritingError:
    
    def __init__(self, win):
        self.win = win

    
    def on_finished(self, evt):
        if evt.id == operations.SUCCESSFUL:
            return None
        
        if evt.id == operations.ERROR:
            title = _('Writing to disc failed')
        else:
            title = _('Writing to disc canceled')
        if evt.error is None:
            msg = _('The writing operation has started. The disc may be unusable.')
        else:
            msg = str(evt.error)
        gtkutil.dialog_error(title, str(msg), parent = self.win)



class ConvertAndWrite(MeasurableOperation, OperationsQueueListener):
    '''
    ConvertAndWrite class is yet another operation. When this operation is
    started it will show the user the dialog for the recording operation.
    It is supposed to be a one time operation.
    '''
    
    def __init__(self, music_list, preferences, parent = None):
        MeasurableOperation.__init__(self)
        self._ConvertAndWrite__queue = operations.OperationsQueue()
        self._ConvertAndWrite__queue.listeners.append(self)
        self._ConvertAndWrite__parent = parent
        self._ConvertAndWrite__prog = gtkutil.HigProgress()
        self._ConvertAndWrite__prog.primary_text = _('Writing Audio Disc')
        self._ConvertAndWrite__prog.secondary_text = _('The audio tracks are going to be written to a disc. This operation may take a long time, depending on data size and write speed.')
        self._ConvertAndWrite__prog.connect('destroy-event', self._ConvertAndWrite__on_prog_destroyed)
        self._ConvertAndWrite__prog.cancel_button.connect('clicked', self._ConvertAndWrite__on_cancel)
        self._ConvertAndWrite__prog.close_button.connect('clicked', self._ConvertAndWrite__on_close)
        self._ConvertAndWrite__prog.set_icon_name('gnome-dev-cdrom-audio')
        self._ConvertAndWrite__music_list = music_list
        self._ConvertAndWrite__preferences = preferences
        self._ConvertAndWrite__drive = preferences.drive
        self._ConvertAndWrite__can_start = True

    preferences = property((lambda self: self._ConvertAndWrite__preferences))
    can_stop = property((lambda self: self._ConvertAndWrite__queue.can_stop))
    can_start = property((lambda self: self._ConvertAndWrite__can_start))
    running = property((lambda self: self._ConvertAndWrite__queue.running))
    progress = property((lambda self: self._ConvertAndWrite__queue.progress))
    drive = property((lambda self: self._ConvertAndWrite__drive))
    
    def __on_prog_destroyed(self, *args):
        if self.cancel_button.is_sensitive():
            self._ConvertAndWrite__prog.hide()
            self._ConvertAndWrite__on_cancel()
        
        return False

    
    def start(self):
        self._ConvertAndWrite__can_start = False
        self._ConvertAndWrite__prog.show()
        if self.preferences.drive.get_media_type() == nautilusburn.MEDIA_TYPE_CDRW:
            gtkutil.dialog_warn(_('CD-RW disc will be erased'), _("Please remove your disc if you want to preserve it's contents."), parent = self._ConvertAndWrite__prog)
        
        self._ConvertAndWrite__blocked = False
        self.preferences.pool.temporary_dir = self.preferences.temporary_dir
        oper = FetchMusicList(self._ConvertAndWrite__music_list, self.preferences.pool)
        self._ConvertAndWrite__filenames = []
        oper.listeners.append(MusicListToFilenames(self._ConvertAndWrite__music_list, self._ConvertAndWrite__filenames))
        oper.listeners.append(ConvertingError(self._ConvertAndWrite__prog))
        self._ConvertAndWrite__fetching = oper
        self._ConvertAndWrite__queue.append(oper)
        oper = WriteAudioDisc(self._ConvertAndWrite__filenames, self.preferences, self._ConvertAndWrite__prog)
        oper.recorder.connect('progress-changed', self._ConvertAndWrite__tick)
        oper.recorder.connect('action-changed', self._ConvertAndWrite__on_action_changed)
        oper.listeners.append(WritingError(self._ConvertAndWrite__prog))
        self._ConvertAndWrite__queue.append(oper)
        self._ConvertAndWrite__recording = oper
        self._ConvertAndWrite__queue.start()
        self._ConvertAndWrite__source = gobject.timeout_add(300, self._ConvertAndWrite__tick)

    
    def stop(self):
        self._ConvertAndWrite__on_cancel()

    
    def __tick(self, *args):
        if self._ConvertAndWrite__queue.running:
            self._ConvertAndWrite__prog.progress_fraction = self._ConvertAndWrite__queue.progress
        
        return True

    
    def __on_cancel(self, *args):
        if self.can_stop:
            self._ConvertAndWrite__prog.cancel_button.set_sensitive(False)
            self._ConvertAndWrite__queue.stop()
        

    
    def __on_close(self, *args):
        self._ConvertAndWrite__prog.destroy()

    
    def __on_action_changed(self, recorder, action, media):
        if action == nautilusburn.RECORDER_ACTION_PREPARING_WRITE:
            self._ConvertAndWrite__prog.sub_progress_text = _('Preparing recorder')
        elif action == nautilusburn.RECORDER_ACTION_WRITING:
            self._ConvertAndWrite__prog.sub_progress_text = _('Writing media files to disc')
        elif action == nautilusburn.RECORDER_ACTION_FIXATING:
            self._ConvertAndWrite__prog.sub_progress_text = _('Fixating disc')
        

    
    def before_operation_starts(self, evt, oper):
        if oper == self._ConvertAndWrite__fetching:
            self._ConvertAndWrite__prog.sub_progress_text = _('Preparing media files')
        else:
            self._ConvertAndWrite__blocked = True

    
    def on_finished(self, evt):
        gobject.source_remove(self._ConvertAndWrite__source)
        if evt.id == operations.SUCCESSFUL:
            gtkutil.dialog_info(_('Writing to disc finished'), _('Disc writing was successful.'), parent = self._ConvertAndWrite__prog)
        
        self._propagate(evt)
        self._ConvertAndWrite__on_close()



class MusicListToFilenames:
    
    def __init__(self, music_list, filenames):
        self.music_list = music_list
        self.filenames = filenames

    
    def on_finished(self, evt):
        for row in self.music_list:
            self.filenames.append(row['cache_location'])
        



class WriteAudioDisc(MeasurableOperation):
    
    def __init__(self, music_list, preferences, parent = None):
        MeasurableOperation.__init__(self)
        self.music_list = music_list
        self._WriteAudioDisc__progress = 0.0
        self._WriteAudioDisc__running = False
        self.parent = parent
        self._WriteAudioDisc__recorder = nautilusburn.Recorder()
        self._WriteAudioDisc__preferences = preferences

    title = _('Writing Files to Disc')
    progress = property((lambda self: self._WriteAudioDisc__progress))
    running = property((lambda self: self._WriteAudioDisc__running))
    recorder = property((lambda self: self._WriteAudioDisc__recorder))
    preferences = property((lambda self: self._WriteAudioDisc__preferences))
    
    def start(self):
        self._WriteAudioDisc__running = True
        tracks = []
        for filename in self.music_list:
            tracks.append(AudioTrack(filename))
        
        if len(tracks) == 0:
            self._send_finished_event(operations.ERROR, _('There were no valid files for writing. Please select at least one.'))
            return None
        
        self.recorder.connect('progress-changed', self._WriteAudioDisc__on_progress)
        self.recorder.connect('insert-media-request', self._WriteAudioDisc__insert_cd)
        self.recorder.connect('warn-data-loss', self._WriteAudioDisc__on_data_loss)
        gobject.idle_add(self._WriteAudioDisc__thread, tracks)

    
    def __on_data_loss(self, recorder):
        return False

    
    def stop(self):
        self.recorder.cancel(False)

    
    def __on_progress(self, source, progress, seconds = None):
        self._WriteAudioDisc__progress = progress

    
    def __thread(self, tracks):
        error = None
        
        try:
            result = self.recorder.write_tracks(self.preferences.drive, tracks, self.preferences.speedWrite, self.preferences.writeFlags)
        except gobject.GError:
            err = None
            result = nautilusburn.RECORDER_RESULT_ERROR
            import traceback
            traceback.print_exc()
            error = str(err)

        if result == nautilusburn.RECORDER_RESULT_FINISHED:
            result = operations.SUCCESSFUL
        elif result == nautilusburn.RECORDER_RESULT_ERROR:
            result = operations.ERROR
        elif result == nautilusburn.RECORDER_RESULT_CANCEL:
            result == operations.ABORTED
        elif result == nautilusburn.RECORDER_RESULT_RETRY:
            result == operations.ERROR
        
        self._send_finished_event(result, error)

    
    def __insert_cd(self, rec, reload_media, can_rewrite, busy_cd):
        if busy_cd:
            msg = _('Please make sure another application is not using the drive.')
            title = _('Drive is busy')
        elif not reload_media and can_rewrite:
            msg = _('Please put a rewritable or blank disc into the drive.')
            title = _('Insert rewritable or blank disc')
        elif not reload_media and not can_rewrite:
            msg = _('Please put a blank disc into the drive.')
            title = _('Insert blank disc')
        elif can_rewrite:
            msg = _('Please replace the disc in the drive with a rewritable or blank disc.')
            title = _('Reload rewritable or blank disc')
        else:
            msg = _('Please replace the disc in the drive a blank disc.')
            title = _('Reload blank disc')
        return gtkutil.dialog_ok_cancel(title, msg, parent = self.parent) == gtk.RESPONSE_OK


if __name__ == '__main__':
    import sys
    import gobject
    
    class MyListener:
        
        def on_finished(self, evt):
            gtk.main_quit()


    
    def print_progress(oper):
        return True

    w = gtk.Window(gtk.WINDOW_TOPLEVEL)
    w.add(gtk.Label('---'))
    w.show_all()
    d = nautilusburn.get_drives_list(False)[0]
    music_lst = [
        {
            'filename': sys.argv[1] }]
    r = RecordMusicList(music_lst, d, d.get_max_speed_write(), 0)
    r.listeners.append(MyListener())
    gobject.timeout_add(250, print_progress, r)
    r.start()
    gtk.main()

