# Source Generated with Decompyle++
# File: in.pyc (Python 2.4)

"""Append module search paths for third-party packages to sys.path.

****************************************************************
* This module is automatically imported during initialization. *
****************************************************************

In earlier versions of Python (up to 1.5a3), scripts or modules that
needed to use site-specific modules would place ``import site''
somewhere near the top of their code.  Because of the automatic
import, this is no longer necessary (but code that does it still
works).

This will append site-specific paths to the module search path.  On
Unix, it starts with sys.prefix and sys.exec_prefix (if different) and
appends lib/python<version>/site-packages as well as lib/site-python.
On other platforms (mainly Mac and Windows), it uses just sys.prefix
(and sys.exec_prefix, if different, but this is unlikely).  The
resulting directories, if they exist, are appended to sys.path, and
also inspected for path configuration files.

FOR DEBIAN, this sys.path is augmented with directories in /usr/local.
Local addons go into /usr/local/lib/python<version>/site-packages
(resp. /usr/local/lib/site-python), Debian addons install into
/usr/{lib,share}/python<version>/site-packages.

A path configuration file is a file whose name has the form
<package>.pth; its contents are additional directories (one per line)
to be added to sys.path.  Non-existing directories (or
non-directories) are never added to sys.path; no directory is added to
sys.path more than once.  Blank lines and lines beginning with
'#' are skipped. Lines starting with 'import' are executed.

For example, suppose sys.prefix and sys.exec_prefix are set to
/usr/local and there is a directory /usr/local/lib/python2.3/site-packages
with three subdirectories, foo, bar and spam, and two path
configuration files, foo.pth and bar.pth.  Assume foo.pth contains the
following:

  # foo package configuration
  foo
  bar
  bletch

and bar.pth contains:

  # bar package configuration
  bar

Then the following directories are added to sys.path, in this order:

  /usr/local/lib/python2.3/site-packages/bar
  /usr/local/lib/python2.3/site-packages/foo

Note that bletch is omitted because it doesn't exist; bar precedes foo
because bar.pth comes alphabetically before foo.pth; and spam is
omitted because it is not mentioned in either path configuration file.

After these path manipulations, an attempt is made to import a module
named sitecustomize, which can perform arbitrary additional
site-specific customizations.  If this import fails with an
ImportError exception, it is silently ignored.

"""
import sys
import os
import __builtin__

def makepath(*paths):
    dir = os.path.abspath(os.path.join(*paths))
    return (dir, os.path.normcase(dir))


def abs__file__():
    """Set all module' __file__ attribute to an absolute path"""
    for m in sys.modules.values():
        
        try:
            m.__file__ = os.path.abspath(m.__file__)
        continue
        except AttributeError:
            continue
            continue
        

    


def removeduppaths():
    ''' Remove duplicate entries from sys.path along with making them
    absolute'''
    L = []
    known_paths = set()
    for dir in sys.path:
        (dir, dircase) = makepath(dir)
        if dircase not in known_paths:
            L.append(dir)
            known_paths.add(dircase)
            continue
    
    sys.path[:] = L
    return known_paths


def addbuilddir():
    """Append ./build/lib.<platform> in case we're running in the build dir
    (especially for Guido :-)"""
    get_platform = get_platform
    import distutils.util
    if not sys.pydebug or '_d':
        pass
    s = 'build/lib%s.%s-%.3s' % ('', get_platform(), sys.version)
    s = os.path.join(os.path.dirname(sys.path[-1]), s)
    sys.path.append(s)


def _init_pathinfo():
    '''Return a set containing all existing directory entries from sys.path'''
    d = set()
    for dir in sys.path:
        
        try:
            if os.path.isdir(dir):
                (dir, dircase) = makepath(dir)
                d.add(dircase)
        continue
        except TypeError:
            continue
            continue
        

    
    return d


def addpackage(sitedir, name, known_paths):
    """Add a new path to known_paths by combining sitedir and 'name' or execute
    sitedir if it starts with 'import'"""
    if known_paths is None:
        _init_pathinfo()
        reset = 1
    else:
        reset = 0
    fullname = os.path.join(sitedir, name)
    
    try:
        f = open(fullname, 'rU')
    except IOError:
        return None

    
    try:
        for line in f:
            if line.startswith('#'):
                continue
            
            if line.startswith('import'):
                exec line
                continue
            
            line = line.rstrip()
            (dir, dircase) = makepath(sitedir, line)
            if dircase not in known_paths and os.path.exists(dir):
                sys.path.append(dir)
                known_paths.add(dircase)
                continue
    finally:
        f.close()

    if reset:
        known_paths = None
    
    return known_paths


def addsitedir(sitedir, known_paths = None):
    """Add 'sitedir' argument to sys.path if missing and handle .pth files in
    'sitedir'"""
    if known_paths is None:
        known_paths = _init_pathinfo()
        reset = 1
    else:
        reset = 0
    (sitedir, sitedircase) = makepath(sitedir)
    if sitedircase not in known_paths:
        sys.path.append(sitedir)
    
    
    try:
        names = os.listdir(sitedir)
    except os.error:
        return None

    names.sort()
    for name in names:
        if name.endswith(os.extsep + 'pth'):
            addpackage(sitedir, name, known_paths)
            continue
    
    if reset:
        known_paths = None
    
    return known_paths


def addsitepackages(known_paths):
    '''Add site-packages (and possibly site-python) to sys.path'''
    prefixes = [
        os.path.join(sys.prefix, 'local'),
        sys.prefix]
    if sys.exec_prefix != sys.prefix:
        prefixes.append(os.path.join(sys.exec_prefix, 'local'))
    
    for prefix in prefixes:
        if prefix:
            if sys.platform in ('os2emx', 'riscos'):
                sitedirs = [
                    os.path.join(prefix, 'Lib', 'site-packages')]
            elif os.sep == '/':
                sitedirs = [
                    os.path.join(prefix, 'lib', 'python' + sys.version[:3], 'site-packages'),
                    os.path.join(prefix, 'lib', 'site-python')]
            else:
                sitedirs = [
                    prefix,
                    os.path.join(prefix, 'lib', 'site-packages')]
            if sys.platform == 'darwin':
                if 'Python.framework' in prefix:
                    home = os.environ.get('HOME')
                    if home:
                        sitedirs.append(os.path.join(home, 'Library', 'Python', sys.version[:3], 'site-packages'))
                    
                
            
            for sitedir in sitedirs:
                if os.path.isdir(sitedir):
                    addsitedir(sitedir, known_paths)
                    continue
            
    


def setBEGINLIBPATH():
    '''The OS/2 EMX port has optional extension modules that do double duty
    as DLLs (and must use the .DLL file extension) for other extensions.
    The library search path needs to be amended so these will be found
    during module import.  Use BEGINLIBPATH so that these are at the start
    of the library search path.

    '''
    dllpath = os.path.join(sys.prefix, 'Lib', 'lib-dynload')
    libpath = os.environ['BEGINLIBPATH'].split(';')
    if libpath[-1]:
        libpath.append(dllpath)
    else:
        libpath[-1] = dllpath
    os.environ['BEGINLIBPATH'] = ';'.join(libpath)


def setquit():
    """Define new built-ins 'quit' and 'exit'.
    These are simply strings that display a hint on how to exit.

    """
    if os.sep == ':':
        exit = 'Use Cmd-Q to quit.'
    elif os.sep == '\\':
        exit = 'Use Ctrl-Z plus Return to exit.'
    else:
        exit = 'Use Ctrl-D (i.e. EOF) to exit.'
    __builtin__.quit = __builtin__.exit = exit


class _Printer(object):
    '''interactive prompt objects for printing the license text, a list of
    contributors and the copyright notice.'''
    MAXLINES = 23
    
    def __init__(self, name, data, files = (), dirs = ()):
        self._Printer__name = name
        self._Printer__data = data
        self._Printer__files = files
        self._Printer__dirs = dirs
        self._Printer__lines = None

    
    def _Printer__setup(self):
        if self._Printer__lines:
            return None
        
        data = None
        for dir in self._Printer__dirs:
            for filename in self._Printer__files:
                filename = os.path.join(dir, filename)
                
                try:
                    fp = file(filename, 'rU')
                    data = fp.read()
                    fp.close()
                continue
                except IOError:
                    continue
                

            
            if data:
                break
                continue
            None<EXCEPTION MATCH>IOError
        
        if not data:
            data = self._Printer__data
        
        self._Printer__lines = data.split('\n')
        self._Printer__linecnt = len(self._Printer__lines)

    
    def __repr__(self):
        self._Printer__setup()
        if len(self._Printer__lines) <= self.MAXLINES:
            return '\n'.join(self._Printer__lines)
        else:
            return 'Type %s() to see the full %s text' % (self._Printer__name,) * 2

    
    def __call__(self):
        self._Printer__setup()
        prompt = 'Hit Return for more, or q (and Return) to quit: '
        lineno = 0
        while None:
            
            try:
                for i in range(lineno, lineno + self.MAXLINES):
                    print self._Printer__lines[i]
            except IndexError:
                break
                continue

            lineno += self.MAXLINES
            key = None
            while key is None:
                key = raw_input(prompt)
                if key not in ('', 'q'):
                    key = None
                    continue
            if key == 'q':
                break
                continue



def setcopyright():
    """Set 'copyright' and 'credits' in __builtin__"""
    __builtin__.copyright = _Printer('copyright', sys.copyright)
    if sys.platform[:4] == 'java':
        __builtin__.credits = _Printer('credits', 'Jython is maintained by the Jython developers (www.jython.org).')
    else:
        __builtin__.credits = _Printer('credits', '    Thanks to CWI, CNRI, BeOpen.com, Zope Corporation and a cast of thousands\n    for supporting Python development.  See www.python.org for more information.')
    here = os.path.dirname(os.__file__)
    __builtin__.license = _Printer('license', 'See http://www.python.org/%.3s/license.html' % sys.version, [
        'LICENSE.txt',
        'LICENSE'], [
        os.path.join(here, os.pardir),
        here,
        os.curdir])


class _Helper(object):
    """Define the built-in 'help'.
    This is a wrapper around pydoc.help (with a twist).

    """
    
    def __repr__(self):
        return 'Type help() for interactive help, or help(object) for help about object.'

    
    def __call__(self, *args, **kwds):
        import pydoc
        return pydoc.help(*args, **kwds)



def sethelper():
    __builtin__.help = _Helper()


def aliasmbcs():
    '''On Windows, some default encodings are not provided by Python,
    while they are always available as "mbcs" in each locale. Make
    them usable by aliasing to "mbcs" in such a case.'''
    if sys.platform == 'win32':
        import locale
        import codecs
        enc = locale.getdefaultlocale()[1]
        if enc.startswith('cp'):
            
            try:
                codecs.lookup(enc)
            except LookupError:
                import encodings
                encodings._cache[enc] = encodings._unknown
                encodings.aliases.aliases[enc] = 'mbcs'
            except:
                None<EXCEPTION MATCH>LookupError
            

        None<EXCEPTION MATCH>LookupError
    


def setencoding():
    """Set the string encoding used by the Unicode implementation.  The
    default is 'ascii', but if you're willing to experiment, you can
    change this."""
    encoding = 'ascii'
    if encoding != 'ascii':
        sys.setdefaultencoding(encoding)
    


def execsitecustomize():
    '''Run custom site specific code, if available.'''
    
    try:
        import sitecustomize
    except ImportError:
        pass



def main():
    abs__file__()
    paths_in_sys = removeduppaths()
    if os.name == 'posix' and sys.path and os.path.basename(sys.path[-1]) == 'Modules':
        addbuilddir()
    
    paths_in_sys = addsitepackages(paths_in_sys)
    if sys.platform == 'os2emx':
        setBEGINLIBPATH()
    
    setquit()
    setcopyright()
    sethelper()
    aliasmbcs()
    setencoding()
    execsitecustomize()
    if hasattr(sys, 'setdefaultencoding'):
        del sys.setdefaultencoding
    
    
    try:
        import apport_python_hook
    except ImportError:
        pass

    apport_python_hook.install()

main()

def _test():
    print 'sys.path = ['
    for dir in sys.path:
        print '    %r,' % (dir,)
    
    print ']'

if __name__ == '__main__':
    _test()

